@@ -33,46 +33,58 @@
  */
 /* copyright --> */
 #include "AbstractCommand.h"
+#include "SegmentMan.h"
+#include "NameResolver.h"
+#include "CUIDCounter.h"
 #include "DlAbortEx.h"
 #include "DlRetryEx.h"
+#include "DownloadFailureException.h"
 #include "InitiateConnectionCommandFactory.h"
 #include "Util.h"
 #include "message.h"
 #include "SleepCommand.h"
 #include "prefs.h"
 #include "DNSCache.h"
-#include "FatalException.h"
+#include "StreamCheckIntegrityEntry.h"
+#include "PeerStat.h"
+#include "Segment.h"
+#include "Option.h"
+#include "PieceStorage.h"
 
-AbstractCommand::AbstractCommand(int cuid,
+AbstractCommand::AbstractCommand(int32_t cuid,
 				 const RequestHandle& req,
 				 RequestGroup* requestGroup,
 				 DownloadEngine* e,
 				 const SocketHandle& s):
-  Command(cuid), req(req), _requestGroup(requestGroup), e(e), socket(s),
+  Command(cuid), RequestGroupAware(requestGroup),
+  req(req), e(e), socket(s),
   checkSocketIsReadable(false), checkSocketIsWritable(false),
-  nameResolverCheck(false) {
-  
+  nameResolverCheck(false)
+{ 
   setReadCheckSocket(socket);
   timeout = this->e->option->getAsInt(PREF_TIMEOUT);
-  ++_requestGroup->numConnection;
+  _requestGroup->increaseStreamConnection();
 }
 
 AbstractCommand::~AbstractCommand() {
   disableReadCheckSocket();
   disableWriteCheckSocket();
-  --_requestGroup->numConnection;
+  _requestGroup->decreaseStreamConnection();
 }
 
 bool AbstractCommand::execute() {
   try {
-    if(_requestGroup->getSegmentMan()->finished()) {
-      logger->debug("CUID#%d - finished.", cuid);
+    if(_requestGroup->downloadFinished() || _requestGroup->isHaltRequested()) {
+      //logger->debug("CUID#%d - finished.", cuid);
       return true;
     }
-    PeerStatHandle peerStat = _requestGroup->getSegmentMan()->getPeerStat(cuid);
-    if(peerStat.get()) {
+    PeerStatHandle peerStat = 0;
+    if(!_requestGroup->getSegmentMan().isNull()) {
+      peerStat = _requestGroup->getSegmentMan()->getPeerStat(cuid);
+    }
+    if(!peerStat.isNull()) {
       if(peerStat->getStatus() == PeerStat::REQUEST_IDLE) {
-	logger->info("CUID#%d - Request idle.", cuid);
+	logger->info(MSG_ABORT_REQUESTED, cuid);
 	onAbort(0);
 	req->resetUrl();
 	tryReserved();
@@ -86,38 +98,39 @@ bool AbstractCommand::execute() {
 #endif // ENABLE_ASYNC_DNS
        !checkSocketIsReadable && !checkSocketIsWritable && !nameResolverCheck) {
       checkPoint.reset();
-      if(_requestGroup->getSegmentMan()->downloadStarted) {
-	// TODO Segment::isNull(), Change method name, it is very confusing.
-	if(segment->isNull()) {
-	  segment = _requestGroup->getSegmentMan()->getSegment(cuid);
+      if(!_requestGroup->getPieceStorage().isNull()) {
+	_segments = _requestGroup->getSegmentMan()->getInFlightSegment(cuid);
+	int32_t maxSegments;
+	if(req->isKeepAlive() && e->option->get(PREF_ENABLE_HTTP_PIPELINING) == V_TRUE) {
+	  maxSegments = e->option->getAsInt(PREF_MAX_HTTP_PIPELINING);
+	} else {
+	  maxSegments = 1;
+	}
+	while((int32_t)_segments.size() < maxSegments) {
+	  SegmentHandle segment = _requestGroup->getSegmentMan()->getSegment(cuid);
 	  if(segment.isNull()) {
-	    logger->info(MSG_NO_SEGMENT_AVAILABLE, cuid);
-	    return prepareForRetry(1);
+	    break;
 	  }
+	  _segments.push_back(segment);
+	}
+	if(_segments.empty()) {
+	  logger->info(MSG_NO_SEGMENT_AVAILABLE, cuid);
+	  return prepareForRetry(1);
 	}
       }
       return executeInternal();
     } else {
-
       if(checkPoint.elapsed(timeout)) {
 	throw new DlRetryEx(EX_TIME_OUT);
       }
       e->commands.push_back(this);
       return false;
     }
-  } catch(FatalException* err) {
-    logger->error(MSG_DOWNLOAD_ABORTED, err, cuid, req->getUrl().c_str());
-    onAbort(err);
-    delete(err);
-    req->resetUrl();
-    _requestGroup->getSegmentMan()->errors++;
-    return true;    
   } catch(DlAbortEx* err) {
     logger->error(MSG_DOWNLOAD_ABORTED, err, cuid, req->getUrl().c_str());
     onAbort(err);
     delete(err);
     req->resetUrl();
-    _requestGroup->getSegmentMan()->errors++;
     tryReserved();
     return true;
   } catch(DlRetryEx* err) {
@@ -132,13 +145,17 @@ bool AbstractCommand::execute() {
       logger->info(MSG_MAX_TRY, cuid, req->getTryCount());
       logger->error(MSG_DOWNLOAD_ABORTED, err, cuid, req->getUrl().c_str());
       delete(err);
-      _requestGroup->getSegmentMan()->errors++;
       tryReserved();
       return true;
     } else {
       delete(err);
       return prepareForRetry(e->option->getAsInt(PREF_RETRY_WAIT));
     }
+  } catch(DownloadFailureException* err) {
+    logger->error(EX_EXCEPTION_CAUGHT, err);
+    delete(err);
+    _requestGroup->setHaltRequested(true);
+    return true;
   }
 }
 
@@ -147,8 +164,10 @@ void AbstractCommand::tryReserved() {
   e->addCommand(commands);
 }
 
-bool AbstractCommand::prepareForRetry(int wait) {
-  _requestGroup->getSegmentMan()->cancelSegment(cuid);
+bool AbstractCommand::prepareForRetry(int32_t wait) {
+  if(!_requestGroup->getPieceStorage().isNull()) {
+    _requestGroup->getSegmentMan()->cancelSegment(cuid);
+  }
   Command* command = InitiateConnectionCommandFactory::createInitiateConnectionCommand(cuid, req, _requestGroup, e);
   if(wait == 0) {
     e->commands.push_back(command);
@@ -162,7 +181,9 @@ bool AbstractCommand::prepareForRetry(int wait) {
 void AbstractCommand::onAbort(Exception* ex) {
   logger->debug(MSG_UNREGISTER_CUID, cuid);
   //_segmentMan->unregisterId(cuid);
-  _requestGroup->getSegmentMan()->cancelSegment(cuid);
+  if(!_requestGroup->getPieceStorage().isNull()) {
+    _requestGroup->getSegmentMan()->cancelSegment(cuid);
+  }
 }
 
 void AbstractCommand::disableReadCheckSocket() {
@@ -224,33 +245,33 @@ bool AbstractCommand::resolveHostname(const string& hostname,
 #ifdef ENABLE_ASYNC_DNS
     switch(resolver->getStatus()) {
     case NameResolver::STATUS_READY:
-      logger->info("CUID#%d - Resolving hostname %s", cuid, hostname.c_str());
+      logger->info(MSG_RESOLVING_HOSTNAME, cuid, hostname.c_str());
       resolver->resolve(hostname);
       setNameResolverCheck(resolver);
       return false;
     case NameResolver::STATUS_SUCCESS:
-      logger->info("CUID#%d - Name resolution complete: %s -> %s", cuid,
+      logger->info(MSG_NAME_RESOLUTION_COMPLETE, cuid,
 		   hostname.c_str(), resolver->getAddrString().c_str());
       DNSCacheSingletonHolder::instance()->put(hostname, resolver->getAddrString());
       return true;
       break;
     case NameResolver::STATUS_ERROR:
-      throw new DlAbortEx("CUID#%d - Name resolution for %s failed:%s", cuid,
+      throw new DlAbortEx(MSG_NAME_RESOLUTION_FAILED, cuid,
 			  hostname.c_str(),
 			  resolver->getError().c_str());
     default:
       return false;
     }
 #else
-    logger->info("CUID#%d - Resolving hostname %s", cuid, hostname.c_str());
+    logger->info(MSG_RESOLVING_HOSTNAME, cuid, hostname.c_str());
     resolver->resolve(hostname);
-    logger->info("CUID#%d - Name resolution complete: %s -> %s", cuid,
+    logger->info(MSG_NAME_RESOLUTION_COMPLETE, cuid,
 		 hostname.c_str(), resolver->getAddrString().c_str());
     DNSCacheSingletonHolder::instance()->put(hostname, resolver->getAddrString());
     return true;
 #endif // ENABLE_ASYNC_DNS
   } else {
-    logger->info("CUID#%d - DNS cache hit: %s -> %s", cuid,
+    logger->info(MSG_DNS_CACHE_HIT, cuid,
 		 hostname.c_str(), ipaddr.c_str());
     resolver->setAddr(ipaddr);
     return true;
@@ -272,3 +293,10 @@ bool AbstractCommand::nameResolveFinished() const {
   return false;
 }
 #endif // ENABLE_ASYNC_DNS
+
+void AbstractCommand::prepareForNextAction(Command* nextCommand)
+{
+  CheckIntegrityEntryHandle entry =
+    new StreamCheckIntegrityEntry(req, _requestGroup, nextCommand);
+  e->addCommand(_requestGroup->processCheckIntegrityEntry(entry, e));
+}
