@@ -36,26 +36,33 @@
 #define _D_ABSTRACT_COMMAND_H_
 
 #include "Command.h"
-#include "Request.h"
-#include "DownloadEngine.h"
-#include "SegmentMan.h"
 #include "TimeA2.h"
-#include "RecoverableException.h"
-#include "RequestGroup.h"
+#include "RequestGroupAware.h"
+#include "Socket.h"
 
-class AbstractCommand : public Command {
+class Request;
+typedef SharedHandle<Request> RequestHandle;
+class DownloadEngine;
+class Segment;
+typedef SharedHandle<Segment> SegmentHandle;
+typedef deque<SegmentHandle> Segments;
+class NameResolver;
+typedef SharedHandle<NameResolver> NameResolverHandle;
+class BtProgressInfoFile;
+typedef SharedHandle<BtProgressInfoFile> BtProgressInfoFileHandle;
+
+class AbstractCommand : public Command, public RequestGroupAware {
 private:
   Time checkPoint;
-  int timeout;
+  int32_t timeout;
 protected:
   RequestHandle req;
-  RequestGroup* _requestGroup;
   DownloadEngine* e;
   SocketHandle socket;
-  SegmentHandle segment;
+  Segments _segments;
 
   void tryReserved();
-  virtual bool prepareForRetry(int wait);
+  virtual bool prepareForRetry(int32_t wait);
   virtual void onAbort(Exception* ex);
   virtual bool executeInternal() = 0;
 
@@ -69,15 +76,19 @@ protected:
   void disableNameResolverCheck(const NameResolverHandle& resolver);
   virtual bool nameResolveFinished() const;
 #endif // ENABLE_ASYNC_DNS
-  void setTimeout(int timeout) { this->timeout = timeout; }
+  void setTimeout(int32_t timeout) { this->timeout = timeout; }
+
+  void prepareForNextAction(Command* nextCommand = 0);
+
 private:
   bool checkSocketIsReadable;
   bool checkSocketIsWritable;
   SocketHandle readCheckTarget;
   SocketHandle writeCheckTarget;
   bool nameResolverCheck;
+
 public:
-  AbstractCommand(int cuid, const RequestHandle& req, RequestGroup* requestGroup, DownloadEngine* e, const SocketHandle& s = SocketHandle());
+  AbstractCommand(int32_t cuid, const RequestHandle& req, RequestGroup* requestGroup, DownloadEngine* e, const SocketHandle& s = SocketHandle());
   virtual ~AbstractCommand();
   bool execute();
 };
