@@ -33,11 +33,12 @@
  */
 /* copyright --> */
 #include "AbstractDiskWriter.h"
-#include "DlAbortEx.h"
 #include "File.h"
 #include "Util.h"
 #include "message.h"
 #include "LogFactory.h"
+#include "a2io.h"
+#include "DlAbortEx.h"
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -46,116 +47,93 @@
 
 AbstractDiskWriter::AbstractDiskWriter():
   fd(-1),
-  fileAllocator(0),
-  glowFileAllocator(0),
-  logger(LogFactory::getInstance())
-{}
+  logger(LogFactory::getInstance()) {}
 
 AbstractDiskWriter::~AbstractDiskWriter()
 {
   closeFile();
 }
 
-void AbstractDiskWriter::openFile(const string& filename, int64_t totalLength) {
+void AbstractDiskWriter::openFile(const string& filename, int64_t totalLength)
+{
   File f(filename);
   if(f.exists()) {
-    openExistingFile(filename);
+    openExistingFile(filename, totalLength);
   } else {
     initAndOpenFile(filename, totalLength);
   }
 }
 
-void AbstractDiskWriter::closeFile() {
+void AbstractDiskWriter::closeFile()
+{
   if(fd >= 0) {
     close(fd);
     fd = -1;
   }
 }
 
-void AbstractDiskWriter::openExistingFile(const string& filename, int64_t totalLength) {
+void AbstractDiskWriter::openExistingFile(const string& filename,
+					  int64_t totalLength)
+{
   this->filename = filename;
   File f(filename);
   if(!f.isFile()) {
-    throw new DlAbortEx(EX_FILE_OPEN, filename.c_str(), "file not found");
+    throw new DlAbortEx(EX_FILE_OPEN, filename.c_str(), MSG_FILE_NOT_FOUND);
   }
 
-  if((fd = open(filename.c_str(), O_RDWR, OPEN_MODE)) < 0) {
+  if((fd = open(filename.c_str(), O_RDWR|O_BINARY, OPEN_MODE)) < 0) {
     throw new DlAbortEx(EX_FILE_OPEN, filename.c_str(), strerror(errno));
   }
-  if(f.size() < totalLength) {
-    if(!fileAllocator.isNull()) {
-      logger->notice("Allocating file %s, %s bytes",
-		     filename.c_str(),
-		     Util::ullitos(totalLength).c_str());
-      glowFileAllocator->allocate(fd, totalLength);
-    }
-  }
 }
 
-void AbstractDiskWriter::createFile(const string& filename, int32_t addFlags) {
+void AbstractDiskWriter::createFile(const string& filename, int32_t addFlags)
+{
   this->filename = filename;
-  // TODO proper filename handling needed
   assert(filename.size());
-//   if(filename.empty()) {
-//     filename = "index.html";
-//   }
-  if((fd = open(filename.c_str(), O_CREAT|O_RDWR|O_TRUNC|addFlags, OPEN_MODE)) < 0) {
+  Util::mkdirs(File(filename).getDirname());
+  if((fd = open(filename.c_str(), O_CREAT|O_RDWR|O_TRUNC|O_BINARY|addFlags, OPEN_MODE)) < 0) {
     throw new DlAbortEx(EX_FILE_OPEN, filename.c_str(), strerror(errno));
   }  
 }
 
-int32_t AbstractDiskWriter::writeDataInternal(const char* data, int32_t len) {
-  return write(fd, data, len);
-}
-
-int32_t AbstractDiskWriter::readDataInternal(char* data, int32_t len) {
-  return read(fd, data, len);
-}
-
-#ifdef ENABLE_MESSAGE_DIGEST
-string AbstractDiskWriter::messageDigest(int64_t offset, int64_t length,
-					 const MessageDigestContext::DigestAlgo& algo)
+int32_t AbstractDiskWriter::writeDataInternal(const unsigned char* data, int32_t len)
 {
-  MessageDigestContext ctx(algo);
-  ctx.digestInit();
-
-  int32_t BUFSIZE = 16*1024;
-  char buf[BUFSIZE];
-  for(int64_t i = 0; i < length/BUFSIZE; i++) {
-    if(BUFSIZE != readData(buf, BUFSIZE, offset)) {
-      throw new DlAbortEx(EX_FILE_SHA1SUM, filename.c_str(), strerror(errno));
+  int32_t writtenLength = 0;
+  while(writtenLength < len) {
+    int32_t ret = 0;
+    while((ret = write(fd, data+writtenLength, len-writtenLength)) == -1 && errno == EINTR);
+    if(ret == -1) {
+      return -1;
     }
-    ctx.digestUpdate(buf, BUFSIZE);
-    offset += BUFSIZE;
+    writtenLength += ret;
   }
-  int32_t r = length%BUFSIZE;
-  if(r > 0) {
-    if(r != readData(buf, r, offset)) {
-      throw new DlAbortEx(EX_FILE_SHA1SUM, filename.c_str(), strerror(errno));
-    }
-    ctx.digestUpdate(buf, r);
-  }
-  unsigned char hashValue[20];
-  ctx.digestFinal(hashValue);
-
-  return Util::toHex(hashValue, 20);
+  return writtenLength;
 }
-#endif // ENABLE_MESSAGE_DIGEST
 
-void AbstractDiskWriter::seek(int64_t offset) {
+int32_t AbstractDiskWriter::readDataInternal(unsigned char* data, int32_t len)
+{
+  int32_t ret = 0;
+  while((ret = read(fd, data, len)) == -1 && errno == EINTR);
+  return ret;
+}
+
+void AbstractDiskWriter::seek(int64_t offset)
+{
   if(offset != lseek(fd, offset, SEEK_SET)) {
     throw new DlAbortEx(EX_FILE_SEEK, filename.c_str(), strerror(errno));
   }
 }
 
-void AbstractDiskWriter::writeData(const char* data, int32_t len, int64_t offset) {
+void AbstractDiskWriter::writeData(const unsigned char* data, int32_t len, int64_t offset)
+{
   seek(offset);
   if(writeDataInternal(data, len) < 0) {
     throw new DlAbortEx(EX_FILE_WRITE, filename.c_str(), strerror(errno));
   }
 }
 
-int32_t AbstractDiskWriter::readData(char* data, int32_t len, int64_t offset) {
+int32_t AbstractDiskWriter::readData(unsigned char* data, int32_t len, int64_t offset)
+{
   int32_t ret;
   seek(offset);
   if((ret = readDataInternal(data, len)) < 0) {
@@ -166,14 +144,41 @@ int32_t AbstractDiskWriter::readData(char* data, int32_t len, int64_t offset) {
 
 void AbstractDiskWriter::truncate(int64_t length)
 {
+  if(fd == -1) {
+    throw new DlAbortEx("File not opened.");
+  }
   ftruncate(fd, length);
 }
 
+// TODO the file descriptor fd must be opened before calling this function.
 int64_t AbstractDiskWriter::size() const
 {
+  if(fd == -1) {
+    throw new DlAbortEx("File not opened.");
+  }
   struct stat fileStat;
   if(fstat(fd, &fileStat) < 0) {
     return 0;
   }
   return fileStat.st_size;
 }
+
+void AbstractDiskWriter::enableDirectIO()
+{
+#ifdef ENABLE_DIRECT_IO
+  if(_directIOAllowed) {
+    int32_t flg;
+    while((flg = fcntl(fd, F_GETFL)) == -1 && errno == EINTR);
+    while(fcntl(fd, F_SETFL, flg|O_DIRECT) == -1 && errno == EINTR);
+  }
+#endif // ENABLE_DIRECT_IO
+}
+
+void AbstractDiskWriter::disableDirectIO()
+{
+#ifdef ENABLE_DIRECT_IO
+  int32_t flg;
+  while((flg = fcntl(fd, F_GETFL)) == -1 && errno == EINTR);
+  while(fcntl(fd, F_SETFL, flg&(~O_DIRECT)) == -1 && errno == EINTR);
+#endif // ENABLE_DIRECT_IO
+}
