@@ -36,7 +36,9 @@
 #define _D_ABSTRACT_PROXY_REQUEST_COMMAND_H_
 
 #include "AbstractCommand.h"
-#include "HttpConnection.h"
+
+class HttpConnection;
+typedef SharedHandle<HttpConnection> HttpConnectionHandle;
 
 class AbstractProxyRequestCommand : public AbstractCommand {
 protected:
