@@ -34,36 +34,59 @@
 /* copyright --> */
 #include "AbstractSingleDiskAdaptor.h"
 #include "File.h"
+#include "SingleFileAllocationIterator.h"
 
-void AbstractSingleDiskAdaptor::initAndOpenFile() {
+void AbstractSingleDiskAdaptor::initAndOpenFile()
+{
   diskWriter->initAndOpenFile(getFilePath(), totalLength);
 }
 
-void AbstractSingleDiskAdaptor::openFile() {
+void AbstractSingleDiskAdaptor::openFile()
+{
   diskWriter->openFile(getFilePath(), totalLength);
 }
 
-void AbstractSingleDiskAdaptor::closeFile() {
+void AbstractSingleDiskAdaptor::closeFile()
+{
   diskWriter->closeFile();
 }
 
-void AbstractSingleDiskAdaptor::openExistingFile() {
+void AbstractSingleDiskAdaptor::openExistingFile()
+{
   diskWriter->openExistingFile(getFilePath(), totalLength);
 }
 
-void AbstractSingleDiskAdaptor::writeData(const unsigned char* data, int32_t len, int64_t offset) {
+void AbstractSingleDiskAdaptor::writeData(const unsigned char* data, int32_t len, int64_t offset)
+{
   diskWriter->writeData(data, len, offset);
 }
 
-int32_t AbstractSingleDiskAdaptor::readData(unsigned char* data, int32_t len, int64_t offset) {
+int32_t AbstractSingleDiskAdaptor::readData(unsigned char* data, int32_t len, int64_t offset)
+{
   return diskWriter->readData(data, len, offset);
 }
 
-string AbstractSingleDiskAdaptor::messageDigest(int64_t offset, int64_t length, const MessageDigestContext::DigestAlgo& algo) {
-  return diskWriter->messageDigest(offset, length, algo);
-}
-
 bool AbstractSingleDiskAdaptor::fileExists()
 {
   return File(getFilePath()).exists();
 }
+
+FileAllocationIteratorHandle AbstractSingleDiskAdaptor::fileAllocationIterator()
+{
+  SingleFileAllocationIteratorHandle h =
+    new SingleFileAllocationIterator(this,
+				     size(),
+				     totalLength);
+  h->init();
+  return h;
+}
+
+void AbstractSingleDiskAdaptor::enableDirectIO()
+{
+  diskWriter->enableDirectIO();
+}
+
+void AbstractSingleDiskAdaptor::disableDirectIO()
+{
+  diskWriter->disableDirectIO();
+}
