@@ -135,9 +135,12 @@ int32_t
 BitfieldMan::getMissingIndexRandomly(const unsigned char* bitfield,
 				     int32_t bitfieldLength) const
 {
+  /*
   int32_t byte = (int32_t)(((double)bitfieldLength)*
 			   randomizer->getRandomNumber()/
 			   (randomizer->getMaxRandomNumber()+1.0));
+  */
+  int32_t byte = randomizer->getRandomNumber(bitfieldLength);
 
   unsigned char lastMask = 0;
   // the number of bytes in the last byte of bitfield
@@ -333,8 +336,10 @@ int32_t BitfieldMan::getSparseMissingUnusedIndex() const {
   if(maxRange.getSize()) {
     if(maxRange.startIndex == 0) {
       return 0;
+    } else if(isUseBitSet(maxRange.startIndex-1)) {
+      return maxRange.getMidIndex();      
     } else {
-      return maxRange.getMidIndex();
+      return maxRange.startIndex;
     }
   } else {
     return -1;
@@ -458,6 +463,9 @@ bool BitfieldMan::isFilteredAllBitSet() const {
 }
 
 bool BitfieldMan::isAllBitSet() const {
+  if(bitfieldLength == 0) {
+    return true;
+  }
   for(int32_t i = 0; i < bitfieldLength-1; ++i) {
     if(bitfield[i] != 0xff) {
       return false;
