@@ -36,33 +36,43 @@
 #define _D_BITFIELD_MAN_H_
 
 #include "common.h"
+#include "Randomizer.h"
 #include <deque>
 
 typedef deque<int> BlockIndexes;
 
 class BitfieldMan {
 private:
-  int blockLength;
-  long long int totalLength;
+  int32_t blockLength;
+  int64_t totalLength;
   unsigned char* bitfield;
   unsigned char* useBitfield;
   unsigned char* filterBitfield;
-  int bitfieldLength;
-  int blocks;
+  int32_t bitfieldLength;
+  int32_t blocks;
   bool filterEnabled;
-  int countSetBit(const unsigned char* bitfield, int len) const;
-  int getNthBitIndex(const unsigned char bit, int nth) const;
-  int getMissingIndexRandomly(const unsigned char* bitfield, int len) const;
-  bool isBitSetInternal(const unsigned char* bitfield, int index) const;
-  bool setBitInternal(unsigned char* bitfield, int index, bool on);
-  bool setFilterBit(int index);
+  RandomizerHandle randomizer;
 
-  int getStartIndex(int index) const;
-  int getEndIndex(int index) const;
+  // for caching
+  int32_t cachedNumMissingBlock;
+  int32_t cachedNumFilteredBlock;
+  int64_t cachedCompletedLength;
+  int64_t cachedFilteredComletedLength;
+  int64_t cachedFilteredTotalLength;
 
-  long long int getCompletedLength(bool useFilter) const;
+  int32_t countSetBit(const unsigned char* bitfield, int32_t len) const;
+  int32_t getNthBitIndex(const unsigned char bit, int32_t nth) const;
+  int32_t getMissingIndexRandomly(const unsigned char* bitfield, int32_t len) const;
+  bool isBitSetInternal(const unsigned char* bitfield, int32_t index) const;
+  bool setBitInternal(unsigned char* bitfield, int32_t index, bool on);
+  bool setFilterBit(int32_t index);
+
+  int32_t getStartIndex(int32_t index) const;
+  int32_t getEndIndex(int32_t index) const;
+
+  int64_t getCompletedLength(bool useFilter) const;
 public:
-  BitfieldMan(int blockLength, long long int totalLength);
+  BitfieldMan(int32_t blockLength, int64_t totalLength);
   BitfieldMan(const BitfieldMan& bitfieldMan);
   ~BitfieldMan();
 
@@ -70,78 +80,81 @@ public:
     if(this != &bitfieldMan) {
       blockLength = bitfieldMan.blockLength;
       totalLength = bitfieldMan.totalLength;
-      if(bitfieldLength != bitfieldMan.bitfieldLength) {
-	delete [] bitfield;
-	delete [] useBitfield;
-	bitfield = new unsigned char[bitfieldMan.bitfieldLength];
-	useBitfield = new unsigned char[bitfieldMan.bitfieldLength];
-      }
       blocks = bitfieldMan.blocks;
       bitfieldLength = bitfieldMan.bitfieldLength;
-      memcpy(bitfield, bitfieldMan.bitfield, bitfieldLength);
-      memcpy(useBitfield, bitfieldMan.useBitfield, bitfieldLength);
       filterEnabled = bitfieldMan.filterEnabled;
-      if(bitfieldLength != bitfieldMan.bitfieldLength) {
-	delete [] filterBitfield;
+
+      delete [] bitfield;
+      bitfield = new unsigned char[bitfieldLength];
+      memcpy(bitfield, bitfieldMan.bitfield, bitfieldLength);
+
+      delete [] useBitfield;
+      useBitfield = new unsigned char[bitfieldLength];
+      memcpy(useBitfield, bitfieldMan.useBitfield, bitfieldLength);
+
+      delete [] filterBitfield;
+      if(filterEnabled) {
+	filterBitfield = new unsigned char[bitfieldLength];
+	memcpy(filterBitfield, bitfieldMan.filterBitfield, bitfieldLength);
+      } else {
 	filterBitfield = 0;
       }
-      if(bitfieldMan.filterBitfield) {
-	if(!filterBitfield) {
-	  filterBitfield = new unsigned char[bitfieldLength];
-	}
-	memcpy(filterBitfield, bitfieldMan.filterBitfield, bitfieldLength);
-      }
+
+      updateCache();
     }
     return *this;
   }
 
-  int getBlockLength() const { return blockLength; }
-  int getLastBlockLength() const {
+  int32_t getBlockLength() const { return blockLength; }
+
+  int32_t getLastBlockLength() const {
     return totalLength-blockLength*(blocks-1);
   }
-  int getBlockLength(int index) const {
-    if(index == blocks-1) {
+
+  int32_t getBlockLength(int32_t index) const {
+    if(index == (int32_t)(blocks-1)) {
       return getLastBlockLength();
-    } else if(0 <= index && index < blocks-1) {
+    } else if(0 <= index && index < (int32_t)(blocks-1)) {
       return getBlockLength();
     } else {
       return 0;
     }
   }
-  long long int getTotalLength() const { return totalLength; }
+
+  int64_t getTotalLength() const { return totalLength; }
 
   /**
    * affected by filter
    */
-  bool hasMissingPiece(const unsigned char* bitfield, int len) const;
+  bool hasMissingPiece(const unsigned char* bitfield, int32_t len) const;
   /**
    * affected by filter
    */
-  int getMissingIndex(const unsigned char* bitfield, int len) const;
+  int32_t getMissingIndex(const unsigned char* bitfield, int32_t len) const;
   /**
    * affected by filter
    */
-  int getMissingIndex() const;
+  int32_t getMissingIndex() const;
   /**
    * affected by filter
    */
-  int getFirstMissingUnusedIndex(const unsigned char* bitfield, int len) const;
+  int32_t getFirstMissingUnusedIndex(const unsigned char* bitfield, int32_t len) const;
   /**
    * affected by filter
    */
-  int getFirstMissingUnusedIndex() const;
+  int32_t getFirstMissingUnusedIndex() const;
   /**
    * affected by filter
    */
-  int getMissingUnusedIndex(const unsigned char* bitfield, int len) const;
+  int32_t getMissingUnusedIndex(const unsigned char* bitfield, int32_t len) const;
   /**
    * affected by filter
    */
-  int getMissingUnusedIndex() const;
+  int32_t getMissingUnusedIndex() const;
   /**
    * affected by filter
    */
-  int getSparseMissingUnusedIndex() const;
+  int32_t getSparseMissingUnusedIndex() const;
   /**
    * affected by filter
    */
@@ -149,34 +162,47 @@ public:
   /**
    * affected by filter
    */
-  BlockIndexes getAllMissingIndexes(const unsigned char* bitfield, int len) const;
+  BlockIndexes getAllMissingIndexes(const unsigned char* bitfield, int32_t len) const;
   /**
    * affected by filter
    */
-  int countMissingBlock() const;
-  bool setUseBit(int index);
-  bool unsetUseBit(int index);
+  int32_t countMissingBlock() const;
+  /**
+   * affected by filter
+   */
+  int32_t countMissingBlockNow() const;
 
-  bool setBit(int index);
-  bool unsetBit(int index);
+  bool setUseBit(int32_t index);
+  bool unsetUseBit(int32_t index);
 
-  bool isBitSet(int index) const;
-  bool isUseBitSet(int index) const;
+  bool setBit(int32_t index);
+  bool unsetBit(int32_t index);
+
+  bool isBitSet(int32_t index) const;
+  bool isUseBitSet(int32_t index) const;
 
   /**
    * affected by filter
    */
+  bool isFilteredAllBitSet() const;
+
   bool isAllBitSet() const;
 
   const unsigned char* getBitfield() const { return bitfield; }
-  int getBitfieldLength() const { return bitfieldLength; }
+  int32_t getBitfieldLength() const { return bitfieldLength; }
 
   /**
    * affected by filter
    */
-  int countBlock() const;
+  int32_t countBlock() const;
+  /**
+   * affected by filter
+   */
+  int32_t countFilteredBlockNow() const;
 
-  void setBitfield(const unsigned char* bitfield, int bitfieldLength);
+  int32_t getMaxIndex() const { return blocks-1; }
+
+  void setBitfield(const unsigned char* bitfield, int32_t bitfieldLength);
 
   void clearAllBit();
   void setAllBit();
@@ -184,7 +210,7 @@ public:
   void clearAllUseBit();
   void setAllUseBit();
 
-  void addFilter(long long int offset, long long int length);
+  void addFilter(int64_t offset, int64_t length);
   /**
    * Clears filter and disables filter
    */
@@ -196,12 +222,45 @@ public:
   /**
    * affected by filter
    */
-  long long int getFilteredTotalLength() const;
-  long long int getCompletedLength() const;
+  int64_t getFilteredTotalLength() const;
   /**
    * affected by filter
    */
-  long long int getFilteredCompletedLength() const;
+  int64_t getFilteredTotalLengthNow() const;
+
+  int64_t getCompletedLength() const;
+
+  int64_t getCompletedLengthNow() const;
+
+  /**
+   * affected by filter
+   */
+  int64_t getFilteredCompletedLength() const;
+  /**
+   * affected by filter
+   */
+  int64_t getFilteredCompletedLengthNow() const;
+
+  void setRandomizer(const RandomizerHandle& randomizer) {
+    this->randomizer = randomizer;
+  }
+
+  RandomizerHandle getRandomizer() const {
+    return randomizer;
+  }
+
+  void updateCache();
+
+  bool isBitRangeSet(int32_t startIndex, int32_t endIndex) const;
+
+  void unsetBitRange(int32_t startIndex, int32_t endIndex);
+
+  void setBitRange(int32_t startIndex, int32_t endIndex);
+
+  bool isBitSetOffsetRange(int64_t offset, int64_t length) const;
+
+  int64_t getMissingUnusedLength(int32_t startingIndex) const;
+
 };
 
 #endif // _D_BITFIELD_MAN_H_
