@@ -49,7 +49,7 @@ public:
 
   virtual void removeAllTargetPiece() = 0;
 
-  virtual int countTargetPiece() = 0;
+  virtual int32_t countTargetPiece() = 0;
 
   virtual void removeCompletedPiece() = 0;
 
