@@ -38,36 +38,38 @@
 #include "common.h"
 #include "SharedHandle.h"
 
-#define MIN_PEERS 15
+#define MIN_PEERS 40
 
 class BtRuntime {
 private:
-  long long int uploadLengthAtStartup;
-  int port;
+  int64_t uploadLengthAtStartup;
+  int32_t port;
   bool halt;
-  int connections;
+  int32_t connections;
+  bool _ready;
 public:
   BtRuntime():
     uploadLengthAtStartup(0),
     port(0),
     halt(false),
-    connections(0)
+    connections(0),
+    _ready(false)
     {}
   ~BtRuntime() {}
 
-  long long int getUploadLengthAtStartup() const {
+  int64_t getUploadLengthAtStartup() const {
     return uploadLengthAtStartup;
   }
 
-  void setUploadLengthAtStartup(long long int length) {
+  void setUploadLengthAtStartup(int64_t length) {
     this->uploadLengthAtStartup = length;
   }
 
-  void setListenPort(int port) {
+  void setListenPort(int32_t port) {
     this->port = port;
   }
 
-  int getListenPort() const { return port; }
+  int32_t getListenPort() const { return port; }
 
   bool isHalt() const { return halt; }
 
@@ -75,7 +77,7 @@ public:
     this->halt = halt;
   }
 
-  int getConnections() const { return connections; }
+  int32_t getConnections() const { return connections; }
 
   void increaseConnections() { connections++; }
 
@@ -83,7 +85,11 @@ public:
 
   bool lessThanMinPeer() const { return connections < MIN_PEERS; }
 
-  bool lessThanEqMinPeer() const { return connections <= MIN_PEERS; }  
+  bool lessThanEqMinPeer() const { return connections <= MIN_PEERS; }
+
+  bool ready() { return _ready; }
+
+  void setReady(bool go) { _ready = go; }
 };
 
 typedef SharedHandle<BtRuntime> BtRuntimeHandle;
