@@ -55,9 +55,8 @@ public:
 
   virtual void openExistingFile(const string& filename, int64_t totalLength = 0);
 
-  // position is ignored
-  virtual void writeData(const char* data, int32_t len, int64_t position = 0);
-  virtual int readData(char* data, int32_t len, int64_t position);
+  virtual void writeData(const unsigned char* data, int32_t len, int64_t position);
+  virtual int32_t readData(unsigned char* data, int32_t len, int64_t position);
 
   // Not implemented yet
   virtual void truncate(int64_t length) {}
@@ -67,13 +66,20 @@ public:
     return buf.str().size();
   }
 
-  // not implemented yet
-#ifdef ENABLE_MESSAGE_DIGEST
-  virtual string messageDigest(int64_t offset, int64_t length,
-			       const MessageDigestContext::DigestAlgo& algo) {
-    return "";
+  virtual void enableDirectIO() {}
+
+  virtual void disableDirectIO() {}
+
+  virtual bool directIOAllowed() const
+  {
+    return false;
+  }
+
+  void setString(const string& s)
+  {
+    buf.str(s);
   }
-#endif // ENABLE_MESSAGE_DIGEST
 };
 
+typedef SharedHandle<ByteArrayDiskWriter> ByteArrayDiskWriterHandle;
 #endif // _D_BYTE_ARRAY_DISK_WRITER_H_
