@@ -36,28 +36,37 @@
 #define _D_CHECK_INTEGRITY_ENTRY_H_
 
 #include "RequestGroupEntry.h"
-#include "IteratableChunkChecksumValidator.h"
 
-class CheckIntegrityEntry : public RequestGroupEntry {
-private:
-  IteratableChunkChecksumValidatorHandle _validator;
+class IteratableValidator;
+typedef SharedHandle<IteratableValidator> IteratableValidatorHandle;
+class Command;
+typedef deque<Command*> Commands;
+class DownloadEngine;
+
+class CheckIntegrityEntry : public RequestGroupEntry,
+			    public ProgressAwareEntry {
+protected:
+  IteratableValidatorHandle _validator;
 public:
-  CheckIntegrityEntry(int cuid,
-		      const RequestHandle& currentRequest,
-		      RequestGroup* requestGroup):
-    RequestGroupEntry(cuid, currentRequest, requestGroup),
-    _validator(0)
-  {}
+  CheckIntegrityEntry(RequestGroup* requestGroup, Command* nextCommand = 0);
 
-  virtual ~CheckIntegrityEntry() {}
+  virtual ~CheckIntegrityEntry();
 
-  virtual int64_t getCurrentLength() const;
+  virtual int64_t getTotalLength();
 
-  virtual bool finished() const;
+  virtual int64_t getCurrentLength();
 
-  void initValidator();
+  virtual void validateChunk();
 
-  void validateChunk();
+  virtual bool finished();
+
+  virtual bool isValidationReady() = 0;
+
+  virtual void initValidator() = 0;
+
+  virtual Commands onDownloadFinished(DownloadEngine* e) = 0;
+
+  virtual Commands onDownloadIncomplete(DownloadEngine* e) = 0;
 };
 
 typedef SharedHandle<CheckIntegrityEntry> CheckIntegrityEntryHandle;
