@@ -34,14 +34,17 @@
 /* copyright --> */
 #include "CopyDiskAdaptor.h"
 #include "Util.h"
+#include "message.h"
 
-void CopyDiskAdaptor::onDownloadComplete() {
+void CopyDiskAdaptor::onDownloadComplete()
+{
   closeFile();
   fixFilename();
   openFile();
 }
 
-void CopyDiskAdaptor::fixFilename() {
+void CopyDiskAdaptor::fixFilename()
+{
   int64_t offset = 0;
   for(FileEntries::iterator itr = fileEntries.begin();
       itr != fileEntries.end(); itr++) {
@@ -49,7 +52,7 @@ void CopyDiskAdaptor::fixFilename() {
       string topDirPath = storeDir+"/"+topDir;
       (*itr)->setupDir(topDirPath);
       string destFilePath = topDirPath+"/"+(*itr)->getPath();
-      logger->info("writing file %s", destFilePath.c_str());
+      logger->info(MSG_WRITING_FILE, destFilePath.c_str());
       Util::rangedFileCopy(destFilePath, getFilePath(),
 			   offset, (*itr)->getLength());
       (*itr)->setExtracted(true);
@@ -58,6 +61,7 @@ void CopyDiskAdaptor::fixFilename() {
   }
 }
 
-string CopyDiskAdaptor::getFilePath() {
+string CopyDiskAdaptor::getFilePath()
+{
   return storeDir+"/"+tempFilename;
 }
