@@ -44,6 +44,7 @@
 #include "prefs.h"
 #include "DlAbortEx.h"
 #include "message.h"
+#include "SimpleRandomizer.h"
 
 DefaultBtAnnounce::DefaultBtAnnounce(BtContextHandle btContext,
 				     const Option* option):
@@ -54,26 +55,28 @@ DefaultBtAnnounce::DefaultBtAnnounce(BtContextHandle btContext,
   complete(0),
   incomplete(0),
   announceList(btContext->getAnnounceTiers()),
-  trackerNumTry(0),
   option(option),
+  logger(LogFactory::getInstance()),
+  _randomizer(SimpleRandomizer::getInstance()),
   btRuntime(BT_RUNTIME(btContext)),
   pieceStorage(PIECE_STORAGE(btContext)),
   peerStorage(PEER_STORAGE(btContext))
 {
   prevAnnounceTime.setTimeInSec(0);
-  key = generateKey();
-  logger = LogFactory::getInstance();
+  generateKey();
 }
 
 DefaultBtAnnounce::~DefaultBtAnnounce() {
 }
 
-string DefaultBtAnnounce::generateKey() const {
-  return Util::randomAlpha(8);
+void DefaultBtAnnounce::generateKey()
+{
+  key = Util::randomAlpha(8, _randomizer);
 }
 
 bool DefaultBtAnnounce::isDefaultAnnounceReady() {
-  return (trackers == 0 && prevAnnounceTime.elapsed(minInterval));
+  return (trackers == 0 && prevAnnounceTime.elapsed(minInterval) &&
+	  !announceList.allTiersFailed());
 }
 
 bool DefaultBtAnnounce::isStoppedAnnounceReady() {
@@ -97,10 +100,14 @@ bool DefaultBtAnnounce::isAnnounceReady() {
 
 string DefaultBtAnnounce::getAnnounceUrl() {
   if(isStoppedAnnounceReady()) {
-    announceList.moveToStoppedAllowedTier();
+    if(!announceList.currentTierAcceptsStoppedEvent()) {
+      announceList.moveToStoppedAllowedTier();
+    }
     announceList.setEvent(AnnounceTier::STOPPED);
   } else if(isCompletedAnnounceReady()) {
-    announceList.moveToCompletedAllowedTier();
+    if(!announceList.currentTierAcceptsCompletedEvent()) {
+      announceList.moveToCompletedAllowedTier();
+    }
     announceList.setEvent(AnnounceTier::COMPLETED);
   } else if(isDefaultAnnounceReady()) {
     // If download completed before "started" event is sent to a tracker,
@@ -110,14 +117,16 @@ string DefaultBtAnnounce::getAnnounceUrl() {
        announceList.getEvent() == AnnounceTier::STARTED) {
       announceList.setEvent(AnnounceTier::STARTED_AFTER_COMPLETION);
     }
+  } else {
+    return "";
   }
-  int numWant = 50;
+  int32_t numWant = 50;
   if(!btRuntime->lessThanEqMinPeer() ||
      btRuntime->isHalt()) {
     numWant = 0;
   }
   TransferStat stat = peerStorage->calculateStat();
-  long long int left = pieceStorage->getTotalLength()-pieceStorage->getCompletedLength();
+  int64_t left = pieceStorage->getTotalLength()-pieceStorage->getCompletedLength();
   if(left < 0) {
     left = 0;
   }
@@ -125,7 +134,6 @@ string DefaultBtAnnounce::getAnnounceUrl() {
     "info_hash="+Util::torrentUrlencode(btContext->getInfoHash(),
 					btContext->getInfoHashLength())+"&"+
     "peer_id="+Util::torrentUrlencode(btContext->getPeerId(), 20)+"&"+
-    "port="+Util::itos(btRuntime->getListenPort())+"&"+
     "uploaded="+Util::llitos(stat.getSessionUploadLength())+"&"+
     "downloaded="+Util::llitos(stat.getSessionDownloadLength())+"&"+
     "left="+Util::llitos(left)+"&"+
@@ -133,6 +141,9 @@ string DefaultBtAnnounce::getAnnounceUrl() {
     "key="+key+"&"+
     "numwant="+Util::itos(numWant)+"&"+
     "no_peer_id=1";
+  if(btRuntime->getListenPort() > 0) {
+    url += string("&")+"port="+Util::itos(btRuntime->getListenPort());
+  }
   string event = announceList.getEventString();
   if(!event.empty()) {
     url += string("&")+"event="+event;
@@ -155,30 +166,28 @@ void DefaultBtAnnounce::announceSuccess() {
 
 void DefaultBtAnnounce::announceFailure() {
   trackers = 0;
-  trackerNumTry++;
   announceList.announceFailure();
 }
 
 bool DefaultBtAnnounce::isAllAnnounceFailed() {
-  return 
-    trackerNumTry >= option->getAsInt(PREF_TRACKER_MAX_TRIES);
+  return announceList.allTiersFailed();
 }
 
 void DefaultBtAnnounce::resetAnnounce() {
   prevAnnounceTime.reset();
-  trackerNumTry = 0;
+  announceList.resetTier();
 }
 
 void
 DefaultBtAnnounce::processAnnounceResponse(const char* trackerResponse,
-						  size_t trackerResponseLength)
+					   size_t trackerResponseLength)
 {
   SharedHandle<MetaEntry> entry(MetaFileUtil::bdecoding(trackerResponse,
 							trackerResponseLength));
   Dictionary* response = (Dictionary*)entry.get();
   Data* failureReasonData = (Data*)response->get("failure reason");
   if(failureReasonData) {
-    throw new DlAbortEx("Tracker returned failure reason: %s",
+    throw new DlAbortEx(EX_TRACKER_FAILURE,
 			failureReasonData->toString().c_str());
   }
   Data* warningMessageData = (Data*)response->get("warning message");
@@ -224,7 +233,7 @@ DefaultBtAnnounce::processAnnounceResponse(const char* trackerResponse,
     peerStorage->addPeer(peers);
   }
   if(!peersEntry) {
-    logger->info("No peer list received.");
+    logger->info(MSG_NO_PEER_LIST_RECEIVED);
   }
 }
 
@@ -237,3 +246,8 @@ bool DefaultBtAnnounce::noMoreAnnounce() {
 void DefaultBtAnnounce::shuffleAnnounce() {
   announceList.shuffle();
 }
+
+void DefaultBtAnnounce::setRandomizer(const RandomizerHandle& randomizer)
+{
+  _randomizer = randomizer;
+}
