@@ -45,23 +45,26 @@
 #include "PieceStorage.h"
 #include "PeerStorage.h"
 
+class Randomizer;
+typedef SharedHandle<Randomizer> RandomizerHandle;
+
 #define DEFAULT_ANNOUNCE_INTERVAL 1800
 
 class DefaultBtAnnounce : public BtAnnounce {
 private:
   BtContextHandle btContext;
-  int trackers;
+  int32_t trackers;
   Time prevAnnounceTime;
-  int interval;
-  int minInterval;
-  int complete;
-  int incomplete;
+  int32_t interval;
+  int32_t minInterval;
+  int32_t complete;
+  int32_t incomplete;
   AnnounceList announceList;
   string trackerId;
   string key;
-  int trackerNumTry;
   const Option* option;
   Logger* logger;
+  RandomizerHandle _randomizer;
   BtRuntimeHandle btRuntime;
   PieceStorageHandle pieceStorage;
   PeerStorageHandle peerStorage;
@@ -111,7 +114,9 @@ public:
 
   virtual void shuffleAnnounce();
 
-  string generateKey() const;  
+  void generateKey();
+
+  void setRandomizer(const RandomizerHandle& randomizer);
 };
 
 #endif // _D_DEFAULT_BT_ANNOUNCE_H_
