@@ -40,15 +40,24 @@
 #include "DlAbortEx.h"
 #include "ShaVisitor.h"
 #include "Util.h"
+#include "MessageDigestHelper.h"
+#include "a2netcompat.h"
+#include "AnnounceTier.h"
+#include "SimpleRandomizer.h"
 #include <libgen.h>
 
-DefaultBtContext::DefaultBtContext() {}
+DefaultBtContext::DefaultBtContext():_peerIdPrefix("-aria2-"),
+				     _randomizer(SimpleRandomizer::getInstance()),
+				     _ownerRequestGroup(0) {}
 
 DefaultBtContext::~DefaultBtContext() {}
 
 string DefaultBtContext::generatePeerId() const {
-  string peerId = "-aria2-";
-  peerId += Util::randomAlpha(20-peerId.size());
+  string peerId = _peerIdPrefix;
+  peerId += Util::randomAlpha(20-_peerIdPrefix.size(), _randomizer);
+  if(peerId.size() > 20) {
+    peerId.erase(20);
+  }
   return peerId;
 }
 
@@ -92,7 +101,8 @@ void DefaultBtContext::extractPieceHash(const unsigned char* hashData,
 }
 
 void DefaultBtContext::extractFileEntries(Dictionary* infoDic,
-					  const string& defaultName) {
+					  const string& defaultName,
+					  const Strings& urlList) {
   // TODO use dynamic_cast
   Data* nameData = (Data*)infoDic->get("name");
   if(nameData) {
@@ -128,9 +138,13 @@ void DefaultBtContext::extractFileEntries(Dictionary* infoDic,
       Data* lastPath = (Data*)paths.back();
       path += lastPath->toString();
 
+      Strings uris;
+      transform(urlList.begin(), urlList.end(), back_inserter(uris),
+		bind2nd(plus<string>(), "/"+name+"/"+path));
       FileEntryHandle fileEntry(new FileEntry(path,
 					      lengthData->toLLInt(),
-					      offset));
+					      offset,
+					      uris));
       fileEntries.push_back(fileEntry);
       offset += fileEntry->getLength();
     }
@@ -140,14 +154,14 @@ void DefaultBtContext::extractFileEntries(Dictionary* infoDic,
     fileMode = BtContext::SINGLE;
     Data* length = (Data*)infoDic->get("length");
     totalLength = length->toLLInt();
-    FileEntryHandle fileEntry(new FileEntry(name, totalLength, 0));
+    FileEntryHandle fileEntry(new FileEntry(name, totalLength, 0, urlList));
     fileEntries.push_back(fileEntry);
   }
 }
 
 void DefaultBtContext::extractAnnounce(Data* announceData) {
   Strings urls;
-  urls.push_back(announceData->toString());
+  urls.push_back(Util::trim(announceData->toString()));
   announceTiers.push_back(AnnounceTierHandle(new AnnounceTier(urls)));
 }
 
@@ -159,7 +173,7 @@ void DefaultBtContext::extractAnnounceList(List* announceListData) {
     for(MetaList::const_iterator elemItr = elem->getList().begin();
 	elemItr != elem->getList().end(); elemItr++) {
       const Data* data = (Data*)*elemItr;
-      urls.push_back(data->toString());
+      urls.push_back(Util::trim(data->toString()));
     }
     if(urls.size()) {
       AnnounceTierHandle tier(new AnnounceTier(urls));
@@ -168,19 +182,52 @@ void DefaultBtContext::extractAnnounceList(List* announceListData) {
   }
 }
 
+Strings DefaultBtContext::extractUrlList(const MetaEntry* obj)
+{
+  Strings uris;
+  if(dynamic_cast<const List*>(obj)) {
+    const List* urlList = (const List*)obj;
+    for(MetaList::const_iterator itr = urlList->getList().begin();
+	itr != urlList->getList().end(); ++itr) {
+      Data* data = dynamic_cast<Data*>(*itr);
+      if(data) {
+	uris.push_back(data->toString());
+      }
+    }
+  } else if(dynamic_cast<const Data*>(obj)) {
+    const Data* urlData = (const Data*)obj;
+    uris.push_back(urlData->toString());
+  }
+  return uris;
+}
+
+void DefaultBtContext::loadFromMemory(const char* content, int32_t length, const string& defaultName)
+{
+  MetaEntry* rootEntry = MetaFileUtil::bdecoding(content, length);
+  if(!dynamic_cast<Dictionary*>(rootEntry)) {
+    throw new DlAbortEx("torrent file does not contain a root dictionary .");
+  }
+  processMetaInfo(rootEntry, defaultName);
+}
+
 void DefaultBtContext::load(const string& torrentFile) {
-  clear();
   MetaEntry* rootEntry = MetaFileUtil::parseMetaFile(torrentFile);
   if(!dynamic_cast<Dictionary*>(rootEntry)) {
     throw new DlAbortEx("torrent file does not contain a root dictionary .");
   }
+  processMetaInfo(rootEntry, torrentFile);
+}
+
+void DefaultBtContext::processMetaInfo(const MetaEntry* rootEntry, const string& defaultName)
+{
+  clear();
   SharedHandle<Dictionary> rootDic =
     SharedHandle<Dictionary>((Dictionary*)rootEntry);
   Dictionary* infoDic = (Dictionary*)rootDic->get("info");
   // retrieve infoHash
   ShaVisitor v;
   infoDic->accept(&v);
-  int32_t len;
+  int len;
   v.getHash(infoHash, len);
   infoHashString = Util::toHex(infoHash, INFO_HASH_LENGTH);
   // calculate the number of pieces
@@ -193,8 +240,12 @@ void DefaultBtContext::load(const string& torrentFile) {
   extractPieceHash((unsigned char*)pieceHashData->getData(),
 		   pieceHashData->getLen(),
 		   PIECE_HASH_LENGTH);
+  // retrieve uri-list.
+  // This implemantation obeys HTTP-Seeding specification:
+  // see http://www.getright.com/seedtorrent.html
+  Strings urlList = extractUrlList(rootDic->get("url-list"));
   // retrieve file entries
-  extractFileEntries(infoDic, torrentFile);
+  extractFileEntries(infoDic, defaultName, urlList);
   // retrieve announce
   Data* announceData = (Data*)rootDic->get("announce");
   List* announceListData = (List*)rootDic->get("announce-list");
@@ -242,3 +293,73 @@ int32_t DefaultBtContext::getPieceLength() const {
 int32_t DefaultBtContext::getNumPieces() const {
   return numPieces;
 }
+
+string DefaultBtContext::getActualBasePath() const
+{
+  return _dir+"/"+name;
+}
+
+Integers DefaultBtContext::computeFastSet(const string& ipaddr, int32_t fastSetSize)
+{
+  Integers fastSet;
+  struct in_addr saddr;
+  if(inet_aton(ipaddr.c_str(), &saddr) == 0) {
+    abort();
+  }
+  unsigned char tx[24];
+  memcpy(tx, (void*)&saddr.s_addr, 4);
+  if((tx[0] & 0x80) == 0 || (tx[0] & 0x40) == 0) {
+    tx[2] = 0x00;
+    tx[3] = 0x00;
+  } else {
+    tx[3] = 0x00;
+  }
+  memcpy(tx+4, infoHash, 20);
+  unsigned char x[20];
+  MessageDigestHelper::digest(x, sizeof(x), "sha1", tx, 24);
+  while((int32_t)fastSet.size() < fastSetSize) {
+    for(int32_t i = 0; i < 5 && (int32_t)fastSet.size() < fastSetSize; i++) {
+      int32_t j = i*4;
+      uint32_t ny;
+      memcpy(&ny, x+j, 4);
+      uint32_t y = ntohl(ny);
+      int32_t index = y%numPieces;
+      if(find(fastSet.begin(), fastSet.end(), index) == fastSet.end()) {
+	fastSet.push_back(index);
+      }
+    }
+    unsigned char temp[20];
+    MessageDigestHelper::digest(temp, sizeof(temp), "sha1", x, sizeof(x));
+    memcpy(x, temp, sizeof(x));
+  }
+  return fastSet;
+}
+
+ostream& operator<<(ostream& o, const DefaultBtContext& ctx)
+{
+  o << "*** BitTorrent File Information ***" << "\n";
+  o << "Mode: " << (ctx.getFileMode() == DownloadContext::SINGLE ? "Single File Torrent":"Multi File Torrent") << "\n";
+  o << "Announce:" << "\n";
+  AnnounceTiers tiers = ctx.getAnnounceTiers();
+  for(AnnounceTiers::const_iterator itr = tiers.begin(); itr != tiers.end(); ++itr) {
+    const AnnounceTierHandle& tier = *itr;
+    for(Strings::const_iterator uriItr = tier->urls.begin(); uriItr != tier->urls.end(); ++uriItr) {
+      o << " " << *uriItr;
+    }
+    o << "\n";
+  }
+  o << "Info Hash: " << ctx.getInfoHashAsString() << "\n";
+  o << "Piece Length: " << Util::abbrevSize(ctx.getPieceLength()) << "B\n";
+  o << "The Number of Pieces: " << ctx.getNumPieces() << "\n";
+  o << "Total Length: " << Util::abbrevSize(ctx.getTotalLength()) << "B\n";
+  if(ctx.getFileMode() == DownloadContext::MULTI) {
+    o << "Name: " << ctx.getName() << "\n";
+  }
+  Util::toStream(o, ctx.getFileEntries());
+  return o;
+}
+
+void DefaultBtContext::setRandomizer(const RandomizerHandle& randomizer)
+{
+  _randomizer = randomizer;
+}
