@@ -39,6 +39,10 @@
 #include "Dictionary.h"
 #include "Data.h"
 #include "List.h"
+
+class Randomizer;
+typedef SharedHandle<Randomizer> RandomizerHandle;
+
 #define INFO_HASH_LENGTH 20
 #define PIECE_HASH_LENGTH 20
 
@@ -56,16 +60,26 @@ private:
   string name;
   int32_t numPieces;
   string peerId;
+  string _peerIdPrefix;
   AnnounceTiers announceTiers;
+  RandomizerHandle _randomizer;
+
+  RequestGroup* _ownerRequestGroup;
 
   void clear();
   void extractPieceHash(const unsigned char* hashData,
 			int32_t hashDataLength,
 			int32_t hashLength);
   void extractFileEntries(Dictionary* infoDic,
-			  const string& defaultName);
+			  const string& defaultName,
+			  const Strings& urlList);
   void extractAnnounce(Data* announceData);
   void extractAnnounceList(List* announceListData);
+
+  Strings extractUrlList(const MetaEntry* obj);
+
+  void processMetaInfo(const MetaEntry* rootEntry, const string& defaultName);
+
  public:
   DefaultBtContext();
   virtual ~DefaultBtContext();
@@ -89,16 +103,25 @@ private:
 
   virtual FileEntries getFileEntries() const;
 
+  virtual string getPieceHashAlgo() const
+  {
+    return "sha1";
+  }
+
   virtual AnnounceTiers getAnnounceTiers() const;
 
   virtual void load(const string& torrentFile);
 
+  void loadFromMemory(const char* content, int32_t length, const string& defaultName);
+
   virtual string getName() const;
 
   virtual int32_t getPieceLength() const;
   
   virtual int32_t getNumPieces() const;
 
+  virtual string getActualBasePath() const;
+
   virtual const unsigned char* getPeerId() {
     if(peerId == "") {
       peerId = generatePeerId();
@@ -106,7 +129,41 @@ private:
     return (const unsigned char*)peerId.c_str();
   }
 
+  virtual Integers computeFastSet(const string& ipaddr, int32_t fastSetSize);
+
+  virtual RequestGroup* getOwnerRequestGroup()
+  {
+    return _ownerRequestGroup;
+  }
+
   string generatePeerId() const;
+
+  void setPeerIdPrefix(const string& peerIdPrefix)
+  {
+    _peerIdPrefix = peerIdPrefix;
+  }
+
+  // for unit test
+  void setInfoHash(const unsigned char* infoHash)
+  {
+    memcpy(this->infoHash, infoHash, sizeof(this->infoHash));
+  }
+
+  void setNumPieces(int32_t numPieces)
+  {
+    this->numPieces = numPieces;
+  }
+   
+  void setOwnerRequestGroup(RequestGroup* owner)
+  {
+    _ownerRequestGroup = owner;
+  }
+
+  void setRandomizer(const RandomizerHandle& randomizer);
+
+  friend ostream& operator<<(ostream& o, const DefaultBtContext& ctx);
 };
 
+typedef SharedHandle<DefaultBtContext> DefaultBtContextHandle;
+
 #endif // _D_DEFAULT_BT_CONTEXT_H_
