@@ -44,10 +44,7 @@ public:
 
   virtual ~DefaultDiskWriter();
 
-  virtual void initAndOpenFile(const string& filename,
-			       int64_t totalLength = 0);
-
-  static DefaultDiskWriter* createNewDiskWriter(const Option* option);
+  virtual void initAndOpenFile(const string& filename, int64_t totalLength = 0);
 };
 
 typedef SharedHandle<DefaultDiskWriter> DefaultDiskWriterHandle;
