@@ -35,6 +35,8 @@
 #include "DefaultPeerStorage.h"
 #include "LogFactory.h"
 #include "BtRegistry.h"
+#include "message.h"
+#include "a2time.h"
 
 DefaultPeerStorage::DefaultPeerStorage(BtContextHandle btContext,
 				       const Option* option):
@@ -80,7 +82,7 @@ void DefaultPeerStorage::addPeer(const Peers& peers) {
       itr != peers.end(); itr++) {
     const PeerHandle& peer = *itr;
     if(addPeer(peer)) {
-      logger->debug("Adding peer %s:%d",
+      logger->debug(MSG_ADDING_PEER,
 		    peer->ipaddr.c_str(), peer->port);
     }
   }  
@@ -110,9 +112,9 @@ PeerHandle DefaultPeerStorage::getUnusedPeer() {
 class FindPeer {
 private:
   string ipaddr;
-  int port;
+  int32_t port;
 public:
-  FindPeer(const string& ipaddr, int port):ipaddr(ipaddr), port(port) {}
+  FindPeer(const string& ipaddr, int32_t port):ipaddr(ipaddr), port(port) {}
 
   bool operator()(const PeerHandle& peer) const {
     return ipaddr == peer->ipaddr && port == peer->port;
@@ -120,7 +122,7 @@ public:
 };
 
 PeerHandle DefaultPeerStorage::getPeer(const string& ipaddr,
-				       int port) const {
+				       int32_t port) const {
   Peers::const_iterator itr = find_if(peers.begin(), peers.end(),
 				      FindPeer(ipaddr, port));
   if(itr == peers.end()) {
@@ -162,12 +164,18 @@ Peers DefaultPeerStorage::getActivePeers() {
 class CalculateStat {
 private:
   TransferStat _stat;
+  struct timeval _now;
 public:
+  CalculateStat()
+  {
+    gettimeofday(&_now, 0);
+  }
+
   void operator()(const PeerHandle& peer)
   {
     if(peer->isActive()) {
-      _stat.downloadSpeed += peer->calculateDownloadSpeed();
-      _stat.uploadSpeed += peer->calculateUploadSpeed();
+      _stat.downloadSpeed += peer->calculateDownloadSpeed(_now);
+      _stat.uploadSpeed += peer->calculateUploadSpeed(_now);
     }
     _stat.sessionDownloadLength += peer->getSessionDownloadLength();
     _stat.sessionUploadLength += peer->getSessionUploadLength();    
@@ -184,10 +192,12 @@ TransferStat DefaultPeerStorage::calculateStat() {
   TransferStat stat = calStat.getTransferStat();
   stat.sessionDownloadLength += removedPeerSessionDownloadLength;
   stat.sessionUploadLength += removedPeerSessionUploadLength;
+  stat.setAllTimeUploadLength(btRuntime->getUploadLengthAtStartup()+
+			      stat.getSessionUploadLength());
   return stat;
 }
 
-void DefaultPeerStorage::deleteUnusedPeer(int delSize) {
+void DefaultPeerStorage::deleteUnusedPeer(int32_t delSize) {
   Peers temp;
   for(Peers::reverse_iterator itr = peers.rbegin();
       itr != peers.rend(); ++itr) {
