@@ -33,28 +33,34 @@
  */
 /* copyright --> */
 #include "DefaultPieceStorage.h"
+#include "DownloadContext.h"
+#include "Piece.h"
+#include "Peer.h"
 #include "LogFactory.h"
 #include "prefs.h"
 #include "DirectDiskAdaptor.h"
 #include "MultiDiskAdaptor.h"
 #include "CopyDiskAdaptor.h"
-#include "DefaultDiskWriter.h"
-#include "DlAbortEx.h"
+#include "DiskWriter.h"
 #include "BitfieldManFactory.h"
-#include "FileAllocationMonitor.h"
-#include "DiskAdaptorWriter.h"
-#include "ChunkChecksumValidator.h"
+#include "message.h"
+#include "DefaultDiskWriterFactory.h"
+#include "DlAbortEx.h"
+#include "Util.h"
+#include "a2functional.h"
+#include <numeric>
 
-DefaultPieceStorage::DefaultPieceStorage(BtContextHandle btContext, const Option* option):
-  btContext(btContext),
+DefaultPieceStorage::DefaultPieceStorage(const DownloadContextHandle& downloadContext, const Option* option):
+  downloadContext(downloadContext),
   diskAdaptor(0),
+  _diskWriterFactory(new DefaultDiskWriterFactory()),
   endGamePieceNum(END_GAME_PIECE_NUM),
   option(option)
 {
   bitfieldMan =
     BitfieldManFactory::getFactoryInstance()->
-    createBitfieldMan(btContext->getPieceLength(),
-		      btContext->getTotalLength());
+    createBitfieldMan(downloadContext->getPieceLength(),
+		      downloadContext->getTotalLength());
   logger = LogFactory::getInstance();
 }
 
@@ -62,17 +68,20 @@ DefaultPieceStorage::~DefaultPieceStorage() {
   delete bitfieldMan;
 }
 
-bool DefaultPieceStorage::hasMissingPiece(const PeerHandle& peer) {
+bool DefaultPieceStorage::hasMissingPiece(const PeerHandle& peer)
+{
   return bitfieldMan->hasMissingPiece(peer->getBitfield(),
 				      peer->getBitfieldLength());
 }
 
-bool DefaultPieceStorage::isEndGame() {
+bool DefaultPieceStorage::isEndGame()
+{
   return bitfieldMan->countMissingBlock() <= endGamePieceNum;
 }
 
-int DefaultPieceStorage::getMissingPieceIndex(const PeerHandle& peer) {
-  int index = -1;
+int32_t DefaultPieceStorage::getMissingPieceIndex(const PeerHandle& peer)
+{
+  int32_t index = -1;
   if(isEndGame()) {
     index = bitfieldMan->getMissingIndex(peer->getBitfield(),
 					 peer->getBitfieldLength());
@@ -83,7 +92,8 @@ int DefaultPieceStorage::getMissingPieceIndex(const PeerHandle& peer) {
   return index;
 }
 
-PieceHandle DefaultPieceStorage::checkOutPiece(int index) {
+PieceHandle DefaultPieceStorage::checkOutPiece(int32_t index)
+{
   if(index == -1) {
     return 0;
   }
@@ -103,7 +113,8 @@ PieceHandle DefaultPieceStorage::checkOutPiece(int index) {
  * Newly instantiated piece is not added to usedPieces.
  * Because it is waste of memory and there is no chance to use them later.
  */
-PieceHandle DefaultPieceStorage::getPiece(int index) {
+PieceHandle DefaultPieceStorage::getPiece(int32_t index)
+{
   if(0 <= index && index <= bitfieldMan->getMaxIndex()) {
     PieceHandle piece = findUsedPiece(index);
     if(piece.isNull()) {
@@ -118,22 +129,24 @@ PieceHandle DefaultPieceStorage::getPiece(int index) {
   }
 }
 
-void DefaultPieceStorage::addUsedPiece(const PieceHandle& piece) {
+void DefaultPieceStorage::addUsedPiece(const PieceHandle& piece)
+{
   usedPieces.push_back(piece);
 }
 
 class FindPiece {
 private:
-  int index;
+  int32_t index;
 public:
-  FindPiece(int index):index(index) {}
+  FindPiece(int32_t index):index(index) {}
 
   bool operator()(const PieceHandle& piece) {
     return piece->getIndex() == index;
   }
 };
 
-PieceHandle DefaultPieceStorage::findUsedPiece(int index) const {
+PieceHandle DefaultPieceStorage::findUsedPiece(int32_t index) const
+{
   Pieces::const_iterator itr = find_if(usedPieces.begin(),
 				       usedPieces.end(),
 				       FindPiece(index));
@@ -144,18 +157,20 @@ PieceHandle DefaultPieceStorage::findUsedPiece(int index) const {
   }
 }
 
-PieceHandle DefaultPieceStorage::getMissingPiece(const PeerHandle& peer) {
-  int index = getMissingPieceIndex(peer);
+PieceHandle DefaultPieceStorage::getMissingPiece(const PeerHandle& peer)
+{
+  int32_t index = getMissingPieceIndex(peer);
   return checkOutPiece(index);
 }
 
-int DefaultPieceStorage::getMissingFastPieceIndex(const PeerHandle& peer) {
-  int index = -1;
-  if(peer->isFastExtensionEnabled() && peer->countFastSet() > 0) {
+int32_t DefaultPieceStorage::getMissingFastPieceIndex(const PeerHandle& peer)
+{
+  int32_t index = -1;
+  if(peer->isFastExtensionEnabled() && peer->countPeerAllowedIndexSet() > 0) {
     BitfieldMan tempBitfield(bitfieldMan->getBlockLength(),
 			     bitfieldMan->getTotalLength());
-    for(Integers::const_iterator itr = peer->getFastSet().begin();
-	itr != peer->getFastSet().end(); itr++) {
+    for(Integers::const_iterator itr = peer->getPeerAllowedIndexSet().begin();
+	itr != peer->getPeerAllowedIndexSet().end(); itr++) {
       if(!bitfieldMan->isBitSet(index) && peer->hasPiece(*itr)) {
 	tempBitfield.setBit(*itr);
       }
@@ -171,12 +186,58 @@ int DefaultPieceStorage::getMissingFastPieceIndex(const PeerHandle& peer) {
   return index;
 }
 
-PieceHandle DefaultPieceStorage::getMissingFastPiece(const PeerHandle& peer) {
-  int index = getMissingFastPieceIndex(peer);
+PieceHandle DefaultPieceStorage::getMissingFastPiece(const PeerHandle& peer)
+{
+  int32_t index = getMissingFastPieceIndex(peer);
   return checkOutPiece(index);
 }
 
-void DefaultPieceStorage::deleteUsedPiece(const PieceHandle& piece) {
+PieceHandle DefaultPieceStorage::getMissingPiece()
+{
+  return checkOutPiece(bitfieldMan->getSparseMissingUnusedIndex());
+}
+
+PieceHandle DefaultPieceStorage::getMissingPiece(const FileEntryHandle& fileEntry)
+{
+  BitfieldMan temp(*bitfieldMan);
+  temp.clearFilter();
+  temp.addFilter(fileEntry->getOffset(), fileEntry->getLength());
+  temp.enableFilter();
+
+  int32_t firstPieceIndex = START_INDEX(fileEntry->getOffset(), downloadContext->getPieceLength());
+  int32_t endPieceIndex = END_INDEX(fileEntry->getOffset(), fileEntry->getLength(), downloadContext->getPieceLength());
+  if(!temp.isBitSet(firstPieceIndex) && !temp.isUseBitSet(firstPieceIndex)) {
+    PieceHandle piece = findUsedPiece(firstPieceIndex);
+    if(!piece.isNull()) {
+      if(piece->isRangeComplete(fileEntry->getOffset()-firstPieceIndex*downloadContext->getPieceLength(),
+				fileEntry->getLength() > downloadContext->getPieceLength() ?
+				downloadContext->getPieceLength():fileEntry->getLength())) {
+	temp.setBit(firstPieceIndex);
+      }
+    }
+  }
+  if(firstPieceIndex != endPieceIndex && !temp.isBitSet(endPieceIndex) && !temp.isUseBitSet(endPieceIndex)) {
+    PieceHandle piece = findUsedPiece(endPieceIndex);
+    if(!piece.isNull()) {
+      if(piece->isRangeComplete(0, fileEntry->getOffset()+fileEntry->getLength()-endPieceIndex*downloadContext->getPieceLength())) {
+	temp.setBit(endPieceIndex);
+      }
+    }    
+  }
+  return checkOutPiece(temp.getSparseMissingUnusedIndex());
+}
+
+PieceHandle DefaultPieceStorage::getMissingPiece(int32_t index)
+{
+  if(hasPiece(index) || isPieceUsed(index)) {
+    return 0;
+  } else {
+    return checkOutPiece(index);
+  }
+}
+
+void DefaultPieceStorage::deleteUsedPiece(const PieceHandle& piece)
+{
   if(piece.isNull()) {
     return;
   }
@@ -186,14 +247,15 @@ void DefaultPieceStorage::deleteUsedPiece(const PieceHandle& piece) {
   }
 }
 
-void DefaultPieceStorage::reduceUsedPieces(int delMax) {
-  int toDelete = usedPieces.size()-delMax;
+void DefaultPieceStorage::reduceUsedPieces(int32_t delMax)
+{
+  int32_t toDelete = usedPieces.size()-delMax;
   if(toDelete <= 0) {
     return;
   }
-  int fillRate = 10;
+  int32_t fillRate = 10;
   while(fillRate < 50) {
-    int deleted = deleteUsedPiecesByFillRate(fillRate, toDelete);
+    int32_t deleted = deleteUsedPiecesByFillRate(fillRate, toDelete);
     if(deleted == 0) {
       break;
     }
@@ -202,15 +264,16 @@ void DefaultPieceStorage::reduceUsedPieces(int delMax) {
   }
 }
 
-int DefaultPieceStorage::deleteUsedPiecesByFillRate(int fillRate,
-							   int toDelete) {
-  int deleted = 0;
+int32_t DefaultPieceStorage::deleteUsedPiecesByFillRate(int32_t fillRate,
+							int32_t toDelete)
+{
+  int32_t deleted = 0;
   for(Pieces::iterator itr = usedPieces.begin();
       itr != usedPieces.end() && deleted < toDelete;) {
     PieceHandle& piece = *itr;
     if(!bitfieldMan->isUseBitSet(piece->getIndex()) &&
        piece->countCompleteBlock() <= piece->countBlock()*(fillRate/100.0)) {
-      logger->debug("Deleting used piece index=%d, fillRate(%%)=%d<=%d",
+      logger->debug(MSG_DELETING_USED_PIECE,
 		    piece->getIndex(),
 		    (piece->countCompleteBlock()*100)/piece->countBlock(),
 		    fillRate);
@@ -223,7 +286,8 @@ int DefaultPieceStorage::deleteUsedPiecesByFillRate(int fillRate,
   return deleted;
 }
 
-void DefaultPieceStorage::completePiece(const PieceHandle& piece) {
+void DefaultPieceStorage::completePiece(const PieceHandle& piece)
+{
   if(piece.isNull()) {
     return;
   }
@@ -239,68 +303,87 @@ void DefaultPieceStorage::completePiece(const PieceHandle& piece) {
   if(downloadFinished()) {
     diskAdaptor->onDownloadComplete();
     if(isSelectiveDownloadingMode()) {
-      logger->notice(_("Download of selected files was complete."));
+      logger->notice(MSG_SELECTIVE_DOWNLOAD_COMPLETED);
       // following line was commented out in order to stop sending request
       // message after user-specified files were downloaded.
       //finishSelectiveDownloadingMode();
     } else {
-      logger->info(_("The download was complete."));
+      logger->info(MSG_DOWNLOAD_COMPLETED);
     }
   }
 }
 
-bool DefaultPieceStorage::isSelectiveDownloadingMode() {
+bool DefaultPieceStorage::isSelectiveDownloadingMode()
+{
   return bitfieldMan->isFilterEnabled();
 }
 
-void DefaultPieceStorage::finishSelectiveDownloadingMode() {
+void DefaultPieceStorage::finishSelectiveDownloadingMode()
+{
   bitfieldMan->clearFilter();
   diskAdaptor->addAllDownloadEntry();
 }
 
 // not unittested
-void DefaultPieceStorage::cancelPiece(const PieceHandle& piece) {
+void DefaultPieceStorage::cancelPiece(const PieceHandle& piece)
+{
   if(piece.isNull()) {
     return;
   }
   bitfieldMan->unsetUseBit(piece->getIndex());
   if(!isEndGame()) {
-    if(piece->countCompleteBlock() == 0) {
+    if(piece->getCompletedLength() == 0) {
       deleteUsedPiece(piece);
     }
   }
 }
 
-bool DefaultPieceStorage::hasPiece(int index) {
+bool DefaultPieceStorage::hasPiece(int32_t index)
+{
   return bitfieldMan->isBitSet(index);
 }
 
-long long int DefaultPieceStorage::getTotalLength() {
+bool DefaultPieceStorage::isPieceUsed(int32_t index)
+{
+  return bitfieldMan->isUseBitSet(index);
+}
+
+int64_t DefaultPieceStorage::getTotalLength()
+{
   return bitfieldMan->getTotalLength();
 }
 
-long long int DefaultPieceStorage::getFilteredTotalLength() {
+int64_t DefaultPieceStorage::getFilteredTotalLength()
+{
   return bitfieldMan->getFilteredTotalLength();
 }
 
-long long int DefaultPieceStorage::getCompletedLength() {
-  return bitfieldMan->getCompletedLength();
+int64_t DefaultPieceStorage::getCompletedLength()
+{
+  return bitfieldMan->getCompletedLength()+getInFlightPieceCompletedLength();
 }
 
-long long int DefaultPieceStorage::getFilteredCompletedLength() {
-  return bitfieldMan->getFilteredCompletedLength();
+int64_t DefaultPieceStorage::getFilteredCompletedLength()
+{
+  return bitfieldMan->getFilteredCompletedLength()+getInFlightPieceCompletedLength();
+}
+
+int32_t DefaultPieceStorage::getInFlightPieceCompletedLength() const
+{
+  return accumulate(usedPieces.begin(), usedPieces.end(), 0, adopt2nd(plus<int32_t>(), mem_fun_sh(&Piece::getCompletedLength)));
 }
 
 // not unittested
-void DefaultPieceStorage::setFileFilter(const Strings& filePaths) {
-  if(btContext->getFileMode() != BtContext::MULTI || filePaths.empty()) {
+void DefaultPieceStorage::setFileFilter(const Strings& filePaths)
+{
+  if(downloadContext->getFileMode() != DownloadContext::MULTI || filePaths.empty()) {
     return;
   }
   diskAdaptor->removeAllDownloadEntry();
   for(Strings::const_iterator pitr = filePaths.begin();
       pitr != filePaths.end(); pitr++) {
     if(!diskAdaptor->addDownloadEntry(*pitr)) {
-      throw new DlAbortEx("No such file entry %s", (*pitr).c_str());
+      throw new DlAbortEx(EX_NO_SUCH_FILE_ENTRY, (*pitr).c_str());
     }
     FileEntryHandle fileEntry = diskAdaptor->getFileEntryFromPath(*pitr);
     bitfieldMan->addFilter(fileEntry->getOffset(), fileEntry->getLength());
@@ -308,10 +391,15 @@ void DefaultPieceStorage::setFileFilter(const Strings& filePaths) {
   bitfieldMan->enableFilter();
 }
 
-void DefaultPieceStorage::setFileFilter(const Integers& fileIndexes) {
+void DefaultPieceStorage::setFileFilter(IntSequence seq)
+{
+  Integers fileIndexes = seq.flush();
+  // TODO Is sorting necessary?
+  sort(fileIndexes.begin(), fileIndexes.end());
+  fileIndexes.erase(unique(fileIndexes.begin(), fileIndexes.end()), fileIndexes.end());
   Strings filePaths;
   const FileEntries& entries = diskAdaptor->getFileEntries();
-  for(int i = 0; i < (int)entries.size(); i++) {
+  for(int32_t i = 0; i < (int32_t)entries.size(); i++) {
     if(find(fileIndexes.begin(), fileIndexes.end(), i+1) != fileIndexes.end()) {
       logger->debug("index=%d is %s", i+1, entries[i]->getPath().c_str());
       filePaths.push_back(entries[i]->getPath());
@@ -321,66 +409,76 @@ void DefaultPieceStorage::setFileFilter(const Integers& fileIndexes) {
 }
 
 // not unittested
-void DefaultPieceStorage::clearFileFilter() {
+void DefaultPieceStorage::clearFileFilter()
+{
   bitfieldMan->clearFilter();
   diskAdaptor->addAllDownloadEntry();
 }
 
 // not unittested
-bool DefaultPieceStorage::downloadFinished() {
+bool DefaultPieceStorage::downloadFinished()
+{
+  // TODO iterate all requested FileEntry and Call bitfieldMan->isBitSetOffsetRange()
   return bitfieldMan->isFilteredAllBitSet();
 }
 
 // not unittested
-bool DefaultPieceStorage::allDownloadFinished() {
+bool DefaultPieceStorage::allDownloadFinished()
+{
   return bitfieldMan->isAllBitSet();
 }
 
 // not unittested
-void DefaultPieceStorage::initStorage() {
-  if(option->get(PREF_DIRECT_FILE_MAPPING) == V_TRUE) {
-    if(btContext->getFileMode() == BtContext::SINGLE) {
-      DefaultDiskWriterHandle writer = DefaultDiskWriter::createNewDiskWriter(option);
-      DirectDiskAdaptorHandle directDiskAdaptor = new DirectDiskAdaptor();
-      directDiskAdaptor->setDiskWriter(writer);
-      directDiskAdaptor->setTotalLength(btContext->getTotalLength());
-      this->diskAdaptor = directDiskAdaptor;
-    } else {
-      MultiDiskAdaptorHandle multiDiskAdaptor = new MultiDiskAdaptor();
-      multiDiskAdaptor->setPieceLength(btContext->getPieceLength());
-      multiDiskAdaptor->setTopDir(btContext->getName());
-      multiDiskAdaptor->setOption(option);
-      this->diskAdaptor = multiDiskAdaptor;
-    }
+void DefaultPieceStorage::initStorage()
+{
+  if(downloadContext->getFileMode() == DownloadContext::SINGLE) {
+    logger->debug("Instantiating DirectDiskAdaptor");
+    DiskWriterHandle writer = _diskWriterFactory->newDiskWriter();
+    writer->setDirectIOAllowed(option->getAsBool(PREF_ENABLE_DIRECT_IO));
+    DirectDiskAdaptorHandle directDiskAdaptor = new DirectDiskAdaptor();
+    directDiskAdaptor->setDiskWriter(writer);
+    directDiskAdaptor->setTotalLength(downloadContext->getTotalLength());
+    this->diskAdaptor = directDiskAdaptor;
   } else {
-    DefaultDiskWriterHandle writer = DefaultDiskWriter::createNewDiskWriter(option);
-    CopyDiskAdaptorHandle copyDiskAdaptor = new CopyDiskAdaptor();
-    copyDiskAdaptor->setDiskWriter(writer);
-    copyDiskAdaptor->setTempFilename(btContext->getName()+".a2tmp");
-    copyDiskAdaptor->setTotalLength(btContext->getTotalLength());
-    if(btContext->getFileMode() == BtContext::MULTI) {
-      copyDiskAdaptor->setTopDir(btContext->getName());
+    // file mode == DownloadContext::MULTI
+    if(option->get(PREF_DIRECT_FILE_MAPPING) == V_TRUE) {
+      logger->debug("Instantiating MultiDiskAdaptor");
+      MultiDiskAdaptorHandle multiDiskAdaptor = new MultiDiskAdaptor();
+      multiDiskAdaptor->setDirectIOAllowed(option->getAsBool(PREF_ENABLE_DIRECT_IO));
+      multiDiskAdaptor->setPieceLength(downloadContext->getPieceLength());
+      multiDiskAdaptor->setTopDir(downloadContext->getName());
+      this->diskAdaptor = multiDiskAdaptor;
+    } else {
+      logger->debug("Instantiating CopyDiskAdaptor");
+      DiskWriterHandle writer = _diskWriterFactory->newDiskWriter();
+      writer->setDirectIOAllowed(option->getAsBool(PREF_ENABLE_DIRECT_IO));
+      CopyDiskAdaptorHandle copyDiskAdaptor = new CopyDiskAdaptor();
+      copyDiskAdaptor->setDiskWriter(writer);
+      copyDiskAdaptor->setTempFilename(downloadContext->getName()+".a2tmp");
+      copyDiskAdaptor->setTotalLength(downloadContext->getTotalLength());
+      if(downloadContext->getFileMode() == DownloadContext::MULTI) {
+	copyDiskAdaptor->setTopDir(downloadContext->getName());
+      }
+      this->diskAdaptor = copyDiskAdaptor;
     }
-    this->diskAdaptor = copyDiskAdaptor;
   }
-  string storeDir = option->get(PREF_DIR);
-  if(storeDir == "") {
-    storeDir = ".";
-  }
-  diskAdaptor->setStoreDir(storeDir);
-  diskAdaptor->setFileEntries(btContext->getFileEntries());
+  diskAdaptor->setStoreDir(downloadContext->getDir());
+  diskAdaptor->setFileEntries(downloadContext->getFileEntries());
 }
 
 void DefaultPieceStorage::setBitfield(const unsigned char* bitfield,
-				      int bitfieldLength) {
+				      int32_t bitfieldLength)
+{
   bitfieldMan->setBitfield(bitfield, bitfieldLength);
 }
   
-int DefaultPieceStorage::getBitfieldLength() {
+int32_t DefaultPieceStorage::getBitfieldLength()
+{
   return bitfieldMan->getBitfieldLength();
 }
 
-const unsigned char* DefaultPieceStorage::getBitfield() {
+const unsigned char* DefaultPieceStorage::getBitfield()
+{
   return bitfieldMan->getBitfield();
 }
 
@@ -388,17 +486,20 @@ DiskAdaptorHandle DefaultPieceStorage::getDiskAdaptor() {
   return diskAdaptor;
 }
 
-int DefaultPieceStorage::getPieceLength(int index) {
+int32_t DefaultPieceStorage::getPieceLength(int32_t index)
+{
   return bitfieldMan->getBlockLength(index);
 }
 
-void DefaultPieceStorage::advertisePiece(int cuid, int index) {
+void DefaultPieceStorage::advertisePiece(int32_t cuid, int32_t index)
+{
   HaveEntry entry(cuid, index);
   haves.push_front(entry);
 }
 
-Integers DefaultPieceStorage::getAdvertisedPieceIndexes(int myCuid,
-							const Time& lastCheckTime) {
+Integers DefaultPieceStorage::getAdvertisedPieceIndexes(int32_t myCuid,
+							const Time& lastCheckTime)
+{
   Integers indexes;
   for(Haves::const_iterator itr = haves.begin(); itr != haves.end(); itr++) {
     const Haves::value_type& have = *itr;
@@ -416,9 +517,9 @@ Integers DefaultPieceStorage::getAdvertisedPieceIndexes(int myCuid,
 class FindElapsedHave
 {
 private:
-  int elapsed;
+  int32_t elapsed;
 public:
-  FindElapsedHave(int elapsed):elapsed(elapsed) {}
+  FindElapsedHave(int32_t elapsed):elapsed(elapsed) {}
 
   bool operator()(const HaveEntry& have) {
     if(have.getRegisteredTime().elapsed(elapsed)) {
@@ -429,11 +530,12 @@ public:
   }
 };
   
-void DefaultPieceStorage::removeAdvertisedPiece(int elapsed) {
+void DefaultPieceStorage::removeAdvertisedPiece(int32_t elapsed)
+{
   Haves::iterator itr =
     find_if(haves.begin(), haves.end(), FindElapsedHave(elapsed));
   if(itr != haves.end()) {
-    logger->debug("Removed %d have entries.", haves.end()-itr);
+    logger->debug(MSG_REMOVED_HAVE_ENTRY, haves.end()-itr);
     haves.erase(itr, haves.end());
   }
 }
@@ -443,14 +545,47 @@ void DefaultPieceStorage::markAllPiecesDone()
   bitfieldMan->setAllBit();
 }
 
-void DefaultPieceStorage::checkIntegrity()
+void DefaultPieceStorage::markPiecesDone(int64_t length)
 {
-  logger->notice("Validating file %s",
-		 diskAdaptor->getFilePath().c_str());
-  ChunkChecksumValidator v;
-  v.setDigestAlgo(DIGEST_ALGO_SHA1);
-  v.setDiskWriter(new DiskAdaptorWriter(diskAdaptor));
-  v.setFileAllocationMonitor(FileAllocationMonitorFactory::getFactory()->createNewMonitor());
-  v.validate(bitfieldMan, btContext->getPieceHashes(),
-	     btContext->getPieceLength());
+  if(length == bitfieldMan->getTotalLength()) {
+    bitfieldMan->setAllBit();
+  } else {
+    int32_t numPiece = length/bitfieldMan->getBlockLength();
+    if(numPiece > 0) {
+      bitfieldMan->setBitRange(0, numPiece-1);
+    }
+    int32_t r = (length%bitfieldMan->getBlockLength())/Piece::BLOCK_LENGTH;
+    if(r > 0) {
+      PieceHandle p = new Piece(numPiece, bitfieldMan->getBlockLength(numPiece));
+      for(int32_t i = 0; i < r; ++i) {
+	p->completeBlock(i);
+      }
+      addUsedPiece(p);
+    }
+  }
+}
+
+void DefaultPieceStorage::markPieceMissing(int32_t index)
+{
+  bitfieldMan->unsetBit(index);
+}
+
+void DefaultPieceStorage::addInFlightPiece(const Pieces& pieces)
+{
+  copy(pieces.begin(), pieces.end(), back_inserter(usedPieces));
+}
+
+int32_t DefaultPieceStorage::countInFlightPiece()
+{
+  return usedPieces.size();
+}
+
+Pieces DefaultPieceStorage::getInFlightPieces()
+{
+  return usedPieces;
+}
+
+void DefaultPieceStorage::setDiskWriterFactory(const DiskWriterFactoryHandle& diskWriterFactory)
+{
+  _diskWriterFactory = diskWriterFactory;
 }
