@@ -36,29 +36,33 @@
 #define _D_DEFAULT_PIECE_STORAGE_H_
 
 #include "PieceStorage.h"
-#include "BtContext.h"
-#include "DiskAdaptor.h"
-#include "BitfieldMan.h"
-#include "Logger.h"
-#include "Option.h"
-#include "Piece.h"
-#include "FileAllocator.h"
+
+class DownloadContext;
+typedef SharedHandle<DownloadContext> DownloadContextHandle;
+class BitfieldMan;
+class Logger;
+class Option;
+typedef deque<PieceHandle> Pieces;
+class DiskWriterFactory;
+typedef SharedHandle<DiskWriterFactory> DiskWriterFactoryHandle;
+class FileEntry;
+typedef SharedHandle<FileEntry> FileEntryHandle;
 
 #define END_GAME_PIECE_NUM 20
 
 class HaveEntry {
 private:
-  int cuid;
-  int index;
+  int32_t cuid;
+  int32_t index;
   Time registeredTime;
 public:
-  HaveEntry(int cuid, int index):
+  HaveEntry(int32_t cuid, int32_t index):
     cuid(cuid),
     index(index) {}
 
-  int getCuid() const { return cuid; }
+  int32_t getCuid() const { return cuid; }
 
-  int getIndex() const { return index; }
+  int32_t getIndex() const { return index; }
 
   const Time& getRegisteredTime() const { return registeredTime; }
 };
@@ -67,25 +71,28 @@ typedef deque<HaveEntry> Haves;
 
 class DefaultPieceStorage : public PieceStorage {
 private:
-  BtContextHandle btContext;
+  DownloadContextHandle downloadContext;
   BitfieldMan* bitfieldMan;
   DiskAdaptorHandle diskAdaptor;
+  DiskWriterFactoryHandle _diskWriterFactory;
   Pieces usedPieces;
   int32_t endGamePieceNum;
   Logger* logger;
   const Option* option;
   Haves haves;
-  FileAllocatorHandle createFileAllocator();
 
-  int getMissingPieceIndex(const PeerHandle& peer);
-  int getMissingFastPieceIndex(const PeerHandle& peer);
-  PieceHandle checkOutPiece(int index);
-  int deleteUsedPiecesByFillRate(int fillRate, int toDelete);
-  void reduceUsedPieces(int delMax);
+  int32_t getMissingPieceIndex(const PeerHandle& peer);
+  int32_t getMissingFastPieceIndex(const PeerHandle& peer);
+  PieceHandle checkOutPiece(int32_t index);
+  int32_t deleteUsedPiecesByFillRate(int32_t fillRate, int32_t toDelete);
+  void reduceUsedPieces(int32_t delMax);
   void deleteUsedPiece(const PieceHandle& piece);
-  PieceHandle findUsedPiece(int index) const;
+  PieceHandle findUsedPiece(int32_t index) const;
+
+  int32_t getInFlightPieceCompletedLength() const;
+
 public:
-  DefaultPieceStorage(BtContextHandle btContext, const Option* option);
+  DefaultPieceStorage(const DownloadContextHandle& downloadContext, const Option* option);
   virtual ~DefaultPieceStorage();
 
   virtual bool hasMissingPiece(const PeerHandle& peer);
@@ -94,27 +101,34 @@ public:
 
   virtual PieceHandle getMissingFastPiece(const PeerHandle& peer);
 
-  virtual PieceHandle getPiece(int index);
+  virtual PieceHandle getMissingPiece();
+  virtual PieceHandle getMissingPiece(const FileEntryHandle& fileEntry);
+
+  virtual PieceHandle getMissingPiece(int32_t index);
+
+  virtual PieceHandle getPiece(int32_t index);
 
   virtual void completePiece(const PieceHandle& piece);
 
   virtual void cancelPiece(const PieceHandle& piece);
 
-  virtual bool hasPiece(int index);
+  virtual bool hasPiece(int32_t index);
 
-  virtual long long int getTotalLength();
+  virtual bool isPieceUsed(int32_t index);
 
-  virtual long long int getFilteredTotalLength();
+  virtual int64_t getTotalLength();
 
-  virtual long long int getCompletedLength();
+  virtual int64_t getFilteredTotalLength();
 
-  virtual long long int getFilteredCompletedLength();
+  virtual int64_t getCompletedLength();
+
+  virtual int64_t getFilteredCompletedLength();
 
   virtual void initStorage();
 
   virtual void setFileFilter(const Strings& filePaths);
 
-  virtual void setFileFilter(const Integers& fileIndexes);
+  virtual void setFileFilter(IntSequence seq);
 
   virtual void clearFileFilter();
 
@@ -123,9 +137,9 @@ public:
   virtual bool allDownloadFinished();
 
   virtual void setBitfield(const unsigned char* bitfield,
-			   int bitfieldLength);
+			   int32_t bitfieldLength);
   
-  virtual int getBitfieldLength();
+  virtual int32_t getBitfieldLength();
 
   virtual const unsigned char* getBitfield();
 
@@ -145,24 +159,35 @@ public:
   
   virtual DiskAdaptorHandle getDiskAdaptor();
 
-  virtual int getPieceLength(int index);
+  virtual int32_t getPieceLength(int32_t index);
 
-  virtual void advertisePiece(int cuid, int index);
+  virtual void advertisePiece(int32_t cuid, int32_t index);
 
-  virtual Integers getAdvertisedPieceIndexes(int myCuid,
+  virtual Integers getAdvertisedPieceIndexes(int32_t myCuid,
 					     const Time& lastCheckTime);
 
-  virtual void removeAdvertisedPiece(int elapsed);
+  virtual void removeAdvertisedPiece(int32_t elapsed);
 
   virtual void markAllPiecesDone();
 
-  virtual void checkIntegrity();
+  virtual void markPiecesDone(int64_t length);
+
+  virtual void markPieceMissing(int32_t index);
+
+  virtual void addInFlightPiece(const Pieces& pieces);
+
+  virtual int32_t countInFlightPiece();
+
+  virtual Pieces getInFlightPieces();
 
   /**
    * This method is made private for test purpose only.
    */
   void addUsedPiece(const PieceHandle& piece);
 
+  void setDiskWriterFactory(const DiskWriterFactoryHandle& diskWriterFactory);
 };
 
+typedef SharedHandle<DefaultPieceStorage> DefaultPieceStorageHandle;
+
 #endif // _D_DEFAULT_PIECE_STORAGE_H_
