@@ -34,11 +34,13 @@
 /* copyright --> */
 #include "DirectDiskAdaptor.h"
 
-string DirectDiskAdaptor::getFilePath() {
+string DirectDiskAdaptor::getFilePath()
+{
   return storeDir+"/"+fileEntries.front()->getPath();
 }
 
-void DirectDiskAdaptor::onDownloadComplete() {
+void DirectDiskAdaptor::onDownloadComplete()
+{
   closeFile();
   openFile();
 }
