@@ -33,24 +33,27 @@
  */
 /* copyright --> */
 #include "DiskAdaptor.h"
-#include "DlAbortEx.h"
 #include "LogFactory.h"
+#include "message.h"
+#include "DlAbortEx.h"
 
 DiskAdaptor::DiskAdaptor():logger(LogFactory::getInstance()) {}
 
 DiskAdaptor::~DiskAdaptor() {}
 
-FileEntryHandle DiskAdaptor::getFileEntryFromPath(const string& fileEntryPath) const {
+FileEntryHandle DiskAdaptor::getFileEntryFromPath(const string& fileEntryPath) const
+{
   for(FileEntries::const_iterator itr = fileEntries.begin();
       itr != fileEntries.end(); itr++) {
     if((*itr)->getPath() == fileEntryPath) {
       return *itr;
     }
   }
-  throw new DlAbortEx("No such file entry <%s>", fileEntryPath.c_str());
+  throw new DlAbortEx(EX_NO_SUCH_FILE_ENTRY, fileEntryPath.c_str());
 }
 
-bool DiskAdaptor::addDownloadEntry(const string& fileEntryPath) {
+bool DiskAdaptor::addDownloadEntry(const string& fileEntryPath)
+{
   for(FileEntries::iterator itr = fileEntries.begin();
       itr != fileEntries.end(); itr++) {
     if((*itr)->getPath() == fileEntryPath) {
@@ -61,7 +64,8 @@ bool DiskAdaptor::addDownloadEntry(const string& fileEntryPath) {
   return false;
 }
 
-bool DiskAdaptor::addDownloadEntry(int index) {
+bool DiskAdaptor::addDownloadEntry(int index)
+{
   if(fileEntries.size() <= (unsigned int)index) {
     return false;
   }
@@ -69,14 +73,16 @@ bool DiskAdaptor::addDownloadEntry(int index) {
   return true;
 }
 
-void DiskAdaptor::addAllDownloadEntry() {
+void DiskAdaptor::addAllDownloadEntry()
+{
   for(FileEntries::iterator itr = fileEntries.begin();
       itr != fileEntries.end(); itr++) {
     (*itr)->setRequested(true);
   }
 }
 
-void DiskAdaptor::removeAllDownloadEntry() {
+void DiskAdaptor::removeAllDownloadEntry()
+{
   for(FileEntries::iterator itr = fileEntries.begin();
       itr != fileEntries.end(); itr++) {
     (*itr)->setRequested(false);
