@@ -35,14 +35,12 @@
 #ifndef _D_DISK_ADAPTOR_H_
 #define _D_DISK_ADAPTOR_H_
 
-#include "common.h"
+#include "BinaryStream.h"
 #include "FileEntry.h"
 #include "Logger.h"
-#ifdef ENABLE_MESSAGE_DIGEST
-#include "messageDigest.h"
-#endif // ENABLE_MESSAGE_DIGEST
+#include "FileAllocationIterator.h"
 
-class DiskAdaptor {
+class DiskAdaptor:public BinaryStream {
 protected:
   string storeDir;
   FileEntries fileEntries;
@@ -59,21 +57,17 @@ public:
 
   virtual void initAndOpenFile() = 0;
 
-  virtual void writeData(const unsigned char* data, int32_t len, int64_t offset) = 0;
-
-  virtual int32_t readData(unsigned char* data, int32_t len, int64_t offset) = 0;
-
-#ifdef ENABLE_MESSAGE_DIGEST
-  virtual string messageDigest(int64_t offset, int64_t length,
-			       const MessageDigestContext::DigestAlgo& algo) = 0;
-#endif // ENABLE_MESSAGE_DIGEST
-
   virtual void onDownloadComplete() = 0;  
 
   virtual bool fileExists() = 0;
 
   virtual string getFilePath() = 0;
 
+  virtual int64_t size() const = 0;
+
+  // optional behavior
+  virtual void truncate(int64_t length) {}
+
   void setFileEntries(const FileEntries& fileEntries) {
     this->fileEntries = fileEntries;
   }
@@ -93,6 +87,12 @@ public:
   void setStoreDir(const string& storeDir) { this->storeDir = storeDir; }
 
   const string& getStoreDir() const { return this->storeDir; }
+
+  virtual FileAllocationIteratorHandle fileAllocationIterator() = 0;
+
+  virtual void enableDirectIO() {}
+
+  virtual void disableDirectIO() {}
 };
 
 typedef SharedHandle<DiskAdaptor> DiskAdaptorHandle;
