@@ -35,33 +35,33 @@
 #ifndef _D_DISK_WRITER_H_
 #define _D_DISK_WRITER_H_
 
-#include <string>
-#include "common.h"
-#ifdef ENABLE_MESSAGE_DIGEST
-#include "messageDigest.h"
-#endif // ENABLE_MESSAGE_DIGEST
-
-using namespace std;
+#include "BinaryStream.h"
 
 /**
  * Interface for writing to a binary stream of bytes.
  *
  */
-class DiskWriter {
+class DiskWriter:public BinaryStream {
+protected:
+  bool _directIOAllowed;
+
 public:
+  DiskWriter():_directIOAllowed(false) {}
+
   virtual ~DiskWriter() {}
   /**
    * Creates a file output stream to write to the file with the specified name.
    * If the file exists, then it is truncated to 0 length.
    * @param filename the file name to be opened.
    */
-  virtual void initAndOpenFile(const string& filename, int64_t totalLength = 0) = 0;
+  virtual void initAndOpenFile(const string& filename, int64_t totalLength = 0)  = 0;
   
   virtual void openFile(const string& filename, int64_t totalLength = 0) = 0;
 
   /**
    * Closes this output stream.
    */
+  // TODO we have to examine the return value of close()
   virtual void closeFile() = 0;
 
   /**
@@ -72,34 +72,24 @@ public:
    */
   virtual void openExistingFile(const string& filename, int64_t totalLength = 0) = 0;
 
-  /*
-   * Writes len bytes from data to this binary stream at offset position.
-   * In case where offset position is not concerned(just write data
-   * sequencially, for example), those subclasses can ignore the offset value.
-   *
-   * @param data the data
-   * @param len the number of bytes to write
-   * @param position the offset of this binary stream
-   */
-  virtual void writeData(const char* data, int32_t len, int64_t position = 0) = 0;
-  virtual void writeData(const unsigned char* data, int32_t len, int64_t position = 0)
-  {
-    writeData((const char*)data, len, position);
-  }
-
-  virtual int readData(char* data, int32_t len, int64_t position) = 0;
-  virtual int readData(unsigned char* data, int32_t len, int64_t position) {
-    return readData((char*)data, len, position);
-  }
-#ifdef ENABLE_MESSAGE_DIGEST
-  virtual string messageDigest(int64_t offset, int64_t length,
-			       const MessageDigestContext::DigestAlgo& algo) = 0;
-#endif // ENABLE_MESSAGE_DIGEST
-
   virtual void truncate(int64_t length) = 0;
 
   // Returns file length
   virtual int64_t size() const = 0;
+
+  virtual void enableDirectIO() = 0;
+
+  virtual void disableDirectIO() = 0;
+
+  virtual bool directIOAllowed() const
+  {
+    return _directIOAllowed;
+  }
+
+  void setDirectIOAllowed(bool f)
+  {
+    _directIOAllowed = f;
+  }
 };
 
 typedef SharedHandle<DiskWriter> DiskWriterHandle;
