@@ -36,11 +36,11 @@
 #define _D_DOWNLOAD_COMMAND_H_
 
 #include "AbstractCommand.h"
-#include "TransferEncoding.h"
-#include "TimeA2.h"
-#include "PeerStat.h"
 
-using namespace std;
+class TransferEncoding;
+typedef SharedHandle<TransferEncoding> TransferEncodingHandle;
+class PeerStat;
+typedef SharedHandle<PeerStat> PeerStatHandle;
 
 class DownloadCommand : public AbstractCommand {
 private:
@@ -48,12 +48,17 @@ private:
   int32_t startupIdleTime;
   int32_t lowestDownloadSpeedLimit;
   PeerStatHandle peerStat;
+
+  void validatePieceHash(const SegmentHandle& segment);
+
+  void checkLowestDownloadSpeed() const;
 protected:
   TransferEncodingHandle transferDecoder;
 
   virtual bool executeInternal();
 
   virtual bool prepareForNextSegment();
+
 public:
   DownloadCommand(int cuid,
 		  const RequestHandle req,
@@ -62,10 +67,7 @@ public:
 		  const SocketHandle& s);
   virtual ~DownloadCommand();
 
-  void setTransferDecoder(const TransferEncodingHandle& transferDecoder)
-  {
-    this->transferDecoder = transferDecoder;
-  }
+  void setTransferDecoder(const TransferEncodingHandle& transferDecoder);
 
   void setMaxDownloadSpeedLimit(int32_t maxDownloadSpeedLimit) {
     this->maxDownloadSpeedLimit = maxDownloadSpeedLimit;
