@@ -33,6 +33,14 @@
  */
 /* copyright --> */
 #include "DownloadEngine.h"
+#include "Socket.h"
+#include "NameResolver.h"
+#include "StatCalc.h"
+#include "DownloadResult.h"
+#include "RequestGroup.h"
+#include "RequestGroupMan.h"
+#include "FileAllocationMan.h"
+#include "CheckIntegrityMan.h"
 #include "Util.h"
 #include "LogFactory.h"
 #include "TimeA2.h"
@@ -40,16 +48,47 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <errno.h>
-#include <algorithm>
+#include <signal.h>
 
-using namespace std;
+// 0 ... running
+// 1 ... stop signal detected
+// 2 ... stop signal processed by DownloadEngine
+// 3 ... 2nd stop signal(force shutdown) detected
+// 4 ... 2nd stop signal processed by DownloadEngine
+volatile sig_atomic_t globalHaltRequested = 0;
+
+SocketEntry::SocketEntry(const SocketHandle& socket,
+			 Command* command,
+			 TYPE type):
+  socket(socket), command(command), type(type) {}
+
+bool SocketEntry::operator==(const SocketEntry& entry)
+{
+  return socket == entry.socket &&
+    command == entry.command &&
+    type == entry.type;
+}
+
+#ifdef ENABLE_ASYNC_DNS
+NameResolverEntry::NameResolverEntry(const NameResolverHandle& nameResolver,
+					     Command* command):
+  nameResolver(nameResolver), command(command) {}
+
+bool NameResolverEntry::operator==(const NameResolverEntry& entry)
+{
+  return nameResolver == entry.nameResolver &&
+    command == entry.command;
+}
+#endif // ENABLE_ASYNC_DNS
 
 DownloadEngine::DownloadEngine():logger(LogFactory::getInstance()),
+				 _statCalc(0),
+				 _haltRequested(false),
 				 noWait(false),
 				 _requestGroupMan(0),
 				 _fileAllocationMan(0),
-				 _checkIntegrityMan(0) {}
+				 _checkIntegrityMan(0)
+{}
 
 DownloadEngine::~DownloadEngine() {
   cleanQueue();
@@ -62,8 +101,8 @@ void DownloadEngine::cleanQueue() {
 
 void DownloadEngine::executeCommand(Command::STATUS statusFilter)
 {
-  int max = commands.size();
-  for(int i = 0; i < max; i++) {
+  int32_t max = commands.size();
+  for(int32_t i = 0; i < max; i++) {
     Command* com = commands.front();
     commands.pop_front();
     if(com->statusMatch(statusFilter)) {
@@ -79,7 +118,6 @@ void DownloadEngine::executeCommand(Command::STATUS statusFilter)
 }
 
 void DownloadEngine::run() {
-  initStatistics();
   Time cp;
   cp.setTimeInSec(0);
   Commands activeCommands;
@@ -112,7 +150,7 @@ void DownloadEngine::shortSleep() const {
 void DownloadEngine::waitData() {
   fd_set rfds;
   fd_set wfds;
-  int retval = 0;
+  int32_t retval = 0;
   struct timeval tv;
   
   memcpy(&rfds, &rfdset, sizeof(fd_set));
@@ -156,7 +194,7 @@ void DownloadEngine::updateFdSet() {
   for(NameResolverEntries::iterator itr = nameResolverEntries.begin();
       itr != nameResolverEntries.end(); ++itr) {
     NameResolverEntry& entry = *itr;
-    int fd = entry.nameResolver->getFds(&rfdset, &wfdset);
+    int32_t fd = entry.nameResolver->getFds(&rfdset, &wfdset);
     if(fdmax < fd) {
       fdmax = fd;
     }
@@ -165,7 +203,7 @@ void DownloadEngine::updateFdSet() {
   for(SocketEntries::iterator itr = socketEntries.begin();
       itr != socketEntries.end(); ++itr) {
     SocketEntry& entry = *itr;
-    int fd = entry.socket->getSockfd();
+    int32_t fd = entry.socket->getSockfd();
     switch(entry.type) {
     case SocketEntry::TYPE_RD:
       FD_SET(fd, &rfdset);
@@ -228,6 +266,48 @@ bool DownloadEngine::deleteSocketForWriteCheck(const SocketHandle& socket,
   return deleteSocket(entry);
 }
 
+void DownloadEngine::calculateStatistics()
+{
+  if(!_statCalc.isNull()) {
+    _statCalc->calculateStat(_requestGroupMan, _fileAllocationMan, _checkIntegrityMan);
+  }
+}
+
+void DownloadEngine::onEndOfRun()
+{
+  _requestGroupMan->closeFile();
+  _requestGroupMan->save();
+}
+
+void DownloadEngine::afterEachIteration()
+{
+  if(globalHaltRequested == 1) {
+    logger->notice(_("Shutdown sequence commencing... Press Ctrl-C again for emergency shutdown."));
+    _haltRequested = true;
+    _requestGroupMan->halt();
+    globalHaltRequested = 2;
+  } else if(globalHaltRequested == 3) {
+    logger->notice(_("Emergency shutdown sequence commencing..."));
+    _requestGroupMan->forceHalt();
+    globalHaltRequested = 4;
+  }
+}
+
+void DownloadEngine::fillCommand()
+{
+  addCommand(_requestGroupMan->getInitialCommands(this));
+}
+
+void DownloadEngine::setStatCalc(const StatCalcHandle& statCalc)
+{
+  _statCalc = statCalc;
+}
+
+void DownloadEngine::addCommand(const Commands& commands)
+{
+  this->commands.insert(this->commands.end(), commands.begin(), commands.end());
+}
+
 #ifdef ENABLE_ASYNC_DNS
 bool DownloadEngine::addNameResolverCheck(const NameResolverHandle& resolver,
 					  Command* command) {
