@@ -35,18 +35,25 @@
 #ifndef _D_DOWNLOAD_ENGINE_H_
 #define _D_DOWNLOAD_ENGINE_H_
 
-#include "Command.h"
-#include "Socket.h"
-#include "SegmentMan.h"
 #include "common.h"
-#include "Logger.h"
-#include "Option.h"
-#include "NameResolver.h"
-#include "RequestGroupMan.h"
-#include "FileAllocationMan.h"
-#include "CheckIntegrityMan.h"
+#include "a2netcompat.h"
+#include "Command.h"
 
+class SocketCore;
+typedef SharedHandle<SocketCore> SocketHandle;
 typedef deque<SocketHandle> Sockets;
+class Logger;
+class Option;
+class NameResolver;
+typedef SharedHandle<NameResolver> NameResolverHandle;
+class RequestGroupMan;
+typedef SharedHandle<RequestGroupMan> RequestGroupManHandle;
+class FileAllocationMan;
+typedef SharedHandle<FileAllocationMan> FileAllocationManHandle;
+class StatCalc;
+typedef SharedHandle<StatCalc> StatCalcHandle;
+class CheckIntegrityMan;
+typedef SharedHandle<CheckIntegrityMan> CheckIntegrityManHandle;
 
 class SocketEntry {
 public:
@@ -61,15 +68,9 @@ public:
 public:
   SocketEntry(const SocketHandle& socket,
 	      Command* command,
-	      TYPE type):
-    socket(socket), command(command), type(type) {}
-  ~SocketEntry() {}
+	      TYPE type);
 
-  bool operator==(const SocketEntry& entry) {
-    return socket == entry.socket &&
-      command == entry.command &&
-      type == entry.type;
-  }
+  bool operator==(const SocketEntry& entry);
 };
 
 typedef deque<SocketEntry> SocketEntries;
@@ -81,14 +82,9 @@ public:
   Command* command;
 public:
   NameResolverEntry(const NameResolverHandle& nameResolver,
-		    Command* command):
-    nameResolver(nameResolver), command(command) {}
-  ~NameResolverEntry() {}
+		    Command* command);
 
-  bool operator==(const NameResolverEntry& entry) {
-    return nameResolver == entry.nameResolver &&
-      command == entry.command;
-  }
+  bool operator==(const NameResolverEntry& entry);
 };
 
 typedef deque<NameResolverEntry> NameResolverEntries;
@@ -104,18 +100,28 @@ private:
 #endif // ENABLE_ASYNC_DNS
   fd_set rfdset;
   fd_set wfdset;
-  int fdmax;
+  int32_t fdmax;
+
+  const Logger* logger;
+  
+  StatCalcHandle _statCalc;
+
+  bool _haltRequested;
 
   void shortSleep() const;
   bool addSocket(const SocketEntry& socketEntry);
   bool deleteSocket(const SocketEntry& socketEntry);
   void executeCommand(Command::STATUS statusFilter);
-protected:
-  const Logger* logger;
-  virtual void initStatistics() = 0;
-  virtual void calculateStatistics() = 0;
-  virtual void onEndOfRun() = 0;
-  virtual void afterEachIteration() {}
+
+  /**
+   * Delegates to StatCalc
+   */
+  void calculateStatistics();
+
+  void onEndOfRun();
+
+  void afterEachIteration();
+
 public:
   bool noWait;
   Commands commands;
@@ -148,11 +154,19 @@ public:
 			       Command* command);
 #endif // ENABLE_ASYNC_DNS
 
-  void addCommand(const Commands& commands)
+  void addCommand(const Commands& commands);
+
+  void fillCommand();
+
+  void setStatCalc(const StatCalcHandle& statCalc);
+
+  bool isHaltRequested() const
   {
-    this->commands.insert(this->commands.end(), commands.begin(), commands.end());
+    return _haltRequested;
   }
 };
 
+typedef SharedHandle<DownloadEngine> DownloadEngineHandle;
+
 #endif // _D_DOWNLOAD_ENGINE_H_
 
