@@ -34,15 +34,19 @@
 /* copyright --> */
 #include "File.h"
 #include "Util.h"
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
+#include "a2io.h"
+#include <libgen.h>
+
+#ifdef __MINGW32__
+# define WIN32_LEAN_AND_MEAN
+# include <windows.h>
+#endif // __MINGW32__
 
 File::File(const string& name):name(name) {}
 
 File::~File() {}
 
-int File::fillStat(struct stat& fstat) {
+int32_t File::fillStat(struct stat& fstat) {
   return stat(name.c_str(), &fstat);
 }
 
@@ -105,7 +109,7 @@ bool File::mkdirs() {
     if(File(accDir).isDir()) {
       continue;
     }
-    if(mkdir(accDir.c_str(), DIR_OPEN_MODE) == -1) {
+    if(a2mkdir(accDir.c_str(), DIR_OPEN_MODE) == -1) {
       return false;
     }
   }
@@ -120,3 +124,42 @@ mode_t File::mode()
   }
   return fstat.st_mode;
 }
+
+string File::getBasename() const
+{
+  char* s = strdup(name.c_str());
+  string bname = basename(s);
+  free(s);
+  return bname;
+}
+
+string File::getDirname() const
+{
+  char* s = strdup(name.c_str());
+  string dname = dirname(s);
+  free(s);
+  return dname;
+}
+
+bool File::isDir(const string& filename)
+{
+  return File(filename).isDir();
+}
+
+bool File::renameTo(const string& dest)
+{
+#ifdef __MINGW32__
+  /* MinGW's rename() doesn't delete an existing destination */
+  if (_access(dest.c_str(), 0) == 0) {
+    if (_unlink(dest.c_str()) != 0) {
+      return false;
+    }
+  }
+#endif // __MINGW32__
+  if(rename(name.c_str(), dest.c_str()) == 0) {
+    name = dest;
+    return true;
+  } else {
+    return false;
+  }
+}
