@@ -48,7 +48,11 @@ using namespace std;
 class File {
 private:
   string name;
-  int fillStat(struct stat& fstat);
+  
+  /**
+   * Returns the return value of stat(...)
+   */
+  int32_t fillStat(struct stat& fstat);
 public:
   File(const string& name);
   ~File();
@@ -86,6 +90,19 @@ public:
   int64_t size();
 
   mode_t mode();
+
+  string getBasename() const;
+
+  string getDirname() const;
+
+  const string& getPath() const
+  {
+    return name;
+  }
+
+  static bool isDir(const string& filename);
+
+  bool renameTo(const string& dest);
 };
 
 #endif // _D_FILE_H_
