@@ -34,31 +34,36 @@
 /* copyright --> */
 #include "FileEntry.h"
 #include "File.h"
-#include "DlAbortEx.h"
+#include "Util.h"
 #include <libgen.h>
 
 FileEntry::FileEntry(const string& path,
-		     long long int length,
-		     long long int offset):
-  path(path), length(length), offset(offset),
+		     int64_t length,
+		     int64_t offset,
+		     const Strings& uris):
+  path(path), _uris(uris), length(length), offset(offset),
   extracted(false), requested(true) {}
 
 FileEntry::~FileEntry() {}
 
-void FileEntry::setupDir(const string& parentDir) {
-  string absPath = parentDir+"/"+path;
-  char* temp = strdup(absPath.c_str());
-  string dir = string(dirname(temp));
-  free(temp);
-  if(!dir.size()) {
-    return;
+void FileEntry::setupDir(const string& parentDir)
+{
+  Util::mkdirs(File(parentDir+"/"+path).getDirname());
+}
+
+FileEntry& FileEntry::operator=(const FileEntry& entry)
+{
+  if(this != &entry) {
+    path = entry.path;
+    length = entry.length;
+    offset = entry.offset;
+    extracted = entry.extracted;
+    requested = entry.requested;
   }
-  File f(dir);
-  if(f.isDir()) {
-    // nothing to do
-  } else if(f.exists()) {
-    throw new DlAbortEx("%s is not a directory.", dir.c_str());
-  } else if(!f.mkdirs()) {
-    throw new DlAbortEx("Failed to create directory %s.", dir.c_str());
-  }  
+  return *this;
+}
+
+bool FileEntry::operator<(const FileEntry& fileEntry) const
+{
+  return offset < fileEntry.offset;
 }
