@@ -34,13 +34,15 @@
 /* copyright --> */
 #include "FtpConnection.h"
 #include "Util.h"
-#include "DlAbortEx.h"
-#include "DlRetryEx.h"
 #include "message.h"
 #include "prefs.h"
 #include "LogFactory.h"
+#include "AuthConfigFactory.h"
+#include "AuthConfig.h"
+#include "DlRetryEx.h"
+#include "DlAbortEx.h"
 
-FtpConnection::FtpConnection(int cuid, const SocketHandle& socket,
+FtpConnection::FtpConnection(int32_t cuid, const SocketHandle& socket,
 			     const RequestHandle req, const Option* op)
   :cuid(cuid), socket(socket), req(req), option(op) {
   logger = LogFactory::getInstance();
@@ -48,19 +50,22 @@ FtpConnection::FtpConnection(int cuid, const SocketHandle& socket,
 
 FtpConnection::~FtpConnection() {}
 
-void FtpConnection::sendUser() const {
-  string request = "USER "+req->resolveFtpAuthConfig()->getUser()+"\r\n";
+void FtpConnection::sendUser() const
+{
+  string request = "USER "+AuthConfigFactorySingleton::instance()->createAuthConfig(req)->getUser()+"\r\n";
   logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
   socket->writeData(request);
 }
 
-void FtpConnection::sendPass() const {
-  string request = "PASS "+req->resolveFtpAuthConfig()->getPassword()+"\r\n";
+void FtpConnection::sendPass() const
+{
+  string request = "PASS "+AuthConfigFactorySingleton::instance()->createAuthConfig(req)->getPassword()+"\r\n";
   logger->info(MSG_SENDING_REQUEST, cuid, "PASS ********");
   socket->writeData(request);
 }
 
-void FtpConnection::sendType() const {
+void FtpConnection::sendType() const
+{
   string type;
   if(option->get(PREF_FTP_TYPE) == V_ASCII) {
     type = "A";
@@ -72,31 +77,35 @@ void FtpConnection::sendType() const {
   socket->writeData(request);
 }
 
-void FtpConnection::sendCwd() const {
-  string request = "CWD "+req->getDir()+"\r\n";
+void FtpConnection::sendCwd() const
+{
+  string request = "CWD "+Util::urldecode(req->getDir())+"\r\n";
   logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
   socket->writeData(request);
 }
 
-void FtpConnection::sendSize() const {
-  string request = "SIZE "+req->getFile()+"\r\n";
+void FtpConnection::sendSize() const
+{
+  string request = "SIZE "+Util::urldecode(req->getFile())+"\r\n";
   logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
   socket->writeData(request);
 }
 
-void FtpConnection::sendPasv() const {
+void FtpConnection::sendPasv() const
+{
   string request = "PASV\r\n";
   logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
   socket->writeData(request);
 }
 
-SocketHandle FtpConnection::sendPort() const {
+SocketHandle FtpConnection::sendPort() const
+{
   SocketHandle serverSocket;
   serverSocket->beginListen();
   
-  pair<string, int> addrinfo;
+  pair<string, int32_t> addrinfo;
   socket->getAddrInfo(addrinfo);
-  int ipaddr[4]; 
+  int32_t ipaddr[4]; 
   sscanf(addrinfo.first.c_str(), "%d.%d.%d.%d",
 	 &ipaddr[0], &ipaddr[1], &ipaddr[2], &ipaddr[3]);
   serverSocket->getAddrInfo(addrinfo);
@@ -109,20 +118,23 @@ SocketHandle FtpConnection::sendPort() const {
   return serverSocket;
 }
 
-void FtpConnection::sendRest(const SegmentHandle& segment) const {
+void FtpConnection::sendRest(const SegmentHandle& segment) const
+{
   string request = "REST "+Util::llitos(segment->getPositionToWrite())+"\r\n";
   logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
   socket->writeData(request);
 }
 
-void FtpConnection::sendRetr() const {
-  string request = "RETR "+req->getFile()+"\r\n";
+void FtpConnection::sendRetr() const
+{
+  string request = "RETR "+Util::urldecode(req->getFile())+"\r\n";
   logger->info(MSG_SENDING_REQUEST, cuid, request.c_str());
   socket->writeData(request);
 }
 
-int FtpConnection::getStatus(const string& response) const {
-  int status;
+int32_t FtpConnection::getStatus(const string& response) const
+{
+  int32_t status;
   // When the response is not like "%d %*s",
   // we return 0.
   if(response.find_first_not_of("0123456789") != 3
@@ -136,7 +148,8 @@ int FtpConnection::getStatus(const string& response) const {
   }
 }
 
-bool FtpConnection::isEndOfResponse(int status, const string& response) const {
+bool FtpConnection::isEndOfResponse(int32_t status, const string& response) const
+{
   if(response.size() <= 4) {
     return false;
   }
@@ -156,10 +169,11 @@ bool FtpConnection::isEndOfResponse(int status, const string& response) const {
   }
 }
 
-bool FtpConnection::bulkReceiveResponse(pair<int, string>& response) {
+bool FtpConnection::bulkReceiveResponse(pair<int32_t, string>& response)
+{
   char buf[1024];  
   while(socket->isReadable(0)) {
-    int size = sizeof(buf)-1;
+    int32_t size = sizeof(buf)-1;
     socket->readData(buf, size);
     if(size == 0) {
       throw new DlRetryEx(EX_GOT_EOF);
@@ -167,11 +181,11 @@ bool FtpConnection::bulkReceiveResponse(pair<int, string>& response) {
     buf[size] = '\0';
     strbuf += buf;
   }
-  int status;
+  int32_t status;
   if(strbuf.size() >= 4) {
     status = getStatus(strbuf);
     if(status == 0) {
-      throw new DlRetryEx(EX_INVALID_RESPONSE);
+      throw new DlAbortEx(EX_INVALID_RESPONSE);
     }
   } else {
     return false;
@@ -188,8 +202,9 @@ bool FtpConnection::bulkReceiveResponse(pair<int, string>& response) {
   }
 }
 
-int FtpConnection::receiveResponse() {
-  pair<int, string> response;
+int32_t FtpConnection::receiveResponse()
+{
+  pair<int32_t, string> response;
   if(bulkReceiveResponse(response)) {
     return response.first;
   } else {
@@ -197,11 +212,12 @@ int FtpConnection::receiveResponse() {
   }
 }
 
-int FtpConnection::receiveSizeResponse(long long int& size) {
-  pair<int, string> response;
+int32_t FtpConnection::receiveSizeResponse(int64_t& size)
+{
+  pair<int32_t, string> response;
   if(bulkReceiveResponse(response)) {
     if(response.first == 213) {
-      sscanf(response.second.c_str(), "%*d %Ld", &size);
+      sscanf(response.second.c_str(), "%*d " LONGLONG_SCANF, &size);
     }
     return response.first;
   } else {
@@ -209,12 +225,13 @@ int FtpConnection::receiveSizeResponse(long long int& size) {
   }
 }
 
-int FtpConnection::receivePasvResponse(pair<string, int>& dest) {
-  pair<int, string> response;
+int32_t FtpConnection::receivePasvResponse(pair<string, int32_t>& dest)
+{
+  pair<int32_t, string> response;
   if(bulkReceiveResponse(response)) {
     if(response.first == 227) {
       // we assume the format of response is "227 Entering Passive Mode (h1,h2,h3,h4,p1,p2)."
-      int h1, h2, h3, h4, p1, p2;
+      int32_t h1, h2, h3, h4, p1, p2;
       string::size_type p = response.second.find("(");
       if(p >= 4) {
 	sscanf(response.second.substr(response.second.find("(")).c_str(),
