@@ -33,6 +33,11 @@
  */
 /* copyright --> */
 #include "FtpInitiateConnectionCommand.h"
+#include "NameResolver.h"
+#include "DownloadEngine.h"
+#include "RequestGroup.h"
+#include "Option.h"
+#include "Request.h"
 #include "FtpNegotiationCommand.h"
 #include "HttpRequestCommand.h"
 #include "FtpTunnelRequestCommand.h"
@@ -40,6 +45,7 @@
 #include "message.h"
 #include "prefs.h"
 #include "Util.h"
+#include "HttpConnection.h"
 
 FtpInitiateConnectionCommand::FtpInitiateConnectionCommand(int cuid,
 							   const RequestHandle& req,
@@ -83,7 +89,7 @@ bool FtpInitiateConnectionCommand::executeInternal() {
 				e->option->getAsInt(PREF_HTTP_PROXY_PORT));
     
     if(useHttpProxyGet()) {
-      command = new HttpRequestCommand(cuid, req, _requestGroup, e, socket);
+      command = new HttpRequestCommand(cuid, req, _requestGroup, new HttpConnection(cuid, socket, e->option), e, socket);
     } else if(useHttpProxyConnect()) {
       command = new FtpTunnelRequestCommand(cuid, req, _requestGroup, e, socket);
     } else {
@@ -111,3 +117,10 @@ bool FtpInitiateConnectionCommand::useHttpProxyGet() const {
 bool FtpInitiateConnectionCommand::useHttpProxyConnect() const {
   return useHttpProxy() && e->option->get(PREF_FTP_VIA_HTTP_PROXY) == V_TUNNEL;
 }
+
+#ifdef ENABLE_ASYNC_DNS
+bool FtpInitiateConnectionCommand::nameResolveFinished() const {
+  return nameResolver->getStatus() ==  NameResolver::STATUS_SUCCESS ||
+    nameResolver->getStatus() == NameResolver::STATUS_ERROR;
+}
+#endif // ENABLE_ASYNC_DNS
