@@ -33,15 +33,21 @@
  */
 /* copyright --> */
 #include "FtpNegotiationCommand.h"
+#include "DownloadEngine.h"
+#include "FtpConnection.h"
+#include "RequestGroup.h"
+#include "PieceStorage.h"
 #include "FtpDownloadCommand.h"
 #include "DlAbortEx.h"
-#include "DlRetryEx.h"
 #include "message.h"
 #include "prefs.h"
 #include "Util.h"
-#include "FatalException.h"
+#include "SingleFileDownloadContext.h"
+#include "DefaultBtProgressInfoFile.h"
+#include "RequestGroupMan.h"
+#include "DownloadFailureException.h"
 
-FtpNegotiationCommand::FtpNegotiationCommand(int cuid,
+FtpNegotiationCommand::FtpNegotiationCommand(int32_t cuid,
 					     const RequestHandle& req,
 					     RequestGroup* requestGroup,
 					     DownloadEngine* e,
@@ -58,7 +64,7 @@ FtpNegotiationCommand::~FtpNegotiationCommand() {
 }
 
 bool FtpNegotiationCommand::executeInternal() {
-  while(processSequence(segment));
+  while(processSequence(_segments.front()));
   if(sequence == SEQ_RETRY) {
     return prepareForRetry(0);
   } else if(sequence == SEQ_NEGOTIATION_COMPLETED) {
@@ -69,8 +75,15 @@ bool FtpNegotiationCommand::executeInternal() {
     command->setLowestDownloadSpeedLimit(e->option->getAsInt(PREF_LOWEST_SPEED_LIMIT));
     e->commands.push_back(command);
     return true;
-  } else if(sequence == SEQ_HEAD_OK || sequence == SEQ_DOWNLOAD_ALREADY_COMPLETED || sequence == SEQ_FILE_PREPARATION) {
+  } else if(sequence == SEQ_HEAD_OK || sequence == SEQ_DOWNLOAD_ALREADY_COMPLETED) {
     return true;
+  } else if(sequence == SEQ_FILE_PREPARATION) {
+    if(e->option->get(PREF_FTP_PASV) == V_TRUE) {
+      sequence = SEQ_SEND_PASV;
+    } else {
+      sequence = SEQ_SEND_PORT;
+    }
+    return false;
   } else {
     e->commands.push_back(this);
     return false;
@@ -82,12 +95,12 @@ bool FtpNegotiationCommand::recvGreeting() {
   disableWriteCheckSocket();
   setReadCheckSocket(socket);
 
-  int status = ftp->receiveResponse();
+  int32_t status = ftp->receiveResponse();
   if(status == 0) {
     return false;
   }
   if(status != 220) {
-    throw new DlRetryEx(EX_CONNECTION_FAILED);
+    throw new DlAbortEx(EX_CONNECTION_FAILED);
   }
   sequence = SEQ_SEND_USER;
 
@@ -101,7 +114,7 @@ bool FtpNegotiationCommand::sendUser() {
 }
 
 bool FtpNegotiationCommand::recvUser() {
-  int status = ftp->receiveResponse();
+  int32_t status = ftp->receiveResponse();
   switch(status) {
   case 0:
     return false;
@@ -112,7 +125,7 @@ bool FtpNegotiationCommand::recvUser() {
     sequence = SEQ_SEND_PASS;
     break;
   default:
-    throw new DlRetryEx(EX_BAD_STATUS, status);
+    throw new DlAbortEx(EX_BAD_STATUS, status);
   }
   return true;
 }
@@ -124,12 +137,12 @@ bool FtpNegotiationCommand::sendPass() {
 }
 
 bool FtpNegotiationCommand::recvPass() {
-  int status = ftp->receiveResponse();
+  int32_t status = ftp->receiveResponse();
   if(status == 0) {
     return false;
   }
   if(status != 230) {
-    throw new DlRetryEx(EX_BAD_STATUS, status);
+    throw new DlAbortEx(EX_BAD_STATUS, status);
   }
   sequence = SEQ_SEND_TYPE;
   return true;
@@ -142,12 +155,12 @@ bool FtpNegotiationCommand::sendType() {
 }
 
 bool FtpNegotiationCommand::recvType() {
-  int status = ftp->receiveResponse();
+  int32_t status = ftp->receiveResponse();
   if(status == 0) {
     return false;
   }
   if(status != 200) {
-    throw new DlRetryEx(EX_BAD_STATUS, status);
+    throw new DlAbortEx(EX_BAD_STATUS, status);
   }
   sequence = SEQ_SEND_CWD;
   return true;
@@ -160,12 +173,12 @@ bool FtpNegotiationCommand::sendCwd() {
 }
 
 bool FtpNegotiationCommand::recvCwd() {
-  int status = ftp->receiveResponse();
+  int32_t status = ftp->receiveResponse();
   if(status == 0) {
     return false;
   }
   if(status != 250) {
-    throw new DlRetryEx(EX_BAD_STATUS, status);
+    throw new DlAbortEx(EX_BAD_STATUS, status);
   }
   sequence = SEQ_SEND_SIZE;
   return true;
@@ -178,65 +191,52 @@ bool FtpNegotiationCommand::sendSize() {
 }
 
 bool FtpNegotiationCommand::recvSize() {
-  long long int size = 0;
-  int status = ftp->receiveSizeResponse(size);
+  int64_t size = 0;
+  int32_t status = ftp->receiveSizeResponse(size);
   if(status == 0) {
     return false;
   }
   if(status != 213) {
-    throw new DlRetryEx(EX_BAD_STATUS, status);
+    throw new DlAbortEx(EX_BAD_STATUS, status);
   }
   if(size == INT64_MAX || size < 0) {
-    throw new DlAbortEx(EX_TOO_LARGE_FILE, size);
+    throw new DlAbortEx(EX_TOO_LARGE_FILE, Util::llitos(size, true).c_str());
   }
-  if(!_requestGroup->getSegmentMan()->downloadStarted) {
-    _requestGroup->getSegmentMan()->downloadStarted = true;
-    _requestGroup->getSegmentMan()->totalSize = size;
-    _requestGroup->getSegmentMan()->filename = Util::urldecode(req->getFile());
-
-    // TODO validate filename and totalsize against hintFilename and hintTotalSize if these are provided.
-    _requestGroup->validateTotalLengthByHint(size);
-
+  if(_requestGroup->getPieceStorage().isNull()) {
+    SingleFileDownloadContextHandle dctx = _requestGroup->getDownloadContext();
+    dctx->setTotalLength(size);
+    dctx->setFilename(Util::urldecode(req->getFile()));
+    _requestGroup->preDownloadProcessing();
     if(e->_requestGroupMan->isSameFileBeingDownloaded(_requestGroup)) {
-      throw new FatalException(EX_DUPLICATE_FILE_DOWNLOAD, _requestGroup->getFilePath().c_str());
+      throw new DownloadFailureException(EX_DUPLICATE_FILE_DOWNLOAD,
+					 _requestGroup->getFilePath().c_str());
     }
 
+    _requestGroup->initPieceStorage();
+
+    // TODO Is this really necessary?
     if(req->getMethod() == Request::METHOD_HEAD) {
-      _requestGroup->getSegmentMan()->isSplittable = false; // TODO because we don't want segment file to be saved.
       sequence = SEQ_HEAD_OK;
       return false;
     }
 
-    if(e->option->get(PREF_CHECK_INTEGRITY) != V_TRUE) {
-      if(_requestGroup->downloadFinishedByFileLength()) {
-	logger->notice(MSG_DOWNLOAD_ALREADY_COMPLETED, cuid, _requestGroup->getFilePath().c_str());
-	sequence = SEQ_DOWNLOAD_ALREADY_COMPLETED;
-	return false;
-      }
+    BtProgressInfoFileHandle infoFile = new DefaultBtProgressInfoFile(_requestGroup->getDownloadContext(), _requestGroup->getPieceStorage(), e->option);
+    if(!infoFile->exists() && _requestGroup->downloadFinishedByFileLength()) {
+      sequence = SEQ_DOWNLOAD_ALREADY_COMPLETED;
+      return false;
     }
-    _requestGroup->loadAndOpenFile();
-    _requestGroup->prepareForNextAction(cuid, req, e);
+    _requestGroup->loadAndOpenFile(infoFile);
+
+    prepareForNextAction(this);
     
     sequence = SEQ_FILE_PREPARATION;
-    e->noWait = true;
+    disableReadCheckSocket();
+    setWriteCheckSocket(dataSocket);
+
+    //e->noWait = true;
     return false;
-
-    /*
-    _requestGroup->getSegmentMan()->initBitfield(e->option->getAsInt(PREF_SEGMENT_SIZE),
-						 _requestGroup->getSegmentMan()->totalSize);
-    bool segFileExists = _requestGroup->getSegmentMan()->segmentFileExists();
-    if(segFileExists) {
-      _requestGroup->getSegmentMan()->load();
-      _requestGroup->getSegmentMan()->diskWriter->openExistingFile(_requestGroup->getSegmentMan()->getFilePath());
-    } else {
-      _requestGroup->getSegmentMan()->diskWriter->initAndOpenFile(_requestGroup->getSegmentMan()->getFilePath(), size);
-    }
-    */
-
   } else {
     _requestGroup->validateTotalLength(size);
-    //if(_requestGroup->getSegmentMan()->totalSize != size) {
-    //throw new DlAbortEx(EX_SIZE_MISMATCH, _requestGroup->getSegmentMan()->totalSize, size);
   }
   if(e->option->get(PREF_FTP_PASV) == V_TRUE) {
     sequence = SEQ_SEND_PASV;
@@ -253,12 +253,12 @@ bool FtpNegotiationCommand::sendPort() {
 }
 
 bool FtpNegotiationCommand::recvPort() {
-  int status = ftp->receiveResponse();
+  int32_t status = ftp->receiveResponse();
   if(status == 0) {
     return false;
   }
   if(status != 200) {
-    throw new DlRetryEx(EX_BAD_STATUS, status);
+    throw new DlAbortEx(EX_BAD_STATUS, status);
   }
   sequence = SEQ_SEND_REST;
   return true;
@@ -271,13 +271,13 @@ bool FtpNegotiationCommand::sendPasv() {
 }
 
 bool FtpNegotiationCommand::recvPasv() {
-  pair<string, int> dest;
-  int status = ftp->receivePasvResponse(dest);
+  pair<string, int32_t> dest;
+  int32_t status = ftp->receivePasvResponse(dest);
   if(status == 0) {
     return false;
   }
   if(status != 227) {
-    throw new DlRetryEx(EX_BAD_STATUS, status);
+    throw new DlAbortEx(EX_BAD_STATUS, status);
   }
   // make a data connection to the server.
   logger->info(MSG_CONNECTING_TO_SERVER, cuid,
@@ -306,13 +306,13 @@ bool FtpNegotiationCommand::sendRest(const SegmentHandle& segment) {
 }
 
 bool FtpNegotiationCommand::recvRest() {
-  int status = ftp->receiveResponse();
+  int32_t status = ftp->receiveResponse();
   if(status == 0) {
     return false;
   }
   // TODO if we recieve negative response, then we set _requestGroup->getSegmentMan()->splittable = false, and continue.
   if(status != 350) {
-    throw new DlRetryEx(EX_BAD_STATUS, status);
+    throw new DlAbortEx(EX_BAD_STATUS, status);
   }
   sequence = SEQ_SEND_RETR;
   return true;
@@ -325,12 +325,12 @@ bool FtpNegotiationCommand::sendRetr() {
 }
 
 bool FtpNegotiationCommand::recvRetr() {
-  int status = ftp->receiveResponse();
+  int32_t status = ftp->receiveResponse();
   if(status == 0) {
     return false;
   }
   if(status != 150 && status != 125) {
-    throw new DlRetryEx(EX_BAD_STATUS, status);
+    throw new DlAbortEx(EX_BAD_STATUS, status);
   }
   if(e->option->get(PREF_FTP_PASV) != V_TRUE) {
     assert(serverSocket->getSockfd() != -1);
