@@ -36,7 +36,8 @@
 #define _D_FTP_NEGOTIATION_COMMAND_H_
 
 #include "AbstractCommand.h"
-#include "FtpConnection.h"
+
+class FtpConnection;
 
 class FtpNegotiationCommand : public AbstractCommand {
 private:
@@ -91,12 +92,12 @@ private:
 
   SocketHandle dataSocket;
   SocketHandle serverSocket;
-  int sequence;
+  int32_t sequence;
   FtpConnection* ftp;
 protected:
   virtual bool executeInternal();
 public:
-  FtpNegotiationCommand(int cuid,
+  FtpNegotiationCommand(int32_t cuid,
 			const RequestHandle& req,
 			RequestGroup* requestGroup,
 			DownloadEngine* e,
