@@ -43,26 +43,46 @@
 #include "common.h"
 #include "Logger.h"
 #include "HttpResponse.h"
-#include <netinet/in.h>
-#include <string>
+#include "HttpHeaderProcessor.h"
 
-#define HEADERBUF_SIZE 4096
+class HttpRequestEntry {
+private:
+  HttpRequestHandle _httpRequest;
+  HttpHeaderProcessorHandle _proc;
+public:
+  HttpRequestEntry(const HttpRequestHandle& httpRequest,
+		   const HttpHeaderProcessorHandle& proc):
+    _httpRequest(httpRequest),
+    _proc(proc) {}
+
+  ~HttpRequestEntry() {}
+
+  HttpRequestHandle getHttpRequest() const
+  {
+    return _httpRequest;
+  }
+
+  HttpHeaderProcessorHandle getHttpHeaderProcessor() const
+  {
+    return _proc;
+  }
+};
+
+typedef SharedHandle<HttpRequestEntry> HttpRequestEntryHandle;
+typedef deque<HttpRequestEntryHandle> HttpRequestEntries;
 
 class HttpConnection {
 private:
-  int cuid;
+  int32_t cuid;
   SocketHandle socket;
   const Option* option;
   const Logger* logger;
-  char headerBuf[HEADERBUF_SIZE+1];
-  int headerBufLength;
 
-  HttpRequests outstandingHttpRequests;
+  HttpRequestEntries outstandingHttpRequests;
 
-  int findEndOfHeader(const char* buf, const char* substr, int bufLength) const;
   string eraseConfidentialInfo(const string& request);
 public:
-  HttpConnection(int cuid,
+  HttpConnection(int32_t cuid,
 		 const SocketHandle& socket,
 		 const Option* op);
 
@@ -96,11 +116,13 @@ public:
   HttpRequestHandle getFirstHttpRequest() const
   {
     if(outstandingHttpRequests.size() > 0) {
-      return outstandingHttpRequests.front();
+      return outstandingHttpRequests.front()->getHttpRequest();
     } else {
       return 0;
     }
   }
+
+  bool isIssued(const SegmentHandle& segment) const;
 };
 
 typedef SharedHandle<HttpConnection> HttpConnectionHandle;
