@@ -33,6 +33,10 @@
  */
 /* copyright --> */
 #include "HttpInitiateConnectionCommand.h"
+#include "NameResolver.h"
+#include "DownloadEngine.h"
+#include "Option.h"
+#include "Request.h"
 #include "HttpRequestCommand.h"
 #include "HttpProxyRequestCommand.h"
 #include "Util.h"
@@ -40,6 +44,7 @@
 #include "DlRetryEx.h"
 #include "message.h"
 #include "prefs.h"
+#include "HttpConnection.h"
 
 HttpInitiateConnectionCommand::HttpInitiateConnectionCommand(int cuid,
 							     const RequestHandle& req,
@@ -84,7 +89,7 @@ bool HttpInitiateConnectionCommand::executeInternal() {
     if(useProxyTunnel()) {
       command = new HttpProxyRequestCommand(cuid, req, _requestGroup, e, socket);
     } else if(useProxyGet()) {
-      command = new HttpRequestCommand(cuid, req, _requestGroup, e, socket);
+      command = new HttpRequestCommand(cuid, req, _requestGroup, new HttpConnection(cuid, socket, e->option), e, socket);
     } else {
       // TODO
       throw new DlAbortEx("ERROR");
@@ -93,7 +98,7 @@ bool HttpInitiateConnectionCommand::executeInternal() {
     logger->info(MSG_CONNECTING_TO_SERVER, cuid, req->getHost().c_str(),
 		 req->getPort());
     socket->establishConnection(hostname, req->getPort());
-    command = new HttpRequestCommand(cuid, req, _requestGroup, e, socket);
+    command = new HttpRequestCommand(cuid, req, _requestGroup, new HttpConnection(cuid, socket, e->option), e, socket);
   }
   e->commands.push_back(command);
   return true;
@@ -110,3 +115,10 @@ bool HttpInitiateConnectionCommand::useProxyGet() {
 bool HttpInitiateConnectionCommand::useProxyTunnel() {
   return e->option->get(PREF_HTTP_PROXY_METHOD) == V_TUNNEL;
 }
+
+#ifdef ENABLE_ASYNC_DNS
+bool HttpInitiateConnectionCommand::nameResolveFinished() const {
+  return nameResolver->getStatus() ==  NameResolver::STATUS_SUCCESS ||
+    nameResolver->getStatus() == NameResolver::STATUS_ERROR;
+}
+#endif // ENABLE_ASYNC_DNS
