@@ -36,11 +36,13 @@
 #include "Util.h"
 #include "Base64.h"
 #include "prefs.h"
+#include "AuthConfigFactory.h"
+#include "AuthConfig.h"
 
 RangeHandle HttpRequest::getRange() const
 {
   // content-length is always 0
-  if(segment->isNull()) {
+  if(segment.isNull()) {
     return new Range(0, 0, 0);
   } else {
     return new Range(getStartByte(), getEndByte(), entityLength);
@@ -49,7 +51,7 @@ RangeHandle HttpRequest::getRange() const
 
 bool HttpRequest::isRangeSatisfied(const RangeHandle& range) const
 {
-  if(segment->isNull()) {
+  if(segment.isNull()) {
     return true;
   }
   if(getStartByte() == range->getStartByte() &&
@@ -63,7 +65,7 @@ bool HttpRequest::isRangeSatisfied(const RangeHandle& range) const
   }  
 }
 
-string HttpRequest::getHostText(const string& host, in_port_t port) const
+string HttpRequest::getHostText(const string& host, int32_t port) const
 {
   return  host+(port == 80 || port == 443 ? "" : ":"+Util::llitos(port));
 }
@@ -83,7 +85,7 @@ string HttpRequest::createRequest() const
   }
   requestLine +=
     string(" HTTP/1.1\r\n")+
-    "User-Agent: "+Util::urlencode(userAgent)+"\r\n"+
+    "User-Agent: "+userAgent+"\r\n"+
     "Accept: */*\r\n"+        /* */
     "Host: "+getHostText(getHost(), getPort())+"\r\n"+
     "Pragma: no-cache\r\n"+
@@ -91,7 +93,8 @@ string HttpRequest::createRequest() const
   if(!request->isKeepAlive()) {
     requestLine += "Connection: close\r\n";
   }
-  if(segment->length > 0) {
+  if(!segment.isNull() && segment->getLength() > 0 &&
+     (request->isKeepAlive() || getStartByte() > 0)) {
     requestLine += "Range: bytes="+Util::llitos(getStartByte());
     requestLine += "-";
     if(request->isKeepAlive()) {
@@ -100,19 +103,22 @@ string HttpRequest::createRequest() const
     requestLine += "\r\n";
   }
   if(proxyEnabled) {
-    requestLine += "Proxy-Connection: close\r\n";
+    if(request->isKeepAlive()) {
+      requestLine += "Proxy-Connection: Keep-Alive\r\n";
+    } else {
+      requestLine += "Proxy-Connection: close\r\n";
+    }
   }
   if(proxyEnabled && proxyAuthEnabled) {
     requestLine += getProxyAuthString();
   }
   if(authEnabled) {
     requestLine += "Authorization: Basic "+
-	Base64::encode(request->resolveHttpAuthConfig()->getAuthText())+"\r\n";
+      Base64::encode(AuthConfigFactorySingleton::instance()->createAuthConfig(request)->getAuthText())+"\r\n";
   }
   if(getPreviousURI().size()) {
     requestLine += "Referer: "+getPreviousURI()+"\r\n";
   }
-
   string cookiesValue;
   Cookies cookies = request->cookieBox->criteriaFind(getHost(),
 						     getDir(),
@@ -134,9 +140,13 @@ string HttpRequest::createProxyRequest() const
   string requestLine =
     string("CONNECT ")+getHost()+":"+Util::itos(getPort())+
     string(" HTTP/1.1\r\n")+
-    "User-Agent: "+Util::urlencode(userAgent)+"\r\n"+
-    "Proxy-Connection: close\r\n"+
+    "User-Agent: "+userAgent+"\r\n"+
     "Host: "+getHost()+":"+Util::itos(getPort())+"\r\n";
+  if(request->isKeepAlive()) {
+    requestLine += "Proxy-Connection: Keep-Alive\r\n";
+  }else {
+    requestLine += "Proxy-Connection: close\r\n";
+  }
   if(proxyAuthEnabled) {
     requestLine += getProxyAuthString();
   }
@@ -146,7 +156,7 @@ string HttpRequest::createProxyRequest() const
 
 string HttpRequest::getProxyAuthString() const {
   return "Proxy-Authorization: Basic "+
-    Base64::encode(request->resolveHttpProxyAuthConfig()->getAuthText())+"\r\n";
+    Base64::encode(AuthConfigFactorySingleton::instance()->createAuthConfigForHttpProxy(request)->getAuthText())+"\r\n";
 }
 
 void HttpRequest::configure(const Option* option)
