@@ -40,7 +40,6 @@
 #include "Range.h"
 #include "Request.h"
 #include "Option.h"
-#include <netinet/in.h>
 
 class HttpRequest {
 private:
@@ -59,7 +58,7 @@ private:
 
   string userAgent;
 
-  string getHostText(const string& host, in_port_t port) const;
+  string getHostText(const string& host, int32_t port) const;
 
   string getProxyAuthString() const;
 
@@ -106,7 +105,7 @@ public:
     return request->getHost();
   }
 
-  in_port_t getPort() const
+  int32_t getPort() const
   {
     return request->getPort();
   }
@@ -169,7 +168,7 @@ public:
       return 0;
     } else {
       if(request->isKeepAlive()) {
-	return segment->getPosition()+segment->length-1;
+	return segment->getPosition()+segment->getLength()-1;
       } else {
 	return 0;
       }
