@@ -33,15 +33,23 @@
  */
 /* copyright --> */
 #include "HttpResponseCommand.h"
+#include "DownloadEngine.h"
+#include "HttpResponse.h"
+#include "HttpConnection.h"
+#include "SegmentMan.h"
 #include "DlAbortEx.h"
-#include "DlRetryEx.h"
 #include "HttpDownloadCommand.h"
 #include "message.h"
 #include "Util.h"
 #include "prefs.h"
 #include "File.h"
 #include "InitiateConnectionCommandFactory.h"
-#include "FatalException.h"
+#include "SingleFileDownloadContext.h"
+#include "DiskAdaptor.h"
+#include "PieceStorage.h"
+#include "DefaultBtProgressInfoFile.h"
+#include "RequestGroupMan.h"
+#include "DownloadFailureException.h"
 #include <sys/types.h>
 #include <unistd.h>
 
@@ -59,10 +67,6 @@ HttpResponseCommand::~HttpResponseCommand() {}
 bool HttpResponseCommand::executeInternal()
 {
   HttpRequestHandle httpRequest = httpConnection->getFirstHttpRequest();
-  if(!(httpRequest->getSegment() == segment)) {
-    logger->info(MSG_SEGMENT_CHANGED, cuid);
-    return prepareForRetry(0);
-  }
   HttpResponseHandle httpResponse = httpConnection->receiveResponse();
   if(httpResponse.isNull()) {
     // The server has not responded to our request yet.
@@ -72,12 +76,6 @@ bool HttpResponseCommand::executeInternal()
   // check HTTP status number
   httpResponse->validateResponse();
   httpResponse->retrieveCookie();
-  // check whether the server supports persistent connections.
-  /*
-  if(Util::toLower(headers.getFirst("Connection")).find("close") != string::npos) {
-    req->setKeepAlive(false);
-  }
-  */
   // check whether Location header exists. If it does, update request object
   // with redirected URL.
   // then establish a connection to the new host and port
@@ -87,86 +85,79 @@ bool HttpResponseCommand::executeInternal()
     e->noWait = true;
     return prepareForRetry(0);
   }
-  if(_requestGroup->getSegmentMan()->downloadStarted) {
-    // validate totalsize
-    _requestGroup->validateFilename(httpResponse->determinFilename());
-    _requestGroup->validateTotalLength(httpResponse->getEntityLength());
-
-    e->commands.push_back(createHttpDownloadCommand(httpResponse));
-    return true;
-  } else {
-    // validate totalsize against hintTotalSize if it is provided.
-    _requestGroup->validateFilenameByHint(httpResponse->determinFilename());
-    _requestGroup->validateTotalLengthByHint(httpResponse->getEntityLength());
-
-    _requestGroup->getSegmentMan()->filename = httpResponse->determinFilename();
+  if(_requestGroup->getPieceStorage().isNull()) {
+    int64_t totalLength = httpResponse->getEntityLength();
+    SingleFileDownloadContextHandle dctx = _requestGroup->getDownloadContext();
+    dctx->setTotalLength(totalLength);
+    dctx->setFilename(httpResponse->determinFilename());
+    dctx->setContentType(httpResponse->getContentType());
+    _requestGroup->preDownloadProcessing();
     if(e->_requestGroupMan->isSameFileBeingDownloaded(_requestGroup)) {
-      throw new FatalException(EX_DUPLICATE_FILE_DOWNLOAD, _requestGroup->getFilePath().c_str());
+      throw new DownloadFailureException(EX_DUPLICATE_FILE_DOWNLOAD,
+					 _requestGroup->getFilePath().c_str());
     }
-
-    if(httpResponse->isTransferEncodingSpecified()) {
+    if(totalLength == 0 || httpResponse->isTransferEncodingSpecified()) {
+      // we ignore content-length when transfer-encoding is set
+      dctx->setTotalLength(0);
       return handleOtherEncoding(httpResponse);
     } else {
       return handleDefaultEncoding(httpResponse);
     }
+  } else {
+    // validate totalsize
+    _requestGroup->validateTotalLength(httpResponse->getEntityLength());
+    e->commands.push_back(createHttpDownloadCommand(httpResponse));
+    return true;
   }
 }
 
 bool HttpResponseCommand::handleDefaultEncoding(const HttpResponseHandle& httpResponse)
 {
   HttpRequestHandle httpRequest = httpResponse->getHttpRequest();
-  // TODO quick and dirty way 
-  if(_requestGroup->isTorrent) {
-    return doTorrentStuff(httpResponse);
-  }
-  int64_t size = httpResponse->getEntityLength();
-  if(size == INT64_MAX || size < 0) {
-    throw new DlAbortEx(EX_TOO_LARGE_FILE, size);
-  }
-  _requestGroup->getSegmentMan()->isSplittable = !(size == 0);
-  _requestGroup->getSegmentMan()->downloadStarted = true;
-  _requestGroup->getSegmentMan()->totalSize = size;
-  
-  // quick hack for method 'head'
+  _requestGroup->initPieceStorage();
+
+  // quick hack for method 'head',, is it necessary?
   if(httpRequest->getMethod() == Request::METHOD_HEAD) {
     // TODO because we don't want segment file to be saved.
-    _requestGroup->getSegmentMan()->isSplittable = false;
     return true;
   }
 
-  if(e->option->get(PREF_CHECK_INTEGRITY) != V_TRUE) {
-    if(_requestGroup->downloadFinishedByFileLength()) {
-      logger->notice(MSG_DOWNLOAD_ALREADY_COMPLETED, cuid, _requestGroup->getFilePath().c_str());
-      return true;
-    }
+  BtProgressInfoFileHandle infoFile = new DefaultBtProgressInfoFile(_requestGroup->getDownloadContext(), _requestGroup->getPieceStorage(), e->option);
+  if(!infoFile->exists() && _requestGroup->downloadFinishedByFileLength()) {
+    return true;
   }
 
   DownloadCommand* command = 0;
-  File file(_requestGroup->getFilePath());
-  if(_requestGroup->getRemainingUris().empty() && !file.exists()) {
-    command = createHttpDownloadCommand(httpResponse);
+  try {
+    _requestGroup->loadAndOpenFile(infoFile);
+    File file(_requestGroup->getFilePath());
+
+    SegmentHandle segment = _requestGroup->getSegmentMan()->getSegment(cuid, 0);
+    if(!segment.isNull() && segment->getPositionToWrite() == 0) {
+      command = createHttpDownloadCommand(httpResponse);
+    } else {
+      _requestGroup->getSegmentMan()->cancelSegment(cuid);
+    }
+    prepareForNextAction(command);
+    e->noWait = true;
+  } catch(Exception* e) {
+    delete command;
+    throw;
   }
-  _requestGroup->loadAndOpenFile();
-  _requestGroup->prepareForNextAction(cuid, req, e, command);
-  e->noWait = true;
   return true;
 }
 
 bool HttpResponseCommand::handleOtherEncoding(const HttpResponseHandle& httpResponse) {
   HttpRequestHandle httpRequest = httpResponse->getHttpRequest();
-  // we ignore content-length when transfer-encoding is set
-  _requestGroup->getSegmentMan()->downloadStarted = true;
-  _requestGroup->getSegmentMan()->isSplittable = false;
-  _requestGroup->getSegmentMan()->totalSize = 0;
-  // quick hack for method 'head'
+  // quick hack for method 'head',, is it necessary?
   if(httpRequest->getMethod() == Request::METHOD_HEAD) {
     return true;
   }
   // disable keep-alive
   req->setKeepAlive(false);
-  segment = _requestGroup->getSegmentMan()->getSegment(cuid);	
+  _requestGroup->initPieceStorage();
   _requestGroup->shouldCancelDownloadForSafety();
-  _requestGroup->getSegmentMan()->diskWriter->initAndOpenFile(_requestGroup->getSegmentMan()->getFilePath());
+  _requestGroup->getPieceStorage()->getDiskAdaptor()->initAndOpenFile();
   e->commands.push_back(createHttpDownloadCommand(httpResponse));
   return true;
 }
@@ -183,7 +174,7 @@ HttpDownloadCommand* HttpResponseCommand::createHttpDownloadCommand(const HttpRe
     enc->init();
   }
   HttpDownloadCommand* command =
-    new HttpDownloadCommand(cuid, req, _requestGroup, e, socket);
+    new HttpDownloadCommand(cuid, req, _requestGroup, httpConnection, e, socket);
   command->setMaxDownloadSpeedLimit(e->option->getAsInt(PREF_MAX_DOWNLOAD_LIMIT));
   command->setStartupIdleTime(e->option->getAsInt(PREF_STARTUP_IDLE_TIME));
   command->setLowestDownloadSpeedLimit(e->option->getAsInt(PREF_LOWEST_SPEED_LIMIT));
@@ -191,20 +182,3 @@ HttpDownloadCommand* HttpResponseCommand::createHttpDownloadCommand(const HttpRe
 
   return command;
 }
-
-bool HttpResponseCommand::doTorrentStuff(const HttpResponseHandle& httpResponse)
-{
-  int64_t size = httpResponse->getEntityLength();
-  _requestGroup->getSegmentMan()->totalSize = size;
-  if(size > 0) {
-    _requestGroup->getSegmentMan()->initBitfield(e->option->getAsInt(PREF_SEGMENT_SIZE),
-			      _requestGroup->getSegmentMan()->totalSize);
-  }
-  // disable keep-alive
-  httpResponse->getHttpRequest()->getRequest()->setKeepAlive(false);
-  _requestGroup->getSegmentMan()->isSplittable = false;
-  _requestGroup->getSegmentMan()->downloadStarted = true;
-  _requestGroup->getSegmentMan()->diskWriter->initAndOpenFile("/tmp/aria2"+Util::itos((int32_t)getpid()));
-  e->commands.push_back(createHttpDownloadCommand(httpResponse));
-  return true;
-}
