@@ -36,14 +36,14 @@
 #include "File.h"
 #include "DlAbortEx.h"
 #include "message.h"
+#include "a2io.h"
 #include <string.h>
-#include <stdlib.h>
 
 MetaEntry* MetaFileUtil::parseMetaFile(const string& file) {
   File f(file);
-  int len = f.size();
+  int32_t len = f.size();
   char* buf = new char[len];
-  FILE* fp = fopen(file.c_str(), "r+");
+  FILE* fp = fopen(file.c_str(), "r+b");
   try {
     if(fp == NULL) {
       throw new DlAbortEx("cannot open metainfo file");
@@ -66,14 +66,14 @@ MetaEntry* MetaFileUtil::parseMetaFile(const string& file) {
   }
 }
 
-MetaEntry* MetaFileUtil::bdecoding(const char* buf, int len) {
+MetaEntry* MetaFileUtil::bdecoding(const char* buf, int32_t len) {
   MetaEntry* entry = NULL;
   try{
     const char* p = buf;
     const char* end = buf+len;
     entry = bdecodingR(&p, end);
     return entry;
-  } catch(DlAbortEx* ex) {
+  } catch(RecoverableException* ex) {
     if(entry != NULL) {
       delete entry;
     }
@@ -157,7 +157,7 @@ Data* MetaFileUtil::decodeInt(const char** pp, const char* end) {
   if(endTerm == NULL) {
     throw new DlAbortEx(EX_MULFORMED_META_INFO);
   }
-  int numSize = endTerm-*pp;
+  int32_t numSize = endTerm-*pp;
 
   Data* data = new Data(*pp, numSize, true);
   *pp += numSize+1;
@@ -173,12 +173,12 @@ Data* MetaFileUtil::decodeWord(const char** pp, const char* end) {
   if(delim == *pp || delim == NULL) {
     throw new DlAbortEx(EX_MULFORMED_META_INFO);
   }
-  int numSize = delim-*pp;
+  int32_t numSize = delim-*pp;
   char* temp = new char[numSize+1];
   memcpy(temp, *pp, numSize);
   temp[numSize] = '\0';
   char* endptr;
-  int size = strtol(temp, &endptr, 10);
+  int32_t size = strtol(temp, &endptr, 10);
   if(*endptr != '\0') {
     delete [] temp;
     throw new DlAbortEx(EX_MULFORMED_META_INFO);
