@@ -37,7 +37,8 @@
 #include <algorithm>
 
 MetalinkEntry::MetalinkEntry():
-  size(0)
+  file(0),
+  maxConnections(-1)
 #ifdef ENABLE_MESSAGE_DIGEST
   ,
   checksum(0),
@@ -49,22 +50,28 @@ MetalinkEntry::~MetalinkEntry() {}
 
 class AddLocationPreference {
 private:
-  string location;
-  int preferenceToAdd;
+  Strings _locations;
+  int32_t _preferenceToAdd;
 public:
-  AddLocationPreference(const string& location, int preferenceToAdd):
-    location(location), preferenceToAdd(preferenceToAdd) {}
+  AddLocationPreference(const Strings& locations, int32_t preferenceToAdd):
+    _locations(locations), _preferenceToAdd(preferenceToAdd)
+  {
+    transform(_locations.begin(), _locations.end(), _locations.begin(), Util::toUpper);
+    sort(_locations.begin(), _locations.end());
+  }
 
   void operator()(MetalinkResourceHandle& res) {
-    if(res->location == location) {
-      res->preference += preferenceToAdd;
+    if(binary_search(_locations.begin(), _locations.end(), res->location)) {
+      res->preference += _preferenceToAdd;
     }
   }
 };
 
-void MetalinkEntry::setLocationPreference(const string& location, int preferenceToAdd) {
+void MetalinkEntry::setLocationPreference(const Strings& locations,
+					  int32_t preferenceToAdd)
+{
   for_each(resources.begin(), resources.end(),
-	   AddLocationPreference(location, preferenceToAdd));
+	   AddLocationPreference(locations, preferenceToAdd));
 }
 
 class PrefOrder {
@@ -86,7 +93,9 @@ public:
     switch(res->type) {
     case MetalinkResource::TYPE_FTP:
     case MetalinkResource::TYPE_HTTP:
+#ifdef ENABLE_SSL
     case MetalinkResource::TYPE_HTTPS:
+#endif // ENABLE_SSL
 #ifdef ENABLE_BITTORRENT
     case MetalinkResource::TYPE_BITTORRENT:
 #endif // ENABLE_BITTORRENT
@@ -102,3 +111,14 @@ void MetalinkEntry::dropUnsupportedResource() {
     partition(resources.begin(), resources.end(), Supported());
   resources.erase(split, resources.end());
 }
+
+FileEntries MetalinkEntry::toFileEntry(const MetalinkEntries& metalinkEntries)
+{
+  FileEntries entries;
+  for(MetalinkEntries::const_iterator itr = metalinkEntries.begin();
+      itr != metalinkEntries.end(); ++itr) {
+    entries.push_back((*itr)->file);
+  }
+  return entries;
+}
+
