@@ -37,50 +37,67 @@
 
 #include "common.h"
 #include "MetalinkResource.h"
-#include "Checksum.h"
-#include "MetalinkChunkChecksum.h"
+#include "FileEntry.h"
+#ifdef ENABLE_MESSAGE_DIGEST
+# include "Checksum.h"
+# include "ChunkChecksum.h"
+#endif // ENABLE_MESSAGE_DIGEST
 #include <deque>
 
+class MetalinkEntry;
+
+typedef SharedHandle<MetalinkEntry> MetalinkEntryHandle;
+typedef deque<MetalinkEntryHandle> MetalinkEntries;
+
 class MetalinkEntry {
 public:
-  string filename;
+  FileEntryHandle file;
   string version;
   string language;
   string os;
-  int64_t size;
-  ChecksumHandle checksum;
-public:
   MetalinkResources resources;
+  int32_t maxConnections;
 #ifdef ENABLE_MESSAGE_DIGEST
-  MetalinkChunkChecksumHandle chunkChecksum;
+  ChecksumHandle checksum;
+  ChunkChecksumHandle chunkChecksum;
 #endif // ENABLE_MESSAGE_DIGEST
 public:
   MetalinkEntry();
+
   ~MetalinkEntry();
 
   MetalinkEntry& operator=(const MetalinkEntry& metalinkEntry) {
     if(this != &metalinkEntry) {
-      this->filename = metalinkEntry.filename;
+      this->file = metalinkEntry.file;
       this->version = metalinkEntry.version;
       this->language = metalinkEntry.language;
       this->os = metalinkEntry.os;
-      this->size = metalinkEntry.size;
-      this->checksum = metalinkEntry.checksum;
+      this->maxConnections = metalinkEntry.maxConnections;
 #ifdef ENABLE_MESSAGE_DIGEST
+      this->checksum = metalinkEntry.checksum;
       this->chunkChecksum = metalinkEntry.chunkChecksum;
 #endif // ENABLE_MESSAGE_DIGEST
     }
     return *this;
   }
 
+  string getPath() const
+  {
+    return file->getPath();
+  }
+
+  int64_t getLength() const
+  {
+    return file->getLength();
+  }
+
   void dropUnsupportedResource();
 
   void reorderResourcesByPreference();
   
-  void setLocationPreference(const string& location, int preferenceToAdd);
+  void setLocationPreference(const Strings& locations, int32_t preferenceToAdd);
+
+  static FileEntries toFileEntry(const MetalinkEntries& metalinkEntries);
 };
 
-typedef SharedHandle<MetalinkEntry> MetalinkEntryHandle;
-typedef deque<MetalinkEntryHandle> MetalinkEntries;
-
 #endif // _D_METALINK_ENTRY_H_
