@@ -34,37 +34,43 @@
 /* copyright --> */
 #include "MultiDiskAdaptor.h"
 #include "DefaultDiskWriter.h"
-#include "DlAbortEx.h"
 #include "message.h"
 #include "Util.h"
-#include <errno.h>
+#include "MultiFileAllocationIterator.h"
+#include "DefaultDiskWriterFactory.h"
+#include "DlAbortEx.h"
 
-void MultiDiskAdaptor::resetDiskWriterEntries() {
+void MultiDiskAdaptor::resetDiskWriterEntries()
+{
   diskWriterEntries.clear();
   for(FileEntries::const_iterator itr = fileEntries.begin();
       itr != fileEntries.end(); itr++) {
     DiskWriterEntryHandle entry = new DiskWriterEntry(*itr);
     if((*itr)->isRequested()) {
-      entry->setDiskWriter(DefaultDiskWriter::createNewDiskWriter(option));
+      entry->setDiskWriter(DefaultDiskWriterFactory().newDiskWriter());
     } else {
       entry->setDiskWriter(new DefaultDiskWriter());
     }
+    entry->getDiskWriter()->setDirectIOAllowed(_directIOAllowed);
     diskWriterEntries.push_back(entry);
   }
 }
 
-string MultiDiskAdaptor::getTopDirPath() const {
+string MultiDiskAdaptor::getTopDirPath() const
+{
   return storeDir+"/"+topDir;
 }
 
-void MultiDiskAdaptor::mkdir() const {
+void MultiDiskAdaptor::mkdir() const
+{
   for(FileEntries::const_iterator itr = fileEntries.begin();
       itr != fileEntries.end(); itr++) {
     (*itr)->setupDir(getTopDirPath());
   }
 }
 
-void MultiDiskAdaptor::openFile() {
+void MultiDiskAdaptor::openFile()
+{
   mkdir();
   resetDiskWriterEntries();
   for(DiskWriterEntries::iterator itr = diskWriterEntries.begin();
@@ -73,7 +79,8 @@ void MultiDiskAdaptor::openFile() {
   }
 }
 
-void MultiDiskAdaptor::initAndOpenFile() {
+void MultiDiskAdaptor::initAndOpenFile()
+{
   mkdir();
   resetDiskWriterEntries();
   for(DiskWriterEntries::iterator itr = diskWriterEntries.begin();
@@ -82,7 +89,8 @@ void MultiDiskAdaptor::initAndOpenFile() {
   }
 }
 
-void MultiDiskAdaptor::openExistingFile() {
+void MultiDiskAdaptor::openExistingFile()
+{
   resetDiskWriterEntries();
   for(DiskWriterEntries::iterator itr = diskWriterEntries.begin();
       itr != diskWriterEntries.end(); itr++) {
@@ -90,14 +98,16 @@ void MultiDiskAdaptor::openExistingFile() {
   }
 }
 
-void MultiDiskAdaptor::closeFile() {
+void MultiDiskAdaptor::closeFile()
+{
   for(DiskWriterEntries::iterator itr = diskWriterEntries.begin();
       itr != diskWriterEntries.end(); itr++) {
     (*itr)->closeFile();
   }
 }
 
-void MultiDiskAdaptor::onDownloadComplete() {
+void MultiDiskAdaptor::onDownloadComplete()
+{
   closeFile();
   openFile();
 }
@@ -121,7 +131,7 @@ void MultiDiskAdaptor::writeData(const unsigned char* data, int32_t len,
     }
   }
   if(!writing) {
-    throw new DlAbortEx(EX_FILE_OFFSET_OUT_OF_RANGE, offset);
+    throw new DlAbortEx(EX_FILE_OFFSET_OUT_OF_RANGE, Util::llitos(offset, true).c_str());
   }
 }
 
@@ -145,7 +155,7 @@ int32_t MultiDiskAdaptor::calculateLength(const DiskWriterEntryHandle entry,
   return length;
 }
 
-int MultiDiskAdaptor::readData(unsigned char* data, int32_t len, int64_t offset)
+int32_t MultiDiskAdaptor::readData(unsigned char* data, int32_t len, int64_t offset)
 {
   int64_t fileOffset = offset;
   bool reading = false;
@@ -164,63 +174,16 @@ int MultiDiskAdaptor::readData(unsigned char* data, int32_t len, int64_t offset)
     }
   }
   if(!reading) {
-    throw new DlAbortEx(EX_FILE_OFFSET_OUT_OF_RANGE, offset);
+    throw new DlAbortEx(EX_FILE_OFFSET_OUT_OF_RANGE, Util::llitos(offset, true).c_str());
   }
   return totalReadLength;
 }
 
-void MultiDiskAdaptor::hashUpdate(MessageDigestContext& ctx,
-				  const DiskWriterEntryHandle& entry,
-				  int64_t offset, int64_t length)
-{
-  int32_t BUFSIZE = 16*1024;
-  unsigned char buf[BUFSIZE];
-  for(int64_t i = 0; i < length/BUFSIZE; i++) {
-    if(BUFSIZE != entry->getDiskWriter()->readData(buf, BUFSIZE, offset)) {
-      throw new DlAbortEx(EX_FILE_SHA1SUM, "", strerror(errno));
-    }
-    ctx.digestUpdate(buf, BUFSIZE);
-    offset += BUFSIZE;
-  }
-  int32_t r = length%BUFSIZE;
-  if(r > 0) {
-    if((int32_t)r != entry->getDiskWriter()->readData(buf, r, offset)) {
-      throw new DlAbortEx(EX_FILE_SHA1SUM, "", strerror(errno));
-    }
-    ctx.digestUpdate(buf, r);
-  }
-}
-
-string MultiDiskAdaptor::messageDigest(int64_t offset, int64_t length,
-				       const MessageDigestContext::DigestAlgo& algo) {
-  int64_t fileOffset = offset;
-  bool reading = false;
-  int32_t rem = length;
-  MessageDigestContext ctx(algo);
-  ctx.digestInit();
-
-  for(DiskWriterEntries::iterator itr = diskWriterEntries.begin();
-      itr != diskWriterEntries.end() && rem != 0; itr++) {
-    if(isInRange(*itr, offset) || reading) {
-      int32_t readLength = calculateLength((*itr), fileOffset, rem);
-      hashUpdate(ctx, *itr, fileOffset, readLength);
-      rem -= readLength;
-      reading = true;
-      fileOffset = 0;
-    } else {
-      fileOffset -= (*itr)->getFileEntry()->getLength();
-    }
-  }
-  if(!reading) {
-    throw new DlAbortEx(EX_FILE_OFFSET_OUT_OF_RANGE, offset);
-  }
-  unsigned char hashValue[20];
-  ctx.digestFinal(hashValue);
-  return Util::toHex(hashValue, 20);
-}
-
 bool MultiDiskAdaptor::fileExists()
 {
+  if(diskWriterEntries.empty()) {
+    resetDiskWriterEntries();
+  }
   for(DiskWriterEntries::iterator itr = diskWriterEntries.begin();
       itr != diskWriterEntries.end(); itr++) {
     if((*itr)->fileExists(getTopDirPath())) {
@@ -229,3 +192,35 @@ bool MultiDiskAdaptor::fileExists()
   }
   return false;
 }
+
+// TODO call DiskWriter::openFile() before calling this function.
+int64_t MultiDiskAdaptor::size() const
+{
+  int64_t size = 0;
+  for(DiskWriterEntries::const_iterator itr = diskWriterEntries.begin();
+      itr != diskWriterEntries.end(); itr++) {
+    size += (*itr)->size();
+  }
+  return size;
+}
+
+FileAllocationIteratorHandle MultiDiskAdaptor::fileAllocationIterator()
+{
+  return new MultiFileAllocationIterator(this);
+}
+
+void MultiDiskAdaptor::enableDirectIO()
+{
+  for(DiskWriterEntries::const_iterator itr = diskWriterEntries.begin();
+      itr != diskWriterEntries.end(); ++itr) {
+    (*itr)->getDiskWriter()->enableDirectIO();
+  }
+}
+
+void MultiDiskAdaptor::disableDirectIO()
+{
+  for(DiskWriterEntries::const_iterator itr = diskWriterEntries.begin();
+      itr != diskWriterEntries.end(); ++itr) {
+    (*itr)->getDiskWriter()->disableDirectIO();
+  }
+}
