@@ -36,10 +36,11 @@
 #define _D_MULTI_DISK_ADAPTOR_H_
 
 #include "DiskAdaptor.h"
-#include "Option.h"
 #include "DiskWriter.h"
 #include "File.h"
 
+class MultiFileAllocationIterator;
+
 class DiskWriterEntry {
 private:
   FileEntryHandle fileEntry;
@@ -81,6 +82,11 @@ public:
     return File(getFilePath(topDir)).exists();
   }
 
+  int64_t size() const
+  {
+    return diskWriter->size();
+  }
+
   FileEntryHandle getFileEntry() const {
     return fileEntry;
   }
@@ -92,6 +98,11 @@ public:
   DiskWriterHandle getDiskWriter() const {
     return diskWriter;
   }
+
+  bool operator<(const DiskWriterEntry& entry) const
+  {
+    return fileEntry < entry.fileEntry;
+  }
 };
 
 typedef SharedHandle<DiskWriterEntry> DiskWriterEntryHandle;
@@ -99,11 +110,13 @@ typedef SharedHandle<DiskWriterEntry> DiskWriterEntryHandle;
 typedef deque<DiskWriterEntryHandle> DiskWriterEntries;
 
 class MultiDiskAdaptor : public DiskAdaptor {
+  friend class MultiFileAllocationIterator;
 private:
   string topDir;
   int32_t pieceLength;
   DiskWriterEntries diskWriterEntries;
-  const Option* option;
+
+  bool _directIOAllowed;
 
   void resetDiskWriterEntries();
 
@@ -115,14 +128,9 @@ private:
 			  int64_t fileOffset,
 			  int32_t rem) const;
 
-  void hashUpdate(MessageDigestContext& ctx,
-		  const DiskWriterEntryHandle& entry,
-		  int64_t offset, int64_t length);
-
   string getTopDirPath() const;
 public:
-  MultiDiskAdaptor():pieceLength(0),
-		     option(0)
+  MultiDiskAdaptor():pieceLength(0)
   {}
 
   virtual ~MultiDiskAdaptor() {}
@@ -140,17 +148,23 @@ public:
   virtual void writeData(const unsigned char* data, int32_t len,
 			 int64_t offset);
 
-  virtual int readData(unsigned char* data, int32_t len, int64_t offset);
-
-  virtual string messageDigest(int64_t offset, int64_t length,
-			       const MessageDigestContext::DigestAlgo& algo);
+  virtual int32_t readData(unsigned char* data, int32_t len, int64_t offset);
 
   virtual bool fileExists();
 
-  virtual string getFilePath() {
+  virtual string getFilePath()
+  {
     return getTopDirPath();
   }
 
+  virtual int64_t size() const;
+
+  virtual FileAllocationIteratorHandle fileAllocationIterator();
+
+  virtual void enableDirectIO();
+
+  virtual void disableDirectIO();
+
   void setTopDir(const string& topDir) {
     this->topDir = topDir;
   }
@@ -167,12 +181,14 @@ public:
     return pieceLength;
   }
 
-  void setOption(const Option* option) {
-    this->option = option;
+  virtual bool directIOAllowed() const
+  {
+    return _directIOAllowed;
   }
 
-  const Option* getOption() const {
-    return option;
+  void setDirectIOAllowed(bool b)
+  {
+    _directIOAllowed = b;
   }
 };
 
