@@ -36,10 +36,7 @@
 #define _D_NAME_RESOLVER_H_
 
 #include "common.h"
-#include <netdb.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
+#include "a2netcompat.h"
 
 #ifdef ENABLE_ASYNC_DNS
 
@@ -51,10 +48,18 @@ extern "C" {
 } /* end of extern "C" */
 #endif
 
+#ifdef HAVE_LIBCARES1_5
+void callback(void* arg, int status, int timeouts, struct hostent* host);
+#else
 void callback(void* arg, int status, struct hostent* host);
+#endif // HAVE_LIBCARES1_5
 
 class NameResolver {
+#ifdef HAVE_LIBCARES1_5
+  friend void callback(void* arg, int status, int timeouts, struct hostent* host);
+#else
   friend void callback(void* arg, int status, struct hostent* host);
+#endif // HAVE_LIBCARES1_5
 
 public:
   enum STATUS {
@@ -100,7 +105,7 @@ public:
     return status;
   }
 
-  int getFds(fd_set* rfdsPtr, fd_set* wfdsPtr) const {
+  int32_t getFds(fd_set* rfdsPtr, fd_set* wfdsPtr) const {
     return ares_fds(channel, rfdsPtr, wfdsPtr);
   }
 
