@@ -35,8 +35,11 @@
 #include "Peer.h"
 #include "BitfieldManFactory.h"
 #include "Util.h"
+#ifdef ENABLE_MESSAGE_DIGEST
+# include "MessageDigestHelper.h"
+#endif // ENABLE_MESSAGE_DIGEST
 
-Peer::Peer(string ipaddr, int port, int pieceLength, long long int totalLength):
+Peer::Peer(string ipaddr, int32_t port, int32_t pieceLength, int64_t totalLength):
   ipaddr(ipaddr),
   port(port),
   sessionUploadLength(0),
@@ -50,7 +53,19 @@ Peer::Peer(string ipaddr, int port, int pieceLength, long long int totalLength):
   this->bitfield = BitfieldManFactory::getFactoryInstance()->
     createBitfieldMan(pieceLength, totalLength);
   string idSeed = ipaddr+":"+Util::itos(port);
-  id = Util::simpleMessageDigest(idSeed);
+#ifdef ENABLE_MESSAGE_DIGEST
+  id = MessageDigestHelper::digestString("sha1", idSeed);
+#else
+  id = idSeed;
+#endif // ENABLE_MESSAGE_DIGEST
+}
+
+void Peer::reconfigure(int32_t pieceLength, int64_t totalLength)
+{
+  delete bitfield;
+  this->pieceLength = pieceLength;
+  this->bitfield = BitfieldManFactory::getFactoryInstance()->
+    createBitfieldMan(this->pieceLength, totalLength);  
 }
 
 /*
@@ -62,7 +77,7 @@ Peer::Peer():entryId(0), ipaddr(""), port(0), bitfield(0),
 }
 */
 
-void Peer::updateBitfield(int index, int operation) {
+void Peer::updateBitfield(int32_t index, int32_t operation) {
   if(operation == 1) {
     bitfield->setBit(index);
   } else if(operation == 0) {
@@ -79,7 +94,7 @@ bool Peer::shouldBeChoking() const {
   return chokingRequired;
 }
 
-bool Peer::hasPiece(int index) const {
+bool Peer::hasPiece(int32_t index) const {
   return bitfield->isBitSet(index);
 }
 
@@ -99,39 +114,28 @@ void Peer::resetStatus() {
   snubbing = false;
   fastExtensionEnabled = false;
   latency = DEFAULT_LATENCY;
-  fastSet.clear();
   peerAllowedIndexSet.clear();
   amAllowedIndexSet.clear();
   peerStat.reset();
 }
 
-bool Peer::isInFastSet(int index) const {
-  return find(fastSet.begin(), fastSet.end(), index) != fastSet.end();
-}
-
-void Peer::addFastSetIndex(int index) {
-  if(!isInFastSet(index)) {
-    fastSet.push_back(index);
-  }
-}
-
-bool Peer::isInPeerAllowedIndexSet(int index) const {
+bool Peer::isInPeerAllowedIndexSet(int32_t index) const {
   return find(peerAllowedIndexSet.begin(), peerAllowedIndexSet.end(),
 	      index) != peerAllowedIndexSet.end();
 }
 
-void Peer::addPeerAllowedIndex(int index) {
+void Peer::addPeerAllowedIndex(int32_t index) {
   if(!isInPeerAllowedIndexSet(index)) {
     peerAllowedIndexSet.push_back(index);
   }
 }
 
-bool Peer::isInAmAllowedIndexSet(int index) const {
+bool Peer::isInAmAllowedIndexSet(int32_t index) const {
   return find(amAllowedIndexSet.begin(), amAllowedIndexSet.end(),
 	      index) != amAllowedIndexSet.end();
 }
 
-void Peer::addAmAllowedIndex(int index) {
+void Peer::addAmAllowedIndex(int32_t index) {
   if(!isInAmAllowedIndexSet(index)) {
     amAllowedIndexSet.push_back(index);
   }
@@ -141,7 +145,7 @@ void Peer::setAllBitfield() {
   bitfield->setAllBit();
 }
 
-void Peer::updateLatency(int latency) {
+void Peer::updateLatency(int32_t latency) {
   this->latency = (this->latency*20+latency*80)/200;
 }
 
