@@ -37,13 +37,9 @@
 
 #include "common.h"
 #include "BitfieldMan.h"
-#include "SharedHandle.h"
 #include "PeerStat.h"
 #include "TimeA2.h"
 #include <string.h>
-#include <string>
-
-using namespace std;
 
 #define PEER_ID_LENGTH 20
 #define DEFAULT_LATENCY 1500
@@ -53,13 +49,13 @@ class Peer {
   friend bool operator!=(const Peer& p1, const Peer& p2);
 public:
   string ipaddr;
-  int port;
+  int32_t port;
   bool amChoking;
   bool amInterested;
   bool peerChoking;
   bool peerInterested;
-  int tryCount;
-  int cuid;
+  int32_t tryCount;
+  int32_t cuid;
   bool chokingRequired;
   bool optUnchoking;
   bool snubbing;
@@ -67,23 +63,21 @@ private:
   unsigned char peerId[PEER_ID_LENGTH];
   BitfieldMan* bitfield;
   bool fastExtensionEnabled;
-  // allowed fast indexes that peer has sent by Allowed Fast message
-  Integers fastSet;
   // fast index set which a peer has sent to localhost.
   Integers peerAllowedIndexSet;
   // fast index set which localhost has sent to a peer.
   Integers amAllowedIndexSet;
   PeerStat peerStat;
-  long long int sessionUploadLength;
-  long long int sessionDownloadLength;
-  int pieceLength;
-  int latency;
+  int64_t sessionUploadLength;
+  int64_t sessionDownloadLength;
+  int32_t pieceLength;
+  int32_t latency;
   bool active;
   string id;
   Time _badConditionStartTime;
-  int _badConditionInterval;
+  int32_t _badConditionInterval;
 public:
-  Peer(string ipaddr, int port, int pieceLength, long long int totalLength);
+  Peer(string ipaddr, int32_t port, int32_t pieceLength, int64_t totalLength);
 
   ~Peer() {
     delete bitfield;
@@ -100,12 +94,12 @@ public:
 
   void resetStatus();
 
-  void updateUploadLength(int bytes) {
+  void updateUploadLength(int32_t bytes) {
     peerStat.updateUploadLength(bytes);
     sessionUploadLength += bytes;
   }
 
-  void updateDownloadLength(int bytes) {
+  void updateDownloadLength(int32_t bytes) {
     peerStat.updateDownloadLength(bytes);
     sessionDownloadLength += bytes;
   }
@@ -113,28 +107,36 @@ public:
   /**
    * Returns the transfer rate from localhost to remote host.
    */
-  int calculateUploadSpeed() {
+  int32_t calculateUploadSpeed() {
     return peerStat.calculateUploadSpeed();
   }
 
+  int32_t calculateUploadSpeed(const struct timeval& now) {
+    return peerStat.calculateUploadSpeed(now);
+  }
+
   /**
    * Returns the transfer rate from remote host to localhost.
    */
-  int calculateDownloadSpeed() {
+  int32_t calculateDownloadSpeed() {
     return peerStat.calculateDownloadSpeed();
   }
 
+  int32_t calculateDownloadSpeed(const struct timeval& now) {
+    return peerStat.calculateDownloadSpeed(now);
+  }
+
   /**
    * Returns the number of bytes uploaded to the remote host.
    */
-  long long int getSessionUploadLength() const {
+  int64_t getSessionUploadLength() const {
     return sessionUploadLength;
   }
 
   /**
    * Returns the number of bytes downloaded from the remote host.
    */
-  long long int getSessionDownloadLength() const {
+  int64_t getSessionDownloadLength() const {
     return sessionDownloadLength;
   }
 
@@ -157,43 +159,48 @@ public:
   }
   const unsigned char* getPeerId() const { return this->peerId; }
   
-  void setBitfield(const unsigned char* bitfield, int bitfieldLength) {
+  void setBitfield(const unsigned char* bitfield, int32_t bitfieldLength) {
     this->bitfield->setBitfield(bitfield, bitfieldLength);
   }
   const unsigned char* getBitfield() const { return bitfield->getBitfield(); }
-  int getBitfieldLength() const { return bitfield->getBitfieldLength(); }
+  int32_t getBitfieldLength() const { return bitfield->getBitfieldLength(); }
   void setAllBitfield();
 
   /**
    * operation = 1: set index-th bit to 1
    * operation = 0: set index-th bit to 0
    */
-  void updateBitfield(int index, int operation);
+  void updateBitfield(int32_t index, int32_t operation);
   
   void setFastExtensionEnabled(bool enabled) {
     fastExtensionEnabled = enabled;
   }
   bool isFastExtensionEnabled() const { return fastExtensionEnabled; }
 
-  void addFastSetIndex(int index);
-  const Integers& getFastSet() const { return fastSet; }
-  bool isInFastSet(int index) const;
-  int countFastSet() const { return fastSet.size(); }
+  void addPeerAllowedIndex(int32_t index);
+  bool isInPeerAllowedIndexSet(int32_t index) const;
 
-  void addPeerAllowedIndex(int index);
-  bool isInPeerAllowedIndexSet(int index) const;
+  int32_t countPeerAllowedIndexSet() const
+  {
+    return peerAllowedIndexSet.size();
+  }
 
-  void addAmAllowedIndex(int index);
-  bool isInAmAllowedIndexSet(int index) const;
+  const Integers& getPeerAllowedIndexSet() const
+  {
+    return peerAllowedIndexSet;
+  }
+
+  void addAmAllowedIndex(int32_t index);
+  bool isInAmAllowedIndexSet(int32_t index) const;
 
   bool shouldBeChoking() const;
 
-  bool hasPiece(int index) const;
+  bool hasPiece(int32_t index) const;
 
   bool isSeeder() const;
 
-  void updateLatency(int latency);
-  int getLatency() const { return latency; }
+  void updateLatency(int32_t latency);
+  int32_t getLatency() const { return latency; }
 
   const string& getId() const {
     return id;
@@ -202,6 +209,8 @@ public:
   void startBadCondition();
 
   bool isGood() const;
+
+  void reconfigure(int32_t pieceLength, int64_t totalLength);
 };
 
 typedef SharedHandle<Peer> PeerHandle;
