@@ -33,33 +33,34 @@
  */
 /* copyright --> */
 #include "PeerAbstractCommand.h"
+#include "Peer.h"
+#include "DownloadEngine.h"
+#include "Option.h"
 #include "DlAbortEx.h"
 #include "DlRetryEx.h"
 #include "Util.h"
 #include "message.h"
 #include "prefs.h"
 
-PeerAbstractCommand::PeerAbstractCommand(int cuid, const PeerHandle& peer,
-					 TorrentDownloadEngine* e,
-					 const BtContextHandle& btContext,
+PeerAbstractCommand::PeerAbstractCommand(int32_t cuid,
+					 const PeerHandle& peer,
+					 DownloadEngine* e,
 					 const SocketHandle& s)
-  :BtContextAwareCommand(cuid, btContext), e(e), socket(s), peer(peer),
+  :Command(cuid), e(e), socket(s), peer(peer),
    checkSocketIsReadable(false), checkSocketIsWritable(false),
    uploadLimitCheck(false), uploadLimit(0), noCheck(false)
 {
   setReadCheckSocket(socket);
   timeout = e->option->getAsInt(PREF_BT_TIMEOUT);
-  btRuntime->increaseConnections();
 }
 
 PeerAbstractCommand::~PeerAbstractCommand() {
   disableReadCheckSocket();
   disableWriteCheckSocket();
-  btRuntime->decreaseConnections();
 }
 
 bool PeerAbstractCommand::execute() {
-  if(btRuntime->isHalt()) {
+  if(exitBeforeExecute()) {
     peer->resetStatus();
     return true;
   }
@@ -79,7 +80,7 @@ bool PeerAbstractCommand::execute() {
     return executeInternal();
   } catch(RecoverableException* err) {
     logger->debug(MSG_TORRENT_DOWNLOAD_ABORTED, err, cuid);
-    logger->debug("CUID#%d - Peer %s:%d banned.",
+    logger->debug(MSG_PEER_BANNED,
 		  cuid, peer->ipaddr.c_str(), peer->port);
     onAbort(err);
     delete err;
@@ -88,18 +89,14 @@ bool PeerAbstractCommand::execute() {
 }
 
 // TODO this method removed when PeerBalancerCommand is implemented
-bool PeerAbstractCommand::prepareForNextPeer(int wait) {
+bool PeerAbstractCommand::prepareForNextPeer(int32_t wait) {
   return true;
 }
 
-bool PeerAbstractCommand::prepareForRetry(int wait) {
+bool PeerAbstractCommand::prepareForRetry(int32_t wait) {
   return true;
 }
 
-void PeerAbstractCommand::onAbort(Exception* ex) {
-  peerStorage->returnPeer(peer);
-}
-
 void PeerAbstractCommand::disableReadCheckSocket() {
   if(checkSocketIsReadable) {
     e->deleteSocketForReadCheck(readCheckTarget, this);
@@ -152,7 +149,7 @@ void PeerAbstractCommand::setWriteCheckSocket(const SocketHandle& socket) {
   }
 }
 
-void PeerAbstractCommand::setUploadLimit(int uploadLimit) {
+void PeerAbstractCommand::setUploadLimit(int32_t uploadLimit) {
   this->uploadLimit = uploadLimit;
 }
 
