@@ -54,7 +54,7 @@ private:
   char resbuf[MAX_PAYLOAD_LEN];
   int32_t resbufLength;
   int32_t currentPayloadLength;
-  unsigned char lenbuf[4];
+  char lenbuf[4];
   int32_t lenbufLength;
 
 public:
@@ -72,7 +72,7 @@ public:
    * In both cases, 'msg' is filled with received bytes and the filled length
    * is assigned to 'length'.
    */
-  bool receiveHandshake(unsigned char* data, int32_t& dataLength);
+  bool receiveHandshake(unsigned char* data, int32_t& dataLength, bool peek = false);
 };
 
 typedef SharedHandle<PeerConnection> PeerConnectionHandle;
