@@ -36,34 +36,44 @@
 #define _D_PEER_INTERACTION_COMMAND_H_
 
 #include "PeerAbstractCommand.h"
+#include "RequestGroupAware.h"
+#include "BtContextAwareCommand.h"
 #include "BtInteractive.h"
+#include "PeerConnection.h"
 
-class PeerInteractionCommand : public PeerAbstractCommand {
+class PeerInteractionCommand : public PeerAbstractCommand,
+			       public BtContextAwareCommand,
+			       public RequestGroupAware
+{
+public:
+  enum Seq {
+    INITIATOR_SEND_HANDSHAKE,
+    INITIATOR_WAIT_HANDSHAKE,
+    //RECEIVER_SEND_HANDSHAKE,
+    RECEIVER_WAIT_HANDSHAKE,
+    WIRED};
 private:
-  int sequence;
+  Seq sequence;
   BtInteractiveHandle btInteractive;
   int32_t maxDownloadSpeedLimit;
 protected:
   virtual bool executeInternal();
-  virtual bool prepareForRetry(int wait);
-  virtual bool prepareForNextPeer(int wait);
+  virtual bool prepareForRetry(int32_t wait);
+  virtual bool prepareForNextPeer(int32_t wait);
   virtual void onAbort(Exception* ex);
+  virtual bool exitBeforeExecute();
 public:
-  PeerInteractionCommand(int cuid,
+  PeerInteractionCommand(int32_t cuid,
+			 RequestGroup* requestGroup,
 			 const PeerHandle& peer,
-			 TorrentDownloadEngine* e,
+			 DownloadEngine* e,
 			 const BtContextHandle& btContext,
 			 const SocketHandle& s,
-			 int sequence);
+			 Seq sequence,
+			 const PeerConnectionHandle& peerConnection = 0);
 
-  ~PeerInteractionCommand();
+  virtual ~PeerInteractionCommand();
 
-  enum Seq {
-    INITIATOR_SEND_HANDSHAKE,
-    INITIATOR_WAIT_HANDSHAKE,
-    RECEIVER_SEND_HANDSHAKE,
-    RECEIVER_WAIT_HANDSHAKE,
-    WIRED};
 };
 
 #endif // _D_PEER_INTERACTION_COMMAND_H_
