@@ -33,31 +33,42 @@
  */
 /* copyright --> */
 #include "PeerListenCommand.h"
-#include "PeerInteractionCommand.h"
+#include "DownloadEngine.h"
+#include "Peer.h"
+#include "RequestGroupMan.h"
 #include "RecoverableException.h"
 #include "CUIDCounter.h"
+#include "message.h"
+#include "PeerReceiveHandshakeCommand.h"
 
-PeerListenCommand::PeerListenCommand(int cuid,
-				     TorrentDownloadEngine* e,
-				     const BtContextHandle& btContext)
-  :BtContextAwareCommand(cuid, btContext),
-   e(e),
-   _lowestSpeedLimit(20*1024) {}
+int32_t PeerListenCommand::__numInstance = 0;
 
-PeerListenCommand::~PeerListenCommand() {}
+PeerListenCommand* PeerListenCommand::__instance = 0;
 
-int PeerListenCommand::bindPort(int portRangeStart, int portRangeEnd) {
-  if(portRangeStart > portRangeEnd) {
-    return -1;
-  }
-  for(int port = portRangeStart; port <= portRangeEnd; port++) {
+PeerListenCommand::PeerListenCommand(int32_t cuid, DownloadEngine* e):
+  Command(cuid),
+  e(e),
+  _lowestSpeedLimit(20*1024)
+{
+  ++__numInstance;
+}
+
+PeerListenCommand::~PeerListenCommand()
+{
+  --__numInstance;
+}
+
+int32_t PeerListenCommand::bindPort(IntSequence& seq)
+{
+  while(seq.hasNext()) {
+    int32_t port = seq.next();
     try {
       socket->beginListen(port);
-      logger->info("CUID#%d - using port %d for accepting new connections",
+      logger->info(MSG_LISTENING_PORT,
 		   cuid, port);
       return port;
     } catch(RecoverableException* ex) {
-      logger->error("CUID#%d - an error occurred while binding port=%d",
+      logger->error(MSG_BIND_FAILURE,
 		    ex, cuid, port);
       socket->closeConnection();
       delete ex;
@@ -67,41 +78,45 @@ int PeerListenCommand::bindPort(int portRangeStart, int portRangeEnd) {
 }
 
 bool PeerListenCommand::execute() {
-  if(btRuntime->isHalt()) {
+  if(e->isHaltRequested() || e->_requestGroupMan->countRequestGroup() == 0) {
     return true;
   }
-  for(int i = 0; i < 3 && socket->isReadable(0); i++) {
+  for(int32_t i = 0; i < 3 && socket->isReadable(0); i++) {
     SocketHandle peerSocket;
     try {
       peerSocket = socket->acceptConnection();
-      pair<string, int> peerInfo;
+      pair<string, int32_t> peerInfo;
       peerSocket->getPeerInfo(peerInfo);
-      pair<string, int> localInfo;
+      pair<string, int32_t> localInfo;
       peerSocket->getAddrInfo(localInfo);
 
-      TransferStat tstat = peerStorage->calculateStat();
-      if(peerInfo.first != localInfo.first &&
-	 (!pieceStorage->downloadFinished() && tstat.getDownloadSpeed() < _lowestSpeedLimit ||
-	  btRuntime->getConnections() < MAX_PEERS)) {
-	PeerHandle peer = PeerHandle(new Peer(peerInfo.first, peerInfo.second,
-					      btContext->getPieceLength(),
-					      btContext->getTotalLength()));
-	if(peerStorage->addIncomingPeer(peer)) {
-	  peer->cuid = CUIDCounterSingletonHolder::instance()->newID();
-	  PeerInteractionCommand* command =
-	    new PeerInteractionCommand(peer->cuid, peer, e,
-				       btContext,
-				       peerSocket,
-				       PeerInteractionCommand::RECEIVER_WAIT_HANDSHAKE);
-	  e->commands.push_back(command);
-	  logger->debug("CUID#%d - incoming connection, adding new command CUID#%d", cuid, peer->cuid);
-	}
+      if(peerInfo.first == localInfo.first) {
+	continue;
       }
+      PeerHandle peer = new Peer(peerInfo.first, peerInfo.second, 0, 0);
+      PeerReceiveHandshakeCommand* command =
+	new PeerReceiveHandshakeCommand(CUIDCounterSingletonHolder::instance()->newID(),
+					peer, e, peerSocket);
+      e->commands.push_back(command);
+      logger->debug("Accepted the connection from %s:%d.",
+		    peer->ipaddr.c_str(),
+		    peer->port);
+      logger->debug("Added CUID#%d to receive Bt handshake.",
+		    command->getCuid());
+
     } catch(RecoverableException* ex) {
-      logger->debug("CUID#%d - error in accepting connection", ex, cuid);
+      logger->debug(MSG_ACCEPT_FAILURE, ex, cuid);
       delete ex;
     }		    
   }
   e->commands.push_back(this);
   return false;
 }
+
+PeerListenCommand* PeerListenCommand::getInstance(DownloadEngine* e)
+{
+  if(__numInstance == 0) {
+    __instance = new PeerListenCommand(CUIDCounterSingletonHolder::instance()->newID(), e);
+  }
+  return __instance;
+}
