@@ -35,29 +35,43 @@
 #ifndef _D_PEER_LISTEN_COMMAND_H_
 #define _D_PEER_LISTEN_COMMAND_H_
 
-#include "BtContextAwareCommand.h"
-#include "TorrentDownloadEngine.h"
+#include "Command.h"
+#include "Socket.h"
+#include "IntSequence.h"
 
-class PeerListenCommand : public BtContextAwareCommand {
+class DownloadEngine;
+
+class PeerListenCommand : public Command {
 private:
-  TorrentDownloadEngine* e;
+  DownloadEngine* e;
   SocketHandle socket;
   int32_t _lowestSpeedLimit;
+
+  static int32_t __numInstance;
+
+  static PeerListenCommand* __instance;
+
 public:
-  PeerListenCommand(int cuid,
-		    TorrentDownloadEngine* e,
-		    const BtContextHandle& btContext);
+  PeerListenCommand(int32_t cuid, DownloadEngine* e);
 
-  ~PeerListenCommand();
+  virtual ~PeerListenCommand();
   
-  bool execute();
+  virtual bool execute();
 
-  int bindPort(int portRangeStart, int portRangeEnd);
+  int32_t bindPort(IntSequence& seq);
 
   void setLowestSpeedLimit(int32_t speed)
   {
     _lowestSpeedLimit = speed;
   }
+
+  static PeerListenCommand* getInstance(DownloadEngine* e);
+
+  static int32_t getNumInstance()
+  {
+    return __numInstance;
+  }
+  
 };
 
 #endif // _D_PEER_LISTEN_COMMAND_H_
