@@ -47,49 +47,57 @@ public:
     REQUEST_IDLE,
   };
 private:
-  int cuid;
+  int32_t cuid;
   SpeedCalc downloadSpeed;
   SpeedCalc uploadSpeed;
   Time downloadStartTime;
   PeerStat::STATUS status;
 public:
 
-  PeerStat(int cuid = 0):cuid(cuid), status(PeerStat::IDLE) {}
+  PeerStat(int32_t cuid = 0):cuid(cuid), status(PeerStat::IDLE) {}
 
   ~PeerStat() {}
 
   /**
    * Returns current download speed in byte per sec.
    */
-  int calculateDownloadSpeed() {
+  int32_t calculateDownloadSpeed() {
     return downloadSpeed.calculateSpeed();
   }
 
-  int calculateUploadSpeed() {
+  int32_t calculateDownloadSpeed(const struct timeval& now) {
+    return downloadSpeed.calculateSpeed(now);
+  }
+
+  int32_t calculateUploadSpeed() {
     return uploadSpeed.calculateSpeed();
   }
 
-  void updateDownloadLength(int bytes) {
+  int32_t calculateUploadSpeed(const struct timeval& now) {
+    return uploadSpeed.calculateSpeed(now);
+  }
+
+  void updateDownloadLength(int32_t bytes) {
     downloadSpeed.update(bytes);
   }
 
-  void updateUploadLength(int bytes) {
+  void updateUploadLength(int32_t bytes) {
     uploadSpeed.update(bytes);
   }
 
-  int getMaxDownloadSpeed() const {
+  int32_t getMaxDownloadSpeed() const {
     return downloadSpeed.getMaxSpeed();
   }
 
-  int getMaxUploadSpeed() const {
+  int32_t getMaxUploadSpeed() const {
     return uploadSpeed.getMaxSpeed();
   }
 
-  int getAvgDownloadSpeed() const {
+  int32_t getAvgDownloadSpeed() const {
     return downloadSpeed.getAvgSpeed();
   }
 
-  int getAvgUploadSpeed() const {
+  int32_t getAvgUploadSpeed() const {
     return uploadSpeed.getAvgSpeed();
   }
 
@@ -121,7 +129,7 @@ public:
     return status;
   }
 
-  int getCuid() const {
+  int32_t getCuid() const {
     return cuid;
   }
 };
