@@ -36,9 +36,16 @@
 #define _D_PIECE_STORAGE_H_
 
 #include "common.h"
-#include "Peer.h"
-#include "Piece.h"
-#include "DiskAdaptor.h"
+#include "TimeA2.h"
+#include "IntSequence.h"
+
+class Piece;
+typedef SharedHandle<Piece> PieceHandle;
+typedef deque<PieceHandle> Pieces;
+class Peer;
+typedef SharedHandle<Peer> PeerHandle;
+class DiskAdaptor;
+typedef SharedHandle<DiskAdaptor> DiskAdaptorHandle;
 
 class PieceStorage {
 public:
@@ -66,11 +73,29 @@ public:
    */
   virtual PieceHandle getMissingFastPiece(const PeerHandle& peer) = 0;
 
+  /**
+   * Returns a missing piece if available. Otherwise returns 0;
+   */
+  virtual PieceHandle getMissingPiece() = 0;
+
+  /**
+   * Returns a missing piece whose index is index.
+   * If a piece whose index is index is already acquired or currently used,
+   * then returns 0.
+   * Also returns 0 if any of missing piece is not available.
+   */
+  virtual PieceHandle getMissingPiece(int32_t index) = 0;
+
   /**
    * Returns the piece denoted by index.
    * No status of the piece is changed in this method.
    */
-  virtual PieceHandle getPiece(int index) = 0;
+  virtual PieceHandle getPiece(int32_t index) = 0;
+
+  /**
+   * Marks the piece whose index is index as missing.
+   */
+  virtual void markPieceMissing(int32_t index) = 0;
 
   /**
    * Tells that the download of the specfied piece completes.
@@ -86,19 +111,21 @@ public:
    * Returns true if the specified piece is already downloaded.
    * Otherwise returns false.
    */
-  virtual bool hasPiece(int index) = 0;
+  virtual bool hasPiece(int32_t index) = 0;
 
-  virtual long long int getTotalLength() = 0;
+  virtual bool isPieceUsed(int32_t index) = 0;
 
-  virtual long long int getFilteredTotalLength() = 0;
+  virtual int64_t getTotalLength() = 0;
 
-  virtual long long int getCompletedLength() = 0;
+  virtual int64_t getFilteredTotalLength() = 0;
 
-  virtual long long int getFilteredCompletedLength() = 0;
+  virtual int64_t getCompletedLength() = 0;
+
+  virtual int64_t getFilteredCompletedLength() = 0;
   
   virtual void setFileFilter(const Strings& filePaths) = 0;
 
-  virtual void setFileFilter(const Integers& fileIndexes) = 0;
+  virtual void setFileFilter(IntSequence seq) = 0;
 
   virtual void clearFileFilter() = 0;
   
@@ -124,9 +151,9 @@ public:
   virtual const unsigned char* getBitfield() = 0;
 
   virtual void setBitfield(const unsigned char* bitfield,
-			   int bitfieldLength) = 0;
+			   int32_t bitfieldLength) = 0;
   
-  virtual int getBitfieldLength() = 0;
+  virtual int32_t getBitfieldLength() = 0;
 
   virtual bool isSelectiveDownloadingMode() = 0;
 
@@ -136,26 +163,26 @@ public:
 
   virtual DiskAdaptorHandle getDiskAdaptor() = 0;
   
-  virtual int getPieceLength(int index) = 0;
+  virtual int32_t getPieceLength(int32_t index) = 0;
 
   /**
    * Adds piece index to advertise to other commands. They send have message
    * based on this information.
    */
-  virtual void advertisePiece(int cuid, int index) = 0;
+  virtual void advertisePiece(int32_t cuid, int32_t index) = 0;
 
   /**
    * Returns piece index which is not advertised by the caller command and
    * newer than lastCheckTime.
    */
-  virtual Integers getAdvertisedPieceIndexes(int myCuid,
+  virtual Integers getAdvertisedPieceIndexes(int32_t myCuid,
 					     const Time& lastCheckTime) = 0;
 
   /**
    * Removes have entry if specified seconds have elapsed since its
    * registration.
    */
-  virtual void removeAdvertisedPiece(int elapsed) = 0;
+  virtual void removeAdvertisedPiece(int32_t elapsed) = 0;
 
   /**
    * Sets all bits in bitfield to 1.
@@ -163,9 +190,15 @@ public:
   virtual void markAllPiecesDone() = 0;
 
   /**
-   * Validates file integrity by comparing checksums.
+   * Sets all bits in bitfield(0 to length) to 1.
    */
-  virtual void checkIntegrity() = 0;
+  virtual void markPiecesDone(int64_t length) = 0;
+
+  virtual void addInFlightPiece(const Pieces& pieces) = 0;
+
+  virtual int32_t countInFlightPiece() = 0;
+
+  virtual Pieces getInFlightPieces() = 0;
 };
 
 typedef SharedHandle<PieceStorage> PieceStorageHandle;
