@@ -36,15 +36,13 @@
 #include "Util.h"
 #include "FeatureConfig.h"
 #include "CookieBoxFactory.h"
+#include "RecoverableException.h"
 
 const string Request::METHOD_GET = "get";
 
 const string Request::METHOD_HEAD = "head";
 
-Request::Request():port(0), tryCount(0), keepAlive(true), method(METHOD_GET),
-		   _httpAuthResolver(0),
-		   _httpProxyAuthResolver(0),
-		   _ftpAuthResolver(0),
+Request::Request():port(0), tryCount(0), keepAlive(false), method(METHOD_GET),
 		   cookieBox(CookieBoxFactorySingletonHolder::instance()->createNewInstance()) {}
 
 Request::~Request() {}
@@ -60,7 +58,8 @@ bool Request::resetUrl() {
 }
 
 bool Request::redirectUrl(const string& url) {
-  previousUrl = currentUrl;
+  previousUrl = "";
+  keepAlive = false;
   return parseUrl(url);
 }
 
@@ -68,61 +67,82 @@ bool Request::parseUrl(const string& url) {
   string tempUrl;
   string::size_type sharpIndex = url.find("#");
   if(sharpIndex != string::npos) {
-    if(FeatureConfig::getInstance()->isSupported("metalink") &&
-       url.find(METALINK_MARK) == sharpIndex) {
-      tempUrl = url.substr(sharpIndex+strlen(METALINK_MARK));
-    } else {
-      tempUrl = url.substr(0, sharpIndex);
-    }
+    tempUrl = urlencode(url.substr(0, sharpIndex));
   } else {
-    tempUrl = url;
+    tempUrl = urlencode(url);
   }
-
   currentUrl = tempUrl;
   string query;
   host = "";
   port = 0;
   dir = "";
   file = "";
-  if(tempUrl.find_first_not_of(SAFE_CHARS) != string::npos) {
-    return false;
-  }
+  _username = "";
+  _password = "";
+  // find query part
   string::size_type startQueryIndex = tempUrl.find("?");
   if(startQueryIndex != string::npos) {
     query = tempUrl.substr(startQueryIndex);
     tempUrl.erase(startQueryIndex);
   }
+  // find protocol
   string::size_type hp = tempUrl.find("://");
   if(hp == string::npos) return false;
   protocol = tempUrl.substr(0, hp);
-  int defPort;
+  int32_t defPort;
   if((defPort = FeatureConfig::getInstance()->getDefaultPort(protocol)) == 0) {
     return false;
   }
   hp += 3;
+  // find host part
   if(tempUrl.size() <= hp) return false;
   string::size_type hep = tempUrl.find("/", hp);
   if(hep == string::npos) {
     hep = tempUrl.size();
   }
+  string hostPart = tempUrl.substr(hp, hep-hp);
+  //   find username and password in host part if they exist
+  string::size_type atmarkp =  hostPart.find_last_of("@");
+  if(atmarkp != string::npos) {
+    string authPart = hostPart.substr(0, atmarkp);
+    pair<string, string> userPass = Util::split(authPart, ":");
+    _username = Util::urldecode(userPass.first);
+    _password = Util::urldecode(userPass.second);
+    hostPart.erase(0, atmarkp+1);
+  }
   pair<string, string> hostAndPort;
-  Util::split(hostAndPort, tempUrl.substr(hp, hep-hp), ':');
+  Util::split(hostAndPort, hostPart, ':');
   host = hostAndPort.first;
   if(hostAndPort.second != "") {
-    port = (int)strtol(hostAndPort.second.c_str(), NULL, 10);
-    if(!(0 < port && port <= 65535)) {
+    try {
+      port = Util::parseInt(hostAndPort.second);
+      if(!(0 < port && port <= 65535)) {
+	return false;
+      }
+    } catch(RecoverableException* e) {
+      delete e;
       return false;
     }
   } else {
     // If port is not specified, then we set it to default port of its protocol..
     port = defPort;
   }
+  // find directory and file part
   string::size_type direp = tempUrl.find_last_of("/");
   if(direp == string::npos || direp <= hep) {
     dir = "/";
     direp = hep;
   } else {
-    dir = tempUrl.substr(hep, direp-hep);
+    string rawDir = tempUrl.substr(hep, direp-hep);
+    string::size_type p = rawDir.find_first_not_of("/");
+    if(p != string::npos) {
+      rawDir.erase(0, p-1);
+    }
+    p = rawDir.find_last_not_of("/");
+    if(p != string::npos) {
+      rawDir.erase(p+1);
+    }
+    dir = rawDir;
   }
   if(tempUrl.size() > direp+1) {
     file = tempUrl.substr(direp+1);
@@ -131,17 +151,30 @@ bool Request::parseUrl(const string& url) {
   return true;
 }
 
-AuthConfigHandle Request::resolveHttpAuthConfig()
+bool Request::isHexNumber(const char c) const
 {
-  return _httpAuthResolver->resolveAuthConfig(getHost());
+  return '0' <= c && c <= '9' || 'A' <= c && c <= 'F' || 'a' <= c && c <= 'f';
 }
 
-AuthConfigHandle Request::resolveFtpAuthConfig()
+string Request::urlencode(const string& src) const
 {
-  return _ftpAuthResolver->resolveAuthConfig(getHost());
-}
-
-AuthConfigHandle Request::resolveHttpProxyAuthConfig()
-{
-  return _httpProxyAuthResolver->resolveAuthConfig(getHost());
+  int32_t lastIndex = src.size()-1;
+  string result = src+"  ";
+  for(int32_t index = lastIndex; index >= 0; --index) {
+    const char c = result[index];
+    // '/' is not urlencoded because src is expected to be a path.
+    if(Util::shouldUrlencode(c)) {
+      if(c == '%') {
+	if(!isHexNumber(result[index+1]) || !isHexNumber(result[index+2])) {
+	  result.replace(index, 1, "%25");
+	}
+      } else {
+	char temp[4];
+	sprintf(temp, "%%%02x", c);
+	temp[3] = '\0';
+	result.replace(index, 1, temp);
+      }
+    }
+  }
+  return result.substr(0, result.size()-2);
 }
