@@ -36,8 +36,6 @@
 #define _D_REQUEST_H_
 #include "common.h"
 #include "CookieBox.h"
-#include "AuthConfig.h"
-#include "AuthResolver.h"
 
 #define SAFE_CHARS "abcdefghijklmnopqrstuvwxyz"\
 "ABCDEFGHIJKLMNOPQRSTUVWXYZ"\
@@ -48,9 +46,15 @@
 "%"\
 "#"
 
-#define METALINK_MARK "#!metalink3!"
-
 class Request {
+public:
+  enum TRACKER_EVENT {
+    AUTO,
+    STARTED,
+    STOPPED,
+    COMPLETED,
+    AFTER_COMPLETED
+  };
 private:
   string url;
   string currentUrl;
@@ -64,21 +68,24 @@ private:
   string referer;
   string protocol;
   string host;
-  int port;
+  int32_t port;
   string dir;
   string file;
-  int tryCount;
-  int trackerEvent;
+  int32_t tryCount;
+  TRACKER_EVENT trackerEvent;
   bool keepAlive;
   string method;
 
-  AuthResolverHandle _httpAuthResolver;
+  string _username;
 
-  AuthResolverHandle _httpProxyAuthResolver;
-
-  AuthResolverHandle _ftpAuthResolver;
+  string _password;
 
   bool parseUrl(const string& url);
+
+  bool isHexNumber(const char c) const;
+
+  string urlencode(const string& src) const;
+
 public:
   CookieBoxHandle cookieBox;
 public:
@@ -95,7 +102,7 @@ public:
   bool resetUrl();
   void resetTryCount() { tryCount = 0; }
   void addTryCount() { tryCount++; }
-  int getTryCount() const { return tryCount; }
+  int32_t getTryCount() const { return tryCount; }
   //bool noMoreTry() const { return tryCount >= PREF_MAX_TRY; }
 
   string getUrl() const { return url; }
@@ -105,39 +112,28 @@ public:
   void setReferer(const string& url) { referer = previousUrl = url; }
   string getProtocol() const { return protocol; }
   string getHost() const { return host; }
-  int getPort() const { return port; }
+  int32_t getPort() const { return port; }
   string getDir() const { return dir; }
   string getFile() const { return file;}
   bool isKeepAlive() const { return keepAlive; }
   void setKeepAlive(bool keepAlive) { this->keepAlive = keepAlive; }
-  void setTrackerEvent(int event) { trackerEvent = event; }
-  int getTrackerEvent() const { return trackerEvent; }
+  void setTrackerEvent(TRACKER_EVENT event) { trackerEvent = event; }
+  TRACKER_EVENT getTrackerEvent() const { return trackerEvent; }
 
   void setMethod(const string& method) {
     this->method = method;
   }
 
-  void setHttpAuthResolver(const AuthResolverHandle& authResolver)
+  const string& getUsername() const
   {
-    _httpAuthResolver = authResolver;
+    return _username;
   }
 
-  void setHttpProxyAuthResolver(const AuthResolverHandle& authResolver)
+  const string& getPassword() const
   {
-    _httpProxyAuthResolver = authResolver;
+    return _password;
   }
 
-  void setFtpAuthResolver(const AuthResolverHandle& authResolver)
-  {
-    _ftpAuthResolver = authResolver;
-  }
-
-  AuthConfigHandle resolveHttpAuthConfig();
-
-  AuthConfigHandle resolveFtpAuthConfig();
-
-  AuthConfigHandle resolveHttpProxyAuthConfig();
-
   const string& getMethod() const {
     return method;
   }
@@ -145,14 +141,6 @@ public:
   static const string METHOD_GET;
   static const string METHOD_HEAD;
 
-  enum TRACKER_EVENT {
-    AUTO,
-    STARTED,
-    STOPPED,
-    COMPLETED,
-    AFTER_COMPLETED
-  };
-
 };
 
 typedef SharedHandle<Request> RequestHandle;
