@@ -36,57 +36,44 @@
 #define _D_SEGMENT_H_
 
 #include "common.h"
-#include <deque>
 
-using namespace std;
+class Piece;
+typedef SharedHandle<Piece> PieceHandle;
 
 class Segment {
 public:
-  int index;
-  int length;
-  int segmentLength;
-  int writtenLength;
+  virtual ~Segment() {}
 
-  Segment(int index, int length, int segmentLength, int writtenLength = 0)
-    :index(index), length(length), segmentLength(segmentLength),
-     writtenLength(writtenLength) {}
+  virtual bool complete() const = 0;
 
-  Segment():index(-1), length(0), segmentLength(0), writtenLength(0) {}
+  virtual int32_t getIndex() const = 0;
 
-  bool complete() const {
-    return length <= writtenLength;
-  }
+  virtual int64_t getPosition() const = 0;
+  
+  virtual int64_t getPositionToWrite() const = 0;
 
-  bool isNull() const {
-    return index == -1;
-  }
+  virtual int32_t getLength() const = 0;
 
-  int64_t getPosition() const {
-    return ((int64_t)index)*segmentLength;
-  }
+  virtual int32_t getSegmentLength() const = 0;
 
-  int64_t getPositionToWrite() const
+  virtual int32_t getWrittenLength() const = 0;
+
+  virtual int32_t getOverflowLength() const = 0;
+
+  virtual void updateWrittenLength(int32_t bytes) = 0;
+
+  virtual void clear() = 0;
+
+  virtual PieceHandle getPiece() const = 0;
+
+  bool operator==(const Segment& segment) const
   {
-    return getPosition()+writtenLength;
+    return getIndex() == segment.getIndex();
   }
-
-  bool operator==(const Segment& segment) const {
-    return index == segment.index &&
-      length == segment.length &&
-      segmentLength == segment.segmentLength &&
-      writtenLength == segment.writtenLength;
-  }
-
-  bool operator!=(const Segment& segment) const {
-    return !(*this == segment);
-  }
-
-  friend ostream& operator<<(ostream& o, const Segment& segment);
 };
 
-ostream& operator<<(ostream& o, const Segment& segment);
-
 typedef SharedHandle<Segment> SegmentHandle;
+typedef deque<SegmentHandle> Segments;
 
 #endif // _D_SEGMENT_H_
 
