@@ -36,11 +36,19 @@
 #include "Util.h"
 #include "DlAbortEx.h"
 #include "message.h"
-#include <time.h>
+#include "a2io.h"
+#include "a2time.h"
 #include <stdarg.h>
-#include <stdio.h>
 #include <errno.h>
 
+#if !defined(va_copy)
+# if defined(__va_copy)
+#  define va_copy(dest, src) __va_copy(dest, src)
+# else
+#  define va_copy(dest, src) (dest = src)
+# endif
+#endif
+
 #define WRITE_LOG(LEVEL, MSG) \
 va_list ap;\
 va_start(ap, MSG);\
@@ -60,7 +68,7 @@ SimpleLogger::~SimpleLogger() {
 }
 
 void SimpleLogger::openFile(const string& filename) {
-  file = fopen(filename.c_str(), "a");
+  file = fopen(filename.c_str(), "ab");
   if(file == NULL) {
     throw new DlAbortEx(EX_FILE_OPEN, filename.c_str(), strerror(errno));
   }
@@ -72,7 +80,7 @@ void SimpleLogger::closeFile() {
   }
 }
 
-void SimpleLogger::setStdout(int level, bool enabled) {
+void SimpleLogger::setStdout(Logger::LEVEL level, bool enabled) {
   if(enabled) {
     stdoutField |= level;
   } else {
@@ -84,8 +92,10 @@ void SimpleLogger::writeHeader(FILE* file, string date, string level) const {
   fprintf(file, "%s %s - ", date.c_str(), level.c_str());
 }
 
-void SimpleLogger::writeLog(FILE* file, int level, const char* msg, va_list ap, Exception* e, bool printHeader) const
+void SimpleLogger::writeLog(FILE* file, Logger::LEVEL level, const char* msg, va_list ap, Exception* e, bool printHeader) const
 {
+  va_list apCopy;
+  va_copy(apCopy, ap);
   string levelStr;
   switch(level) {
   case Logger::DEBUG:
@@ -114,7 +124,7 @@ void SimpleLogger::writeLog(FILE* file, int level, const char* msg, va_list ap,
   if(printHeader) {
     writeHeader(file, datestr, levelStr);
   }
-  vfprintf(file, string(Util::replace(msg, "\r", "")+"\n").c_str(), ap);
+  vfprintf(file, string(Util::replace(msg, "\r", "")+"\n").c_str(), apCopy);
   for(Exception* nestedEx = e; nestedEx; nestedEx = nestedEx->getCause()) {
     // TODO a quick hack not to print header in console
     if(printHeader) {
@@ -123,9 +133,10 @@ void SimpleLogger::writeLog(FILE* file, int level, const char* msg, va_list ap,
     fprintf(file, "exception: %s\n", Util::replace(nestedEx->getMsg(), "\r", "").c_str());
   }
   fflush(file);
+  va_end(apCopy);
 }
 
-void SimpleLogger::writeFile(int level, const char* msg, va_list ap, Exception* e) const {
+void SimpleLogger::writeFile(Logger::LEVEL level, const char* msg, va_list ap, Exception* e) const {
   writeLog(file, level, msg, ap, e);
   if(stdoutField&level) {
     fprintf(stdout, "\n");
