@@ -36,8 +36,8 @@
 #define _D_SIMPLE_RANDOMIZER_H_
 
 #include "Randomizer.h"
+#include "a2time.h"
 #include <stdlib.h>
-#include <time.h>
 
 class SimpleRandomizer : public Randomizer {
 private:
@@ -54,17 +54,25 @@ public:
   }
   
   static void init() {
-    srandom(time(0));
+    srand(time(0));
   }
 
   virtual ~SimpleRandomizer() {}
 
-  virtual int getRandomNumber() {
-    return random();
+  virtual long int getRandomNumber() {
+    return rand();
   }
 
-  virtual int getMaxRandomNumber() {
-    return RAND_MAX;
+  virtual long int getMaxRandomNumber() {
+      return RAND_MAX;
+  }
+
+  /**
+   * Returns random number in [0, to).
+   */
+  virtual long int getRandomNumber(long int to)
+  {
+    return(int32_t)(((double)to)*getRandomNumber()/(getMaxRandomNumber()+1.0));
   }
 };
 
