@@ -33,30 +33,40 @@
  */
 /* copyright --> */
 #include "Util.h"
-#include "DlAbortEx.h"
 #include "File.h"
 #include "message.h"
+#include "Randomizer.h"
+#include "a2netcompat.h"
+#include "a2time.h"
+#include "DlAbortEx.h"
+#include "BitfieldMan.h"
+#include "DefaultDiskWriter.h"
+#include "BinaryStream.h"
+#include "FatalException.h"
 #include <ctype.h>
 #include <errno.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <fcntl.h>
-#include <netinet/in.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
 #include <unistd.h>
 #include <signal.h>
+#include <iomanip>
+#include <sstream>
 
+#ifndef HAVE_SLEEP
+# ifdef HAVE_WINSOCK_H
+#  define WIN32_LEAN_AND_MEAN
+#  include <windows.h>
+# endif // HAVE_WINSOCK_H
+#endif // HAVE_SLEEP
 
 template<typename T>
-string uint2str(T value, bool comma) {
+string uint2str(T value, bool comma)
+{
   string str;
   if(value == 0) {
     str = "0";
     return str;
   }
-  int count = 0;
+  int32_t count = 0;
   while(value) {
     ++count;
     char digit = value%10+'0';
@@ -70,7 +80,8 @@ string uint2str(T value, bool comma) {
 }
 
 template<typename T>
-string int2str(T value, bool comma) {
+string int2str(T value, bool comma)
+{
   bool flag = false;
   if(value < 0) {
     flag = true;
@@ -85,23 +96,28 @@ string int2str(T value, bool comma) {
 
 
 
-string Util::uitos(uint16_t value, bool comma) {
+string Util::uitos(uint16_t value, bool comma)
+{
   return uint2str<uint16_t>(value, comma);
 }
 
-string Util::itos(int16_t value, bool comma) {
+string Util::itos(int16_t value, bool comma)
+{
   return int2str<int16_t>(value, comma);
 }
 
-string Util::uitos(uint32_t value, bool comma) {
+string Util::uitos(uint32_t value, bool comma)
+{
   return uint2str<uint32_t>(value, comma);
 }
 
-string Util::itos(int32_t value, bool comma) {
+string Util::itos(int32_t value, bool comma)
+{
   return int2str<int32_t>(value, comma);
 }
 
-string Util::ullitos(uint64_t value, bool comma) {
+string Util::ullitos(uint64_t value, bool comma)
+{
   return uint2str<uint64_t>(value, comma);
 }
 
@@ -110,9 +126,10 @@ string Util::llitos(int64_t value, bool comma)
   return int2str<int64_t>(value, comma);
 }
 
-string Util::trim(const string& src) {
-  string::size_type sp = src.find_first_not_of("\r\n\t ");
-  string::size_type ep = src.find_last_not_of("\r\n\t ");
+string Util::trim(const string& src, const string& trimCharset)
+{
+  string::size_type sp = src.find_first_not_of(trimCharset);
+  string::size_type ep = src.find_last_not_of(trimCharset);
   if(sp == string::npos || ep == string::npos) {
     return "";
   } else {
@@ -120,7 +137,8 @@ string Util::trim(const string& src) {
   }
 }
 
-void Util::split(pair<string, string>& hp, const string& src, char delim) {
+void Util::split(pair<string, string>& hp, const string& src, char delim)
+{
   hp.first = "";
   hp.second = "";
   string::size_type p = src.find(delim);
@@ -149,15 +167,15 @@ pair<string, string> Util::split(const string& src, const string& delims)
   return hp;
 }
 
-long long int Util::difftv(struct timeval tv1, struct timeval tv2) {
+int64_t Util::difftv(struct timeval tv1, struct timeval tv2) {
   if(tv1.tv_sec < tv2.tv_sec || tv1.tv_sec == tv2.tv_sec && tv1.tv_usec < tv2.tv_usec) {
     return 0;
   }
-  return ((long long int)(tv1.tv_sec-tv2.tv_sec)*1000000+
+  return ((int64_t)(tv1.tv_sec-tv2.tv_sec)*1000000+
 	  tv1.tv_usec-tv2.tv_usec);
 }
 
-int Util::difftvsec(struct timeval tv1, struct timeval tv2) {
+int32_t Util::difftvsec(struct timeval tv1, struct timeval tv2) {
   if(tv1.tv_sec < tv2.tv_sec) {
     return 0;
   }
@@ -234,18 +252,26 @@ string Util::replace(const string& target, const string& oldstr, const string& n
   return result;
 }
 
-string Util::urlencode(const unsigned char* target, int len) {
+bool Util::shouldUrlencode(const char c)
+{
+  return !(// ALPHA
+	   'A' <= c && c <= 'Z' || 'a' <= c && c <= 'z' ||
+	   // DIGIT
+	   '0' <= c && c <= '9' ||
+	   // safe
+	   '$' == c || '-' == c || '_' == c || '.' == c ||
+	   // extra
+	   '!' == c || '*' == c || '\'' == c ||'(' == c ||
+	   ')' == c || ',' == c ||
+	   // reserved
+	   ';' == c || '/' == c || '?' == c  || ':' == c ||
+	   '@' == c || '&' == c || '=' == c || '+' == c);	   
+}
+
+string Util::urlencode(const unsigned char* target, int32_t len) {
   string dest;
-  for(int i = 0; i < len; i++) {
-    if(!('0' <= target[i] && target[i] <= '9' ||
-	 'A' <= target[i] && target[i] <= 'Z' ||
-	 'a' <= target[i] && target[i] <= 'z' ||
-	 '$' == target[i] || '-' == target[i] ||
-	 '_' == target[i] || '.' == target[i] ||
-	 '+' == target[i] || '!' == target[i] ||
-	 '*' == target[i] || '\'' == target[i] ||
-	 '(' == target[i] || ')' == target[i] ||
-	 ',' == target[i])) {
+  for(int32_t i = 0; i < len; i++) {
+    if(shouldUrlencode(target[i])) {
       char temp[4];
       sprintf(temp, "%%%02x", target[i]);
       temp[sizeof(temp)-1] = '\0';
@@ -257,9 +283,9 @@ string Util::urlencode(const unsigned char* target, int len) {
   return dest;
 }
 
-string Util::torrentUrlencode(const unsigned char* target, int len) {
+string Util::torrentUrlencode(const unsigned char* target, int32_t len) {
   string dest;
-  for(int i = 0; i < len; i++) {
+  for(int32_t i = 0; i < len; i++) {
     if('0' <= target[i] && target[i] <= '9' ||
        'A' <= target[i] && target[i] <= 'Z' ||
        'a' <= target[i] && target[i] <= 'z') {
@@ -281,11 +307,7 @@ string Util::urldecode(const string& target) {
     if(*itr == '%') {
       if(itr+1 != target.end() && itr+2 != target.end() &&
 	 isxdigit(*(itr+1)) && isxdigit(*(itr+2))) {
-	char temp[3];
-	temp[0] = *(itr+1);
-	temp[1] = *(itr+2);
-	temp[2] = '\0';
-	result += strtol(temp, 0, 16);
+	result += Util::parseInt(string(itr+1, itr+3), 16);
 	itr += 2;
       } else {
 	result += *itr;
@@ -297,9 +319,9 @@ string Util::urldecode(const string& target) {
   return result;
 }
 
-string Util::toHex(const unsigned char* src, int len) {
+string Util::toHex(const unsigned char* src, int32_t len) {
   char* temp = new char[len*2+1];
-  for(int i = 0; i < len; i++) {
+  for(int32_t i = 0; i < len; i++) {
     sprintf(temp+i*2, "%02x", src[i]);
   }
   temp[len*2] = '\0';
@@ -318,57 +340,41 @@ void Util::fileCopy(const string& dest, const string& src) {
   rangedFileCopy(dest, src, 0, file.size());
 }
 
-void Util::rangedFileCopy(const string& dest, const string& src, long long int srcOffset, long long int length) {
-  int bufSize = 4096;
-  char buf[bufSize];
-  int destFd = -1;
-  int srcFd = -1;
-  try {
-    if((destFd = open(dest.c_str(), O_CREAT|O_WRONLY|O_TRUNC, OPEN_MODE)) == -1) {
-      throw new DlAbortEx(EX_FILE_OPEN, dest.c_str(), strerror(errno));
+void Util::rangedFileCopy(const string& dest, const string& src, int64_t srcOffset, int64_t length)
+{
+  int32_t bufSize = 4096;
+  unsigned char buf[bufSize];
+  DefaultDiskWriter srcdw;
+  DefaultDiskWriter destdw;
+
+  srcdw.openExistingFile(src);
+  destdw.initAndOpenFile(dest);
+
+  int32_t x = length/bufSize;
+  int32_t r = length%bufSize;
+
+  int64_t initialOffset = srcOffset;
+  for(int32_t i = 0; i < x; ++i) {
+    int32_t readLength = 0;
+    while(readLength < bufSize) {
+      int32_t ret = srcdw.readData(buf, bufSize-readLength, srcOffset);
+      destdw.writeData(buf, ret, srcOffset-initialOffset);
+      srcOffset += ret;
+      readLength += ret;
     }
-    if((srcFd = open(src.c_str(), O_RDONLY, OPEN_MODE)) == -1) {
-      throw new DlAbortEx(EX_FILE_OPEN, src.c_str(), strerror(errno));
-    }
-    if(lseek(srcFd, srcOffset, SEEK_SET) != srcOffset) {
-      throw new DlAbortEx(EX_FILE_SEEK, src.c_str(), strerror(errno));
-    }
-    int x = length/bufSize;
-    int r = length%bufSize;
-    for(int i = 0; i < x; i++) {
-      int readLength;
-      if((readLength = read(srcFd, buf, bufSize)) == -1 || readLength != bufSize) {
-	throw new DlAbortEx(EX_FILE_READ, src.c_str(), strerror(errno));
-      }
-      if(write(destFd, buf, readLength) == -1) {
-	throw new DlAbortEx(EX_FILE_WRITE, dest.c_str(), strerror(errno));
-      }
-    }
-    if(r > 0) {
-      int readLength;
-      if((readLength = read(srcFd, buf, r)) == -1 || readLength != r) {
-	throw new DlAbortEx(EX_FILE_READ, src.c_str(), strerror(errno));
-      }
-      if(write(destFd, buf, r) == -1) {
-	throw new DlAbortEx(EX_FILE_WRITE, dest.c_str(), strerror(errno));
-      }
-    }
-    close(srcFd);
-    close(destFd);
-    srcFd = -1;
-    destFd = -1;
-  } catch(RecoverableException* e) {
-    if(srcFd != -1) {
-      close(srcFd);
-    }
-    if(destFd != -1) {
-      close(destFd);
-    }
-    throw;
   }
+  if(r > 0) {
+    int32_t readLength = 0;
+    while(readLength < r) { 
+      int32_t ret = srcdw.readData(buf, r-readLength, srcOffset);
+      destdw.writeData(buf, ret, srcOffset-initialOffset);
+      srcOffset += ret;
+      readLength += ret;
+    }
+  }     
 }
 
-bool Util::isPowerOf(int num, int base) {
+bool Util::isPowerOf(int32_t num, int32_t base) {
   if(base <= 0) { return false; }
   if(base == 1) { return true; }
 
@@ -381,14 +387,14 @@ bool Util::isPowerOf(int num, int base) {
   return false;
 }
 
-string Util::secfmt(int sec) {
+string Util::secfmt(int32_t sec) {
   string str;
   if(sec >= 3600) {
     str = itos(sec/3600)+"h";
     sec %= 3600;
   }
   if(sec >= 60) {
-    int min = sec/60;
+    int32_t min = sec/60;
     if(min < 10) {
       str += "0";
     }
@@ -402,7 +408,7 @@ string Util::secfmt(int sec) {
   return str;
 }
 
-int Util::expandBuffer(char** pbuf, int curLength, int newLength) {
+int32_t Util::expandBuffer(char** pbuf, int32_t curLength, int32_t newLength) {
   char* newbuf = new char[newLength];
   memcpy(newbuf, *pbuf, curLength);
   delete [] *pbuf;
@@ -410,11 +416,11 @@ int Util::expandBuffer(char** pbuf, int curLength, int newLength) {
   return newLength;
 }
 
-int getNum(const char* buf, int offset, int length) {
+int32_t getNum(const char* buf, int32_t offset, int32_t length) {
   char* temp = new char[length+1];
   memcpy(temp, buf+offset, length);
   temp[length] = '\0';
-  int x = strtol(temp, NULL, 10);
+  int32_t x = strtol(temp, NULL, 10);
   delete [] temp;
   return x;
 }
@@ -430,18 +436,18 @@ void unfoldSubRange(const string& src, Integers& range) {
     range.push_back(atoi(src.c_str()));
   } else {
     if(src.at(p) == ',') {
-      int num = getNum(src.c_str(), 0, p);
+      int32_t num = getNum(src.c_str(), 0, p);
       range.push_back(num);
       unfoldSubRange(src.substr(p+1), range);
     } else if(src.at(p) == '-') {
-      int rightNumBegin = p+1;
+      int32_t rightNumBegin = p+1;
       string::size_type nextDelim = src.find_first_of(",", rightNumBegin);
       if(nextDelim == string::npos) {
 	nextDelim = src.size();
       }
-      int left = getNum(src.c_str(), 0, p);
-      int right = getNum(src.c_str(), rightNumBegin, nextDelim-rightNumBegin);
-      for(int i = left; i <= right; i++) {
+      int32_t left = getNum(src.c_str(), 0, p);
+      int32_t right = getNum(src.c_str(), rightNumBegin, nextDelim-rightNumBegin);
+      for(int32_t i = left; i <= right; i++) {
 	range.push_back(i);
       }
       if(src.size() > nextDelim) {
@@ -457,118 +463,97 @@ void Util::unfoldRange(const string& src, Integers& range) {
   range.erase(unique(range.begin(), range.end()), range.end());
 }
 
+int32_t Util::parseInt(const string& s, int32_t base)
+{
+  string trimed = Util::trim(s);
+  if(trimed.empty()) {
+    throw new DlAbortEx(MSG_STRING_INTEGER_CONVERSION_FAILURE,
+			"empty string");
+  }
+  char* stop;
+  errno = 0;
+  long int v = strtol(trimed.c_str(), &stop, base);
+  if(*stop != '\0') {
+    throw new DlAbortEx(MSG_STRING_INTEGER_CONVERSION_FAILURE,
+			trimed.c_str());
+  } else if((v == LONG_MIN || v == LONG_MAX) && errno == ERANGE || v > INT32_MAX || v < INT32_MIN) {
+    throw new DlAbortEx(MSG_STRING_INTEGER_CONVERSION_FAILURE,
+			trimed.c_str());
+  }
+  return v;
+}
+
+int64_t Util::parseLLInt(const string& s, int32_t base)
+{
+  string trimed = Util::trim(s);
+  if(trimed.empty()) {
+    throw new DlAbortEx(MSG_STRING_INTEGER_CONVERSION_FAILURE,
+			"empty string");
+  }
+  char* stop;
+  errno = 0;
+  int64_t v = strtoll(trimed.c_str(), &stop, base);
+  if(*stop != '\0') {
+    throw new DlAbortEx(MSG_STRING_INTEGER_CONVERSION_FAILURE,
+			trimed.c_str());
+  } else if((v == INT64_MIN || v == INT64_MAX) && errno == ERANGE) {
+    throw new DlAbortEx(MSG_STRING_INTEGER_CONVERSION_FAILURE,
+			trimed.c_str());
+  }
+  return v;
+}
+
+IntSequence Util::parseIntRange(const string& src)
+{
+  IntSequence::Values values;
+  string temp = src;
+  while(temp.size()) {
+    pair<string, string> p = Util::split(temp, ",");
+    temp = p.second;
+    if(p.first.empty()) {
+      continue;
+    }
+    if(p.first.find("-") == string::npos) {
+      int32_t v = Util::parseInt(p.first.c_str());
+      values.push_back(IntSequence::Value(v, v+1));
+    } else {
+      pair<string, string> vp = Util::split(p.first.c_str(), "-");
+      if(vp.first.empty() || vp.second.empty()) {
+	throw new DlAbortEx(MSG_INCOMPLETE_RANGE, p.first.c_str());
+      }
+      int32_t v1 = Util::parseInt(vp.first.c_str());
+      int32_t v2 = Util::parseInt(vp.second.c_str());
+      values.push_back(IntSequence::Value(v1, v2+1));
+    } 
+  }
+  return values;
+}
+
 string Util::getContentDispositionFilename(const string& header) {
-  string::size_type attributesp = header.find("filename=\"");
+  string keyName = "filename=";
+  string::size_type attributesp = header.find(keyName);
   if(attributesp == string::npos) {
     return "";
   }
-  string::size_type filenamesp = attributesp+strlen("filename=\"");
-  string::size_type filenameep = header.find("\"", filenamesp);
-  if(filenameep == string::npos) {
+  string::size_type filenamesp = attributesp+strlen(keyName.c_str());
+  string::size_type filenameep;
+  if(filenamesp == header.size()) {
     return "";
   }
-  return trim(header.substr(filenamesp, filenameep-filenamesp));
-}
-
-#ifdef ENABLE_MESSAGE_DIGEST
-void Util::sha1Sum(unsigned char* digest, const void* data, int dataLength) {
-  MessageDigestContext ctx(DIGEST_ALGO_SHA1);
-  ctx.digestInit();
-  ctx.digestUpdate(data, dataLength);
-  ctx.digestFinal(digest);
-}
-
-string Util::simpleMessageDigest(const string& data) {
-  unsigned char checksum[20];
-  sha1Sum(checksum, data.c_str(), data.size());
-  return Util::toHex(checksum, sizeof(checksum));
-}
-
-#endif // ENABLE_MESSAGE_DIGEST
-
-#ifdef ENABLE_MESSAGE_DIGEST
-void Util::fileChecksum(const string& filename, unsigned char* digest,
-			MessageDigestContext::DigestAlgo algo) {
-  MessageDigestContext ctx(algo);
-  ctx.digestInit();
-
-  int BUFLEN = 4096;
-  char buf[BUFLEN];
-
-  int fd;
-  if((fd = open(filename.c_str(), O_RDWR, OPEN_MODE)) < 0) {
-    throw new DlAbortEx(EX_FILE_OPEN, filename.c_str(), strerror(errno));
-  }
-  while(1) {
-    int size = read(fd, buf, BUFLEN);
-    if(size == -1) {
-      if(errno == EINTR) {
-	continue;
-      } else {
-	close(fd);
-	throw new DlAbortEx(EX_FILE_READ, filename.c_str(), strerror(errno));
-      }
-    } else if(size > 0) {
-      ctx.digestUpdate(buf, size);
-    }
-    if(size < BUFLEN) {
-      break;
-    }
-  }
-  ctx.digestFinal(digest);
-}
-#endif // ENABLE_MESSAGE_DIGEST
-
-#ifdef ENABLE_BITTORRENT
-Integers Util::computeFastSet(string ipaddr, const unsigned char* infoHash,
-			     int pieces, int fastSetSize) {
-  Integers fastSet;
-  struct in_addr saddr;
-  if(inet_aton(ipaddr.c_str(), &saddr) == 0) {
-    abort();
-  }
-  unsigned char tx[24];
-  memcpy(tx, (void*)&saddr.s_addr, 4);
-  if((tx[0] & 0x80) == 0 || (tx[0] & 0x40) == 0) {
-    tx[2] = 0x00;
-    tx[3] = 0x00;
+  
+  if(header[filenamesp] == '\'' || header[filenamesp] == '"') {
+    char quoteChar = header[filenamesp];
+    filenameep = header.find(quoteChar, filenamesp+1);
   } else {
-    tx[3] = 0x00;
+    filenameep = header.find(';', filenamesp);
   }
-  memcpy(tx+4, infoHash, 20);
-  unsigned char x[20];
-  sha1Sum(x, tx, 24);
-  while((int)fastSet.size() < fastSetSize) {
-    for(int i = 0; i < 5 && (int)fastSet.size() < fastSetSize; i++) {
-      int j = i*4;
-      unsigned int ny;
-      memcpy(&ny, x+j, 4);
-      unsigned int y = ntohl(ny);
-      int index = y%pieces;
-      if(find(fastSet.begin(), fastSet.end(), index) == fastSet.end()) {
-	fastSet.push_back(index);
-      }
-    }
-    unsigned char temp[20];
-    sha1Sum(temp, x, 20);
-    memcpy(x, temp, sizeof(x));
+  if(filenameep == string::npos) {
+    filenameep = header.size();
   }
-  return fastSet;
+  return trim(header.substr(filenamesp, filenameep-filenamesp), "\r\n '\"");
 }
-#endif // ENABLE_BITTORRENT
 
-/*
-int Util::countBit(unsigned int n) {
-  int count = 0;
-  while(n > 0) {
-    count++;
-    n &= (n-1);
-  }
-  return count;
-}
-*/
-
-static int nbits[] = {
+static int32_t nbits[] = {
   0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4, 
   1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 
   1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5, 
@@ -595,44 +580,25 @@ int32_t Util::countBit(uint32_t n) {
     nbits[(n >> 24)&0xffu];
 }
 
-string Util::randomAlpha(int length) {
+string Util::randomAlpha(int32_t length, const RandomizerHandle& randomizer) {
+  static const char *random_chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
   string str;
-  for(int i = 0; i < length; i++) {
-    int index = (int)(((double)52)*random()/(RAND_MAX+1.0));
-    char ch;
-    if(index < 26) {
-      ch = (char)('A'+index);
-    } else {
-      ch = (char)('a'+index-26);
-    }
-    str += ch;
+  for(int32_t i = 0; i < length; i++) {
+    int32_t index = randomizer->getRandomNumber(strlen(random_chars));
+    str += random_chars[index];
   }
   return str;
 }
 
-class UpperCase {
-public:
-  void operator()(char& ch) {
-    ch = toupper(ch);
-  }
-};
-
 string Util::toUpper(const string& src) {
   string temp = src;
-  for_each(temp.begin(), temp.end(), UpperCase());
+  transform(temp.begin(), temp.end(), temp.begin(), ::toupper);
   return temp;
 }
 
-class LowerCase {
-public:
-  void operator()(char& ch) {
-    ch = tolower(ch);
-  }
-};
-
 string Util::toLower(const string& src) {
   string temp = src;
-  for_each(temp.begin(), temp.end(), LowerCase());
+  transform(temp.begin(), temp.end(), temp.begin(), ::tolower);
   return temp;
 }
 
@@ -645,12 +611,16 @@ bool Util::isNumbersAndDotsNotation(const string& name) {
   }
 }
 
-void Util::setGlobalSignalHandler(int signal, void (*handler)(int), int flags) {
+void Util::setGlobalSignalHandler(int32_t sig, void (*handler)(int), int32_t flags) {
+#ifdef HAVE_SIGACTION
   struct sigaction sigact;
   sigact.sa_handler = handler;
   sigact.sa_flags = flags;
   sigemptyset(&sigact.sa_mask);
-  sigaction(signal, &sigact, NULL);
+  sigaction(sig, &sigact, NULL);
+#else
+  signal(sig, handler);
+#endif // HAVE_SIGACTION
 }
 
 void Util::indexRange(int32_t& startIndex, int32_t& endIndex,
@@ -678,7 +648,7 @@ int64_t Util::getRealSize(const string& sizeWithUnit)
 {
   string::size_type p = sizeWithUnit.find_first_of("KM");
   string size;
-  int mult = 1;
+  int32_t mult = 1;
   if(p == string::npos) {
     size = sizeWithUnit;
   } else {
@@ -689,7 +659,15 @@ int64_t Util::getRealSize(const string& sizeWithUnit)
     }
     size = sizeWithUnit.substr(0, p);
   }
-  return strtoll(size.c_str(), 0, 10)*mult;
+  int64_t v = Util::parseLLInt(size);
+
+  if(v < 0) {
+    throw new DlAbortEx("Negative value detected: %s", sizeWithUnit.c_str());
+  } else if(v*mult < 0) {
+    throw new DlAbortEx(MSG_STRING_INTEGER_CONVERSION_FAILURE,
+			"overflow/underflow");
+  }
+  return v*mult;
 }
 
 string Util::abbrevSize(int64_t size)
@@ -705,8 +683,8 @@ string Util::abbrevSize(int64_t size)
   for(; i < numUnit-1 && size >= 1024; ++i) {
     r = size&0x3ff;
     size >>= 10;
-  } 
-  return Util::llitos(size, true)+"."+Util::itos(r*10/1024)+units[i];
+  }
+  return Util::llitos(size, true)+"."+Util::itos(r*10/1024)+units[i]+"i";
 }
 
 time_t Util::httpGMT(const string& httpStdTime)
@@ -717,3 +695,173 @@ time_t Util::httpGMT(const string& httpStdTime)
   time_t thetime = timegm(&tm);
   return thetime;
 }
+
+void Util::toStream(ostream& os, const FileEntries& fileEntries)
+{
+  os << _("Files:") << "\n";
+  os << "idx|path/length" << "\n";
+  os << "===+===========================================================================" << "\n";
+  int32_t count = 1;
+  for(FileEntries::const_iterator itr = fileEntries.begin();
+      itr != fileEntries.end(); count++, itr++) {
+    os << setw(3) << count << "|" << (*itr)->getPath() << "\n";
+    os << "   |" << Util::abbrevSize((*itr)->getLength()) << "B" << "\n";
+    os << "---+---------------------------------------------------------------------------" << "\n";
+  }
+}
+
+void Util::sleep(long seconds) {
+#ifdef HAVE_SLEEP
+  ::sleep(seconds);
+#elif defined(HAVE_USLEEP)
+  ::usleep(seconds * 1000000);
+#elif defined(HAVE_WINSOCK2_H)
+  ::Sleep(seconds * 1000);
+#else
+#error no sleep function is available (nanosleep?)
+#endif
+}
+
+void Util::usleep(long microseconds) {
+#ifdef HAVE_USLEEP
+  ::usleep(microseconds);
+#elif defined(HAVE_WINSOCK2_H)
+
+	LARGE_INTEGER current, freq, end;
+
+	static enum {GET_FREQUENCY, GET_MICROSECONDS, SKIP_MICROSECONDS} state = GET_FREQUENCY;
+
+	if (state == GET_FREQUENCY) {
+		if (QueryPerformanceFrequency(&freq))
+			state = GET_MICROSECONDS;
+		else
+			state = SKIP_MICROSECONDS;
+	}
+	
+	long msec = microseconds / 1000;
+	microseconds %= 1000;    
+
+	if (state == GET_MICROSECONDS && microseconds) {
+		QueryPerformanceCounter(&end);
+
+		end.QuadPart += (freq.QuadPart * microseconds) / 1000000;
+
+		while (QueryPerformanceCounter(&current) && (current.QuadPart <= end.QuadPart))
+			/* noop */ ;
+	}
+
+	if (msec)
+		Sleep(msec);
+#else
+	#error no usleep function is available (nanosleep?)
+#endif
+}
+
+bool Util::isNumber(const string& what)
+{
+  if(what.empty()) {
+    return false;
+  }
+  for(uint32_t i = 0; i < what.size(); ++i) {
+    if(!isdigit(what[i])) {
+      return false;
+    }
+  }
+  return true;
+}
+
+bool Util::isLowercase(const string& what)
+{
+  if(what.empty()) {
+    return false;
+  }
+  for(uint32_t i = 0; i < what.size(); ++i) {
+    if(!('a' <= what[i] && what[i] <= 'z')) {
+      return false;
+    }
+  }
+  return true;
+}
+
+bool Util::isUppercase(const string& what)
+{
+  if(what.empty()) {
+    return false;
+  }
+  for(uint32_t i = 0; i < what.size(); ++i) {
+    if(!('A' <= what[i] && what[i] <= 'Z')) {
+      return false;
+    }
+  }
+  return true;
+}
+
+int32_t Util::alphaToNum(const string& alphabets)
+{
+  if(alphabets.empty()) {
+    return 0;
+  }
+  char base;
+  if(islower(alphabets[0])) {
+    base = 'a';
+  } else {
+    base = 'A';
+  }
+  int32_t num = 0;
+  for(uint32_t i = 0; i < alphabets.size(); ++i) {
+    int32_t v = alphabets[i]-base;
+    num = num*26+v;
+  }
+  return num;
+}
+
+void Util::mkdirs(const string& dirpath)
+{
+  File dir(dirpath);
+  if(dir.isDir()) {
+    // do nothing
+  } else if(dir.exists()) {
+    throw new DlAbortEx(EX_MAKE_DIR, dir.getPath().c_str(), "File already exists.");
+  } else if(!dir.mkdirs()) {
+    throw new DlAbortEx(EX_MAKE_DIR, dir.getPath().c_str(), strerror(errno));
+  }
+}
+
+void Util::convertBitfield(BitfieldMan* dest, const BitfieldMan* src)
+{
+  for(int32_t index = 0; index < dest->countBlock(); ++index) {
+    if(src->isBitSetOffsetRange((int64_t)index*dest->getBlockLength(),
+				dest->getBlockLength())) {
+      dest->setBit(index);
+    }
+  }
+}
+
+string Util::toString(const BinaryStreamHandle& binaryStream)
+{
+  stringstream strm;
+  char data[2048];
+  while(1) {
+    int32_t dataLength = binaryStream->readData((unsigned char*)data, sizeof(data), strm.tellp());
+    strm.write(data, dataLength);
+    if(dataLength == 0) {
+      break;
+    }
+  }
+  return strm.str();
+}
+
+#ifdef HAVE_POSIX_MEMALIGN
+/**
+ * In linux 2.6, alignment and size should be a multiple of 512.
+ */
+void* Util::allocateAlignedMemory(size_t alignment, size_t size)
+{
+  void* buffer;
+  int32_t res;
+  if((res = posix_memalign(&buffer, alignment, size)) != 0) {
+    throw new FatalException("Error in posix_memalign: %s", strerror(res));
+  }
+  return buffer;
+}
+#endif // HAVE_POSIX_MEMALIGN
