@@ -36,18 +36,23 @@
 #define _D_UTIL_H_
 
 #include "common.h"
-#ifdef ENABLE_MESSAGE_DIGEST
-#include "messageDigest.h"
-#endif // ENABLE_MESSAGE_DIGEST
-#include <string>
+#include "a2time.h"
+#include "FileEntry.h"
+#include "IntSequence.h"
 #include <utility>
 #include <deque>
-#include <sys/time.h>
-#include <stdio.h>
+#include <ostream>
 
-using namespace std;
+class Randomizer;
+typedef SharedHandle<Randomizer> RandomizerHandle;
+class BitfieldMan;
+class BinaryStream;
+typedef SharedHandle<BinaryStream> BinaryStreamHandle;
 
-#define STRTOLL(X) strtoll(X, (char**)NULL, 10);
+#define STRTOLL(X) strtoll(X, (char**)NULL, 10)
+
+#define START_INDEX(OFFSET, PIECE_LENGTH) ((OFFSET)/(PIECE_LENGTH))
+#define END_INDEX(OFFSET, LENGTH, PIECE_LENGTH) (((OFFSET)+(LENGTH)-1)/(PIECE_LENGTH))
 
 class Util {
 public:
@@ -65,15 +70,15 @@ public:
    * assuming tv1 is newer than tv2.
    * If tv1 is older than tv2, then this method returns 0.
    */
-  static long long int difftv(struct timeval tv1, struct timeval tv2);
-  static int difftvsec(struct timeval tv1, struct timeval tv2);
+  static int64_t difftv(struct timeval tv1, struct timeval tv2);
+  static int32_t difftvsec(struct timeval tv1, struct timeval tv2);
   /**
    * Take a string src which is a deliminated list and add its elements
    * into result. result is not cleared before conversion begins.
    */
   static void slice(Strings& result, const string& src, char delim, bool trim = false);
   
-  static string trim(const string& src);
+  static string trim(const string& src, const string& trimCharset = "\r\n\t ");
 
   static bool startsWith(const string& target, const string& part);
 
@@ -81,65 +86,55 @@ public:
 
   static string replace(const string& target, const string& oldstr, const string& newstr);
 
-  static string urlencode(const unsigned char* target, int len);
+  static string urlencode(const unsigned char* target, int32_t len);
 
   static string urlencode(const string& target)
   {
     return urlencode((const unsigned char*)target.c_str(), target.size());
   }
 
+  static bool shouldUrlencode(const char c);
+
   static string urldecode(const string& target);
 
-  static string torrentUrlencode(const unsigned char* target, int len);
+  static string torrentUrlencode(const unsigned char* target, int32_t len);
 
-  static string toHex(const unsigned char* src, int len);
+  static string toHex(const unsigned char* src, int32_t len);
 
   static FILE* openFile(const string& filename, const string& mode);
 
   static void fileCopy(const string& destFile, const string& src);
 
-  static void rangedFileCopy(const string& destFile, const string& src, long long int srcOffset, long long int length);
+  static void rangedFileCopy(const string& destFile, const string& src, int64_t srcOffset, int64_t length);
 
-  static bool isPowerOf(int num, int base);
+  static bool isPowerOf(int32_t num, int32_t base);
 
-  static string secfmt(int sec);
+  static string secfmt(int32_t sec);
 
-  static int expandBuffer(char** pbuf, int curLength, int newLength);
+  static int32_t expandBuffer(char** pbuf, int32_t curLength, int32_t newLength);
 
   static void unfoldRange(const string& src, Integers& range);
 
+  static int32_t parseInt(const string& s, int32_t base = 10);
+
+  static int64_t parseLLInt(const string& s, int32_t base = 10);
+
+  static IntSequence parseIntRange(const string& src);
+
   // this function temporarily put here
   static string getContentDispositionFilename(const string& header);
 
-  // digest must be at least 20 bytes long.
-#ifdef ENABLE_MESSAGE_DIGEST
-  static void sha1Sum(unsigned char* digest, const void* data, int dataLength);
-  static string simpleMessageDigest(const string& data);
-#endif // ENABLE_MESSAGE_DIGEST
-
-  // Before call this method, allocate enough memory to the parameter "digest".
-  // For sha1, you need 20 bytes. For md5, 16 bytes.
-#ifdef ENABLE_MESSAGE_DIGEST
-  static void fileChecksum(const string& filename, unsigned char* digest,
-			   MessageDigestContext::DigestAlgo algo);
-#endif // ENABLE_MESSAGE_DIGEST
-
-#ifdef ENABLE_BITTORRENT
-  static Integers computeFastSet(string ipaddr, const unsigned char* infoHash,
-				int pieces, int fastSetSize);
-#endif // ENABLE_BITTORRENT
-
   static int32_t countBit(uint32_t n);
-
-  static string randomAlpha(int length);
-
+  
+  static string randomAlpha(int32_t length, const RandomizerHandle& randomizer);
+  
   static string toUpper(const string& src);
 
   static string toLower(const string& src);
 
   static bool isNumbersAndDotsNotation(const string& name);
 
-  static void setGlobalSignalHandler(int signal, void (*handler)(int), int flags);
+  static void setGlobalSignalHandler(int32_t signal, void (*handler)(int), int32_t flags);
 
   static void indexRange(int32_t& startIndex, int32_t& endIndex,
 			 int64_t offset,
@@ -152,6 +147,31 @@ public:
   static string abbrevSize(int64_t size);
 
   static time_t httpGMT(const string& httpTimeFormat);
+
+  static void toStream(ostream& os, const FileEntries& entries);
+
+  static void sleep(long seconds);
+
+  static void usleep(long microseconds);
+
+  static bool isNumber(const string& what);
+
+  static bool isLowercase(const string& what);
+
+  static bool isUppercase(const string& what);
+
+  static int32_t alphaToNum(const string& alphabets);
+
+  static void mkdirs(const string& dirpath);
+
+  static void convertBitfield(BitfieldMan* dest, const BitfieldMan* src);
+
+  // binaryStream has to be opened before calling this function.
+  static string toString(const BinaryStreamHandle& binaryStream);
+
+#ifdef HAVE_POSIX_MEMALIGN
+  static void* allocateAlignedMemory(size_t alignment, size_t size);
+#endif // HAVE_POSIX_MEMALIGN
 };
 
 #endif // _D_UTIL_H_
