@@ -35,6 +35,7 @@
 #include "Xml2MetalinkProcessor.h"
 #include "DlAbortEx.h"
 #include "Util.h"
+#include "BinaryStream.h"
 #include <libxml/parser.h>
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
@@ -62,6 +63,49 @@ MetalinkerHandle Xml2MetalinkProcessor::parseFile(const string& filename) {
   if(!doc) {
     throw new DlAbortEx("Cannot parse metalink file %s", filename.c_str());
   }
+  return processDoc(doc);
+}
+
+MetalinkerHandle Xml2MetalinkProcessor::parseFromBinaryStream(const BinaryStreamHandle& binaryStream) {
+  release();
+  int32_t bufSize = 4096;
+  unsigned char buf[bufSize];
+
+  int32_t res = binaryStream->readData(buf, 4, 0);
+  if(res != 4) {
+    throw new DlAbortEx("Too small data for metalink parsing.");
+  }
+
+  xmlParserCtxtPtr ctx = xmlCreatePushParserCtxt(0, 0, (const char*)buf, res, 0);
+  try {
+    int64_t readOffset = res;
+    while(1) {
+      int32_t res = binaryStream->readData(buf, bufSize, readOffset);
+      if(res == 0) {
+	break;
+      }
+      if(xmlParseChunk(ctx, (const char*)buf, res, 0) != 0) {
+	throw new DlAbortEx("Cannot parse metalink file");
+      }
+      readOffset += res;
+    }
+    xmlParseChunk(ctx, (const char*)buf, 0, 1);
+    doc = ctx->myDoc;
+
+    xmlFreeParserCtxt(ctx);
+  } catch(Exception* e) {
+    xmlFreeParserCtxt(ctx);
+    throw;
+  }
+
+  if(!doc) {
+    throw new DlAbortEx("Cannot parse metalink file");
+  }
+  return processDoc(doc);
+}
+
+MetalinkerHandle Xml2MetalinkProcessor::processDoc(xmlDocPtr doc)
+{
   context = xmlXPathNewContext(doc);
   if(!context) {
     throw new DlAbortEx("Cannot create new xpath context");
@@ -75,8 +119,8 @@ MetalinkerHandle Xml2MetalinkProcessor::parseFile(const string& filename) {
   
   string xpath = "/m:metalink/m:files/m:file";
   MetalinkerHandle metalinker(new Metalinker());
-  for(int index = 1; 1; index++) {
-    MetalinkEntryHandle entry = getEntry(xpath+"["+Util::itos(index)+"]");
+  for(uint32_t index = 1; 1; index++) {
+    MetalinkEntryHandle entry = getEntry(xpath+"["+Util::uitos(index)+"]");
     if(!entry.get()) {
       break;
     } else {
@@ -98,42 +142,62 @@ MetalinkEntryHandle Xml2MetalinkProcessor::getEntry(const string& xpath) {
 
   MetalinkEntryHandle entry(new MetalinkEntry());
 
-  entry->filename = filename;
+  FileEntryHandle fileEntry = new FileEntry(filename, 0, 0);
+  
   string sizeStr = Util::trim(xpathContent(xpath+"/m:size"));
   if(sizeStr == "") {
-    entry->size = 0;
+    fileEntry->setLength(0);
   } else {
-    entry->size = STRTOLL(sizeStr.c_str());
+    fileEntry->setLength(strtoll(sizeStr.c_str(), 0, 10));
   }
+  entry->file = fileEntry;
   entry->version = Util::trim(xpathContent(xpath+"/m:version"));
   entry->language = Util::trim(xpathContent(xpath+"/m:language"));
   entry->os = Util::trim(xpathContent(xpath+"/m:os"));
 #ifdef ENABLE_MESSAGE_DIGEST
-  string md;
-  md = Util::toLower(Util::trim(xpathContent(xpath+"/m:verification/m:hash[@type=\"sha1\"]")));
-  if(md.size() > 0) {
-    entry->checksum = new Checksum();
-    entry->checksum->setMessageDigest(md);
-    entry->checksum->setDigestAlgo(DIGEST_ALGO_SHA1);
-  } else {
-    md = Util::toLower(Util::trim(xpathContent(xpath+"/m:verification/m:hash[@type=\"md5\"]")));
-    if(md.size() > 0) {
-      entry->checksum = new Checksum();
-      entry->checksum->setMessageDigest(md);
-      entry->checksum->setDigestAlgo(DIGEST_ALGO_MD5);
+  xmlXPathObjectPtr hashPathObj = xpathEvaluation(xpath+"/m:verification/m:hash");
+  if(hashPathObj) {
+    xmlNodeSetPtr nodeSet = hashPathObj->nodesetval;
+    for(int32_t i = 0; i < nodeSet->nodeNr; ++i) {
+      xmlNodePtr node = nodeSet->nodeTab[i];
+      string algo = Util::trim(xmlAttribute(node, "type"));
+      if(MessageDigestContext::supports(algo)) {
+	entry->checksum = new Checksum(algo, Util::trim(xmlContent(node)));
+	break;
+      }
     }
   }
+  xmlXPathFreeObject(hashPathObj);
+
   string piecesPath = xpath+"/m:verification/m:pieces";
-  string sha1PiecesPath = piecesPath+"[@type=\"sha1\"]";
-  string md5PiecesPath = piecesPath+"[@type=\"md5\"]";
-  if(xpathExists(sha1PiecesPath)) {
-    entry->chunkChecksum = getPieceHash(sha1PiecesPath, entry->size);
-  } else if(xpathExists(md5PiecesPath)) {
-    entry->chunkChecksum = getPieceHash(md5PiecesPath, entry->size);
+  xmlXPathObjectPtr pieceHashPathObj = xpathEvaluation(piecesPath);
+  if(pieceHashPathObj) {
+    xmlNodeSetPtr nodeSet = pieceHashPathObj->nodesetval;
+    for(int32_t i = 0; i < nodeSet->nodeNr; ++i) {
+      xmlNodePtr node = nodeSet->nodeTab[i];
+      string algo = Util::trim(xmlAttribute(node, "type"));
+      if(MessageDigestContext::supports(algo)) {
+	entry->chunkChecksum = getPieceHash(piecesPath+"[@type=\""+algo+"\"]",
+					    entry->getLength());
+	break;
+      }
+    }
   }
+  xmlXPathFreeObject(pieceHashPathObj);
 #endif // ENABLE_MESSAGE_DIGEST
-  for(int index = 1; 1; index++) {
-    MetalinkResourceHandle resource(getResource(xpath+"/m:resources/m:url["+Util::itos(index)+"]"));
+  
+  string resourcesPath = xpath+"/m:resources[@maxconnections]";
+  xmlXPathObjectPtr resourcesPathObj = xpathEvaluation(resourcesPath);
+  if(resourcesPathObj) {
+    xmlNodeSetPtr nodeSet = resourcesPathObj->nodesetval;
+    xmlNodePtr node = nodeSet->nodeTab[0];
+    int32_t maxConnections = strtol(Util::trim(xmlAttribute(node, "maxconnections")).c_str(), 0, 10);
+    entry->maxConnections = maxConnections;
+  }
+  xmlXPathFreeObject(resourcesPathObj);
+
+  for(uint32_t index = 1; 1; index++) {
+    MetalinkResourceHandle resource(getResource(xpath+"/m:resources/m:url["+Util::uitos(index)+"]"));
     if(!resource.get()) {
       break;
     } else {
@@ -144,40 +208,35 @@ MetalinkEntryHandle Xml2MetalinkProcessor::getEntry(const string& xpath) {
 }
 
 #ifdef ENABLE_MESSAGE_DIGEST
-MetalinkChunkChecksumHandle Xml2MetalinkProcessor::getPieceHash(const string& xpath,
+ChunkChecksumHandle Xml2MetalinkProcessor::getPieceHash(const string& xpath,
 								int64_t totalSize)
 {
-  MetalinkChunkChecksumHandle chunkChecksum = new MetalinkChunkChecksum();
-
   xmlXPathObjectPtr result = xpathEvaluation(xpath);
   if(!result) {
     return 0;
   }
   xmlNodeSetPtr nodeSet = result->nodesetval;
   xmlNodePtr node = nodeSet->nodeTab[0];
-  chunkChecksum->pieceLength = STRTOLL(Util::trim(xmlAttribute(node, "length")).c_str());
-  string algo = Util::trim(xmlAttribute(node, "type"));
+
+  int64_t checksumLength = STRTOLL(Util::trim(xmlAttribute(node, "length")).c_str());
+  string algoString = Util::trim(xmlAttribute(node, "type"));
   xmlXPathFreeObject(result);
-  if(algo == "sha1") {
-    chunkChecksum->digestAlgo = DIGEST_ALGO_SHA1;
-  } else if(algo == "md5") {
-    chunkChecksum->digestAlgo = DIGEST_ALGO_MD5;
-  } else {
+
+  if(!MessageDigestContext::supports(algoString)) {
     // unknown checksum type
-    chunkChecksum->pieceLength = 0;
-    return chunkChecksum;
+    return 0;
   }
 
-  int64_t numPiece =
-    (totalSize+chunkChecksum->pieceLength-1)/chunkChecksum->pieceLength;
-  for(int64_t i = 0; i < numPiece; ++i) {
+  Strings checksums;
+  uint64_t numPiece = (totalSize+checksumLength-1)/checksumLength;
+  for(uint64_t i = 0; i < numPiece; ++i) {
     string pieceHash = Util::trim(xpathContent(xpath+"/m:hash[@piece=\""+Util::ullitos(i)+"\"]"));
     if(pieceHash == "") {
-      throw new DlAbortEx("Piece hash missing. index=%d", i);
+      throw new DlAbortEx("Piece hash missing. index=%s", Util::ullitos(i).c_str());
     }
-    chunkChecksum->pieceHashes.push_back(pieceHash);
+    checksums.push_back(pieceHash);
   }
-  return chunkChecksum;
+  return new ChunkChecksum(algoString, checksums, checksumLength);
 }
 #endif // ENABLE_MESSAGE_DIGEST
 
@@ -209,10 +268,17 @@ MetalinkResourceHandle Xml2MetalinkProcessor::getResource(const string& xpath) {
   } else {
     resource->preference = STRTOLL(pref.c_str());
   }
-  resource->location = Util::trim(xmlAttribute(node, "location"));
+  resource->location = Util::toUpper(Util::trim(xmlAttribute(node, "location")));
 
   resource->url = Util::trim(xmlContent(node));
 
+  {
+    string cnn = Util::trim(xmlAttribute(node, "maxconnections"));
+    if(!cnn.empty()) {
+      resource->maxConnections = strtol(cnn.c_str(), 0, 10);
+    }
+  }
+
   xmlXPathFreeObject(result);
 
   return resource;
