@@ -37,23 +37,32 @@
 // use C99 limit macros
 #define __STDC_LIMIT_MACROS
 #ifdef HAVE_CONFIG_H
-# include <config.h>
+# include "config.h"
 #endif
+
+#ifdef __MINGW32__
+#ifdef malloc
+#	undef malloc
+#endif
+#ifdef realloc
+#	undef realloc
+#endif
+#endif // __MINGW32__
+
 #include <stdio.h>
 #include <stdint.h>
 #include <iostream>
 #include <assert.h>
 #include <limits.h>
 #include <unistd.h>
+#include <stdlib.h>
+// TODO this should be included where only it is need.
+#include <cstring>
 #include <string>
 #include <deque>
 #include <algorithm>
-#if ENABLE_NLS
-#  include <gettext.h>
-#  define _(String) gettext (String)
-#else
-#  define _(String) (String)
-#endif
+#include "gettext.h"
+#define _(String) gettext(String)
 
 #define USER_AGENT "aria2"
 
@@ -81,4 +90,16 @@ typedef deque<int32_t> Integers;
 #define OPEN_MODE S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH
 #define DIR_OPEN_MODE S_IRWXU|S_IRWXG|S_IRWXO
 
+#ifdef __MINGW32__
+# define LONGLONG_PRINTF "%I64d"
+# define ULONGLONG_PRINTF "%I64u"
+# define LONGLONG_SCANF "%I64d"
+# define ULONGLONG_SCANF "%I64u"
+#else
+# define LONGLONG_PRINTF "%lld"
+# define ULONGLONG_PRINTF "%llu"
+# define LONGLONG_SCANF "%Ld"
+# define ULONGLONG_SCANF "%Lu"
+#endif // __MINGW32__
+
 #endif // _D_COMMON_H_
