@@ -40,10 +40,13 @@
 /**
  * Constants
  */
+#undef V_TRUE
 #define V_TRUE "true"
+#undef V_FALSE
 #define V_FALSE "false"
+#undef V_NONE
 #define V_NONE "none"
-
+#define V_MEM "mem"
 /**
  * General preferences
  */
@@ -84,6 +87,8 @@
 // value: prealloc | none
 #define PREF_FILE_ALLOCATION "file-allocation"
 #  define V_PREALLOC "prealloc"
+#// value: 1*digit
+#define PREF_NO_FILE_ALLOCATION_LIMIT "no-file-allocation-limit"
 // value: true | false
 #define PREF_ALLOW_OVERWRITE "allow-overwrite"
 // value: true | false
@@ -104,6 +109,16 @@
 #define PREF_MAX_CONCURRENT_DOWNLOADS "max-concurrent-downloads"
 // value: 1*digit
 #define PREF_DIRECT_DOWNLOAD_TIMEOUT "direct-download-timeout"
+// value: true | false
+#define PREF_FORCE_SEQUENTIAL "force-sequential"
+// value: true | false
+#define PREF_AUTO_FILE_RENAMING "auto-file-renaming"
+// value: true | false
+#define PREF_PARAMETERIZED_URI "parameterized-uri"
+// value: true | false
+#define PREF_ENABLE_DIRECT_IO "enable-direct-io"
+// value: true | false
+#define PREF_ALLOW_PIECE_LENGTH_CHANGE "allow-piece-length-change"
 
 /**
  * FTP related preferences
@@ -131,12 +146,16 @@
 #  define V_BASIC "basic"
 // values: true | false
 #define PREF_HTTP_AUTH_ENABLED "http-auth-enabled"
-// values: true | false
-#define PREF_HTTP_KEEP_ALIVE "http-keep-alive"
 // values: string
 #define PREF_USER_AGENT "user-agent"
 // value: string that your file system recognizes as a file name.
 #define PREF_LOAD_COOKIES "load-cookies"
+// values: true | false
+#define PREF_ENABLE_HTTP_KEEP_ALIVE "enable-http-keep-alive"
+// values: true | false
+#define PREF_ENABLE_HTTP_PIPELINING "enable-http-pipelining"
+// value: 1*digit
+#define PREF_MAX_HTTP_PIPELINING "max-http-pipelining"
 
 /** 
  * HTTP proxy related preferences
@@ -174,7 +193,7 @@
 #define PREF_TORRENT_FILE "torrent-file"
 // values: 1*digit
 #define PREF_LISTEN_PORT "listen-port"
-// values: true | false
+// values: true | false | mem
 #define PREF_FOLLOW_TORRENT "follow-torrent"
 // values: 1*digit *( (,|-) 1*digit)
 #define PREF_SELECT_FILE "select-file"
@@ -186,6 +205,8 @@
 #define PREF_TRACKER_MAX_TRIES "tracker-max-tries"
 // values: 1*digit
 #define PREF_BT_KEEP_ALIVE_INTERVAL "bt-keep-alive-interval"
+// values: a string, less than or equals to 20 bytes length
+#define PREF_PEER_ID_PREFIX "peer-id-prefix"
 
 /**
  * Metalink related preferences
@@ -202,7 +223,7 @@
 #define PREF_METALINK_LOCATION "metalink-location"
 // values: 1*digit
 #define PREF_METALINK_SERVERS "metalink-servers"
-// values: true | false
+// values: true | false | mem
 #define PREF_FOLLOW_METALINK "follow-metalink"
 
 #endif // _D_PREFS_H_
