@@ -1,3 +1,4 @@
+#include "CookieBoxFactory.h"
 #include <cppunit/CompilerOutputter.h>
 #include <cppunit/extensions/TestFactoryRegistry.h>
 #include <cppunit/ui/text/TestRunner.h>
@@ -9,6 +10,10 @@ int main(int argc, char* argv[]) {
 
   runner.setOutputter(new CppUnit::CompilerOutputter(&runner.result(), std::cerr));
   
+  // setup
+  CookieBoxFactoryHandle cookieBoxFactory = new CookieBoxFactory();
+  CookieBoxFactorySingletonHolder::instance(cookieBoxFactory);
+
   // Run the tests.
   bool successfull = runner.run();
 
