@@ -26,7 +26,7 @@ void BtKeepAliveMessageTest::testGetMessage() {
   memset(msg, 0, sizeof(msg));
   BtKeepAliveMessage message;
   CPPUNIT_ASSERT_EQUAL((int8_t)99, message.getId());
-  CPPUNIT_ASSERT_EQUAL(4, message.getMessageLength());
+  CPPUNIT_ASSERT_EQUAL((int32_t)4, message.getMessageLength());
   CPPUNIT_ASSERT(memcmp(msg, message.getMessage(), 4) == 0);
 }
 
