@@ -1,5 +1,4 @@
 #include "DefaultDiskWriter.h"
-#include <string>
 #include <cppunit/extensions/HelperMacros.h>
 
 using namespace std;
@@ -7,7 +6,7 @@ using namespace std;
 class DefaultDiskWriterTest:public CppUnit::TestFixture {
 
   CPPUNIT_TEST_SUITE(DefaultDiskWriterTest);
-  CPPUNIT_TEST(testMessageDigest);
+  CPPUNIT_TEST(testSize);
   CPPUNIT_TEST_SUITE_END();
 private:
 
@@ -15,21 +14,16 @@ public:
   void setUp() {
   }
 
-  void testMessageDigest();
+  void testSize();
 };
 
 
 CPPUNIT_TEST_SUITE_REGISTRATION( DefaultDiskWriterTest );
 
-void DefaultDiskWriterTest::testMessageDigest() {
+void DefaultDiskWriterTest::testSize()
+{
   DefaultDiskWriter dw;
   dw.openExistingFile("4096chunk.txt");
-  
-  CPPUNIT_ASSERT_EQUAL(string("608cabc0f2fa18c260cafd974516865c772363d5"),
-		       dw.messageDigest(0, 4096, DIGEST_ALGO_SHA1));
-
-  CPPUNIT_ASSERT_EQUAL(string("7a4a9ae537ebbbb826b1060e704490ad0f365ead"),
-		       dw.messageDigest(5, 100, DIGEST_ALGO_SHA1));
-
-  dw.closeFile();
+  CPPUNIT_ASSERT_EQUAL((int64_t)4096, dw.size());
 }
+
