@@ -3,6 +3,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <string>
+#include <fstream>
 #include <cppunit/extensions/HelperMacros.h>
 
 using namespace std;
@@ -16,6 +17,9 @@ class FileTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testRemove);
   CPPUNIT_TEST(testSize);
   CPPUNIT_TEST(testMkdir);
+  CPPUNIT_TEST(testGetDirname);
+  CPPUNIT_TEST(testGetBasename);
+  CPPUNIT_TEST(testRenameTo);
   CPPUNIT_TEST_SUITE_END();
 private:
 
@@ -29,6 +33,9 @@ public:
   void testRemove();
   void testSize();
   void testMkdir();
+  void testGetDirname();
+  void testGetBasename();
+  void testRenameTo();
 };
 
 
@@ -83,7 +90,11 @@ void FileTest::testRemove() {
   CPPUNIT_ASSERT(!f.remove());
 
   string dir = "/tmp/aria2testdir";
+#ifdef __MINGW32__
+  mkdir(dir.c_str());
+#else
   mkdir(dir.c_str(), 0777);
+#endif // __MINGW32__
   File d(dir);
   CPPUNIT_ASSERT(d.exists());
   CPPUNIT_ASSERT(d.remove());
@@ -112,3 +123,35 @@ void FileTest::testMkdir() {
   // already exists.
   CPPUNIT_ASSERT(!d.mkdirs());
 }
+
+void FileTest::testGetDirname()
+{
+  File f("/tmp/dist/aria2.tar.bz2");
+  CPPUNIT_ASSERT_EQUAL(string("/tmp/dist"), f.getDirname());
+}
+
+void FileTest::testGetBasename()
+{
+  File f("/tmp/dist/aria2.tar.bz2");
+  CPPUNIT_ASSERT_EQUAL(string("aria2.tar.bz2"), f.getBasename());
+}
+
+void FileTest::testRenameTo()
+{
+  string fname = "FileTest_testRenameTo.txt";
+  ofstream of(fname.c_str());
+  of.close();
+
+  File f(fname);
+  string fnameTo = "FileTest_testRenameTo_dest.txt";
+  CPPUNIT_ASSERT(f.renameTo(fnameTo));
+  CPPUNIT_ASSERT(f.exists());
+  CPPUNIT_ASSERT(!File(fname).exists());
+  CPPUNIT_ASSERT_EQUAL(fnameTo, f.getBasename());
+
+  // to see renameTo() work even when the destination file exists
+  of.open(fname.c_str());
+  of.close();
+  
+  CPPUNIT_ASSERT(f.renameTo(fname));
+}
