@@ -1,6 +1,8 @@
 #include "HttpRequest.h"
 #include "prefs.h"
-#include "RequestFactory.h"
+#include "AuthConfigFactory.h"
+#include "PiecedSegment.h"
+#include "Piece.h"
 #include <cppunit/extensions/HelperMacros.h>
 
 using namespace std;
@@ -15,6 +17,7 @@ class HttpRequestTest : public CppUnit::TestFixture {
   CPPUNIT_TEST(testCreateRequest_with_cookie);
   CPPUNIT_TEST(testCreateProxyRequest);
   CPPUNIT_TEST(testIsRangeSatisfied);
+  CPPUNIT_TEST(testUserAgent);
   CPPUNIT_TEST_SUITE_END();
 private:
 
@@ -28,6 +31,7 @@ public:
   void testCreateRequest_with_cookie();
   void testCreateProxyRequest();
   void testIsRangeSatisfied();
+  void testUserAgent();
 };
 
 
@@ -36,13 +40,13 @@ CPPUNIT_TEST_SUITE_REGISTRATION( HttpRequestTest );
 void HttpRequestTest::testGetStartByte()
 {
   HttpRequest httpRequest;
-  SegmentHandle segment = new Segment(1, 1024*1024, 1024*1024, 0);
+  SegmentHandle segment = new PiecedSegment(1024, new Piece(1, 1024));
 
   CPPUNIT_ASSERT_EQUAL((int64_t)0, httpRequest.getStartByte());
 
   httpRequest.setSegment(segment);
   
-  CPPUNIT_ASSERT_EQUAL((int64_t)1024*1024, httpRequest.getStartByte());
+  CPPUNIT_ASSERT_EQUAL((int64_t)1024, httpRequest.getStartByte());
 
 }
 
@@ -51,17 +55,17 @@ void HttpRequestTest::testGetEndByte()
   int32_t index = 1;
   int32_t length = 1024*1024-1024;
   int32_t segmentLength = 1024*1024;
-  int32_t writtenLength = 1024;
 
   HttpRequest httpRequest;
-  SegmentHandle segment = new Segment(index, length, segmentLength, writtenLength);
+  SegmentHandle segment = new PiecedSegment(segmentLength,
+					    new Piece(index, length));
+  
 
   CPPUNIT_ASSERT_EQUAL((int64_t)0, httpRequest.getEndByte());
 
   httpRequest.setSegment(segment);
 
-  CPPUNIT_ASSERT_EQUAL((int64_t)0,
-		       httpRequest.getEndByte());
+  CPPUNIT_ASSERT_EQUAL((int64_t)0, httpRequest.getEndByte());
 
   RequestHandle request = new Request();
   request->setKeepAlive(true);
@@ -79,35 +83,37 @@ void HttpRequestTest::testGetEndByte()
 
 void HttpRequestTest::testCreateRequest()
 {
-  SharedHandle<Option> option = new Option();
-  option->put(PREF_HTTP_AUTH_ENABLED, V_FALSE);
-  option->put(PREF_HTTP_PROXY_ENABLED, V_FALSE);
-  option->put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
-  option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
-  option->put(PREF_HTTP_USER, "aria2user");
-  option->put(PREF_HTTP_PASSWD, "aria2passwd");
-  option->put(PREF_HTTP_PROXY_USER, "aria2proxyuser");
-  option->put(PREF_HTTP_PROXY_PASSWD, "aria2proxypasswd");
-  RequestFactory requestFactory;
-  requestFactory.setOption(option.get());
+  Option option;
+  option.put(PREF_HTTP_AUTH_ENABLED, V_FALSE);
+  option.put(PREF_HTTP_PROXY_ENABLED, V_FALSE);
+  option.put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
+  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
+  option.put(PREF_HTTP_USER, "aria2user");
+  option.put(PREF_HTTP_PASSWD, "aria2passwd");
+  option.put(PREF_HTTP_PROXY_USER, "aria2proxyuser");
+  option.put(PREF_HTTP_PROXY_PASSWD, "aria2proxypasswd");
 
-  RequestHandle request = requestFactory.createRequest();
+  AuthConfigFactoryHandle authConfigFactory = new AuthConfigFactory(&option);
+  AuthConfigFactorySingleton::instance(authConfigFactory);
 
+  RequestHandle request = new Request();
   request->setUrl("http://localhost:8080/archives/aria2-1.0.0.tar.bz2");
 
-  SegmentHandle segment = new Segment();
+  SegmentHandle segment = new PiecedSegment(1024, new Piece(0, 1024));
 
   HttpRequest httpRequest;
-
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
 
+  request->setKeepAlive(true);
+  
   string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
     "Accept: */*\r\n"
     "Host: localhost:8080\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
+    "Range: bytes=0-1023\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
@@ -125,10 +131,8 @@ void HttpRequestTest::testCreateRequest()
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  segment->index = 1;
-  segment->length = 1024*1024;
-  segment->segmentLength = 1024*1024;
-  segment->writtenLength = 0;
+  segment = new PiecedSegment(1024*1024, new Piece(1, 1024*1024));
+  httpRequest.setSegment(segment);
 
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -152,10 +156,10 @@ void HttpRequestTest::testCreateRequest()
     "Cache-Control: no-cache\r\n"
     "Range: bytes=1048576-2097151\r\n"
     "\r\n";
-
+  
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  httpRequest.setSegment(new Segment());
+  request->setKeepAlive(false);
 
   request->redirectUrl("http://localhost:8080/archives/download/aria2-1.0.0.tar.bz2");
 
@@ -165,17 +169,21 @@ void HttpRequestTest::testCreateRequest()
     "Host: localhost:8080\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
-    "Referer: http://localhost:8080/archives/aria2-1.0.0.tar.bz2\r\n"
+    "Connection: close\r\n"
+    "Range: bytes=1048576-\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
   
   request->resetUrl();
 
-  // enable http auth
-  option->put(PREF_HTTP_AUTH_ENABLED, V_TRUE);
+  segment = new PiecedSegment(1024*1024, new Piece(0, 1024*1024));
+  httpRequest.setSegment(segment);
 
-  httpRequest.configure(option.get());
+  // enable http auth
+  option.put(PREF_HTTP_AUTH_ENABLED, V_TRUE);
+  
+  httpRequest.configure(&option);
 
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -183,15 +191,16 @@ void HttpRequestTest::testCreateRequest()
     "Host: localhost:8080\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
     "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
   // enable http proxy auth
-  option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
+  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
 
-  httpRequest.configure(option.get());
+  httpRequest.configure(&option);
 
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -199,14 +208,15 @@ void HttpRequestTest::testCreateRequest()
     "Host: localhost:8080\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
     "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  option->put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
+  option.put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
 
-  httpRequest.configure(option.get());
+  httpRequest.configure(&option);
 
   expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -214,14 +224,15 @@ void HttpRequestTest::testCreateRequest()
     "Host: localhost:8080\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
     "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  option->put(PREF_HTTP_PROXY_METHOD, V_GET);
+  option.put(PREF_HTTP_PROXY_METHOD, V_GET);
 
-  httpRequest.configure(option.get());
+  httpRequest.configure(&option);
 
   expectedText = "GET http://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -229,6 +240,7 @@ void HttpRequestTest::testCreateRequest()
     "Host: localhost:8080\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
     "Proxy-Connection: close\r\n"
     "Proxy-Authorization: Basic YXJpYTJwcm94eXVzZXI6YXJpYTJwcm94eXBhc3N3ZA==\r\n"
     "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
@@ -236,9 +248,7 @@ void HttpRequestTest::testCreateRequest()
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
-  option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
-
-  httpRequest.configure(option.get());
+  request->setKeepAlive(true);
 
   expectedText = "GET http://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -246,6 +256,27 @@ void HttpRequestTest::testCreateRequest()
     "Host: localhost:8080\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
+    "Range: bytes=0-1048575\r\n"
+    "Proxy-Connection: Keep-Alive\r\n"
+    "Proxy-Authorization: Basic YXJpYTJwcm94eXVzZXI6YXJpYTJwcm94eXBhc3N3ZA==\r\n"
+    "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
+
+  request->setKeepAlive(false);
+
+  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
+
+  httpRequest.configure(&option);
+
+  expectedText = "GET http://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Accept: */*\r\n"
+    "Host: localhost:8080\r\n"
+    "Pragma: no-cache\r\n"
+    "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
     "Proxy-Connection: close\r\n"
     "Authorization: Basic YXJpYTJ1c2VyOmFyaWEycGFzc3dk\r\n"
     "\r\n";
@@ -255,30 +286,28 @@ void HttpRequestTest::testCreateRequest()
 
 void HttpRequestTest::testCreateRequest_ftp()
 {
-  SharedHandle<Option> option = new Option();
-  option->put(PREF_HTTP_AUTH_ENABLED, V_FALSE);
-  option->put(PREF_HTTP_PROXY_ENABLED, V_FALSE);
-  option->put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
-  option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
-  option->put(PREF_HTTP_USER, "aria2user");
-  option->put(PREF_HTTP_PASSWD, "aria2passwd");
-  option->put(PREF_HTTP_PROXY_USER, "aria2proxyuser");
-  option->put(PREF_HTTP_PROXY_PASSWD, "aria2proxypasswd");
+  Option option;
+  option.put(PREF_HTTP_AUTH_ENABLED, V_FALSE);
+  option.put(PREF_HTTP_PROXY_ENABLED, V_FALSE);
+  option.put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
+  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_FALSE);
+  option.put(PREF_HTTP_USER, "aria2user");
+  option.put(PREF_HTTP_PASSWD, "aria2passwd");
+  option.put(PREF_HTTP_PROXY_USER, "aria2proxyuser");
+  option.put(PREF_HTTP_PROXY_PASSWD, "aria2proxypasswd");
 
-  RequestFactory requestFactory;
-  requestFactory.setOption(option.get());
+  AuthConfigFactoryHandle authConfigFactory = new AuthConfigFactory(&option);
+  AuthConfigFactorySingleton::instance(authConfigFactory);
 
-  RequestHandle request = requestFactory.createRequest();
+  RequestHandle request = new Request();
   request->setUrl("ftp://localhost:8080/archives/aria2-1.0.0.tar.bz2");
 
-  SegmentHandle segment = new Segment();
-
   HttpRequest httpRequest;
-
+  SegmentHandle segment = new PiecedSegment(1024*1024, new Piece(0, 1024*1024));
   httpRequest.setRequest(request);
   httpRequest.setSegment(segment);
 
-  httpRequest.configure(option.get());
+  httpRequest.configure(&option);
 
   string expectedText = "GET ftp://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -286,16 +315,17 @@ void HttpRequestTest::testCreateRequest_ftp()
     "Host: localhost:8080\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
 
   // How to enable HTTP proxy authorization in FTP download via HTTP proxy
-  option->put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
-  option->put(PREF_HTTP_PROXY_METHOD, V_GET);
-  option->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
+  option.put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
+  option.put(PREF_HTTP_PROXY_METHOD, V_GET);
+  option.put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
 
-  httpRequest.configure(option.get());
+  httpRequest.configure(&option);
 
   expectedText = "GET ftp://localhost:8080/archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
@@ -303,6 +333,7 @@ void HttpRequestTest::testCreateRequest_ftp()
     "Host: localhost:8080\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
     "Proxy-Connection: close\r\n"
     "Proxy-Authorization: Basic YXJpYTJwcm94eXVzZXI6YXJpYTJwcm94eXBhc3N3ZA==\r\n"
     "\r\n";
@@ -315,7 +346,7 @@ void HttpRequestTest::testCreateRequest_with_cookie()
 {
   RequestHandle request = new Request();
   request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
-  SegmentHandle segment = new Segment();
+  SegmentHandle segment = new PiecedSegment(1024*1024, new Piece(0, 1024*1024));
 
   Cookie cookie1("name1", "value1", "/archives", "localhost", false);
   Cookie cookie2("name2", "value2", "/archives/download", "localhost", false);
@@ -338,6 +369,7 @@ void HttpRequestTest::testCreateRequest_with_cookie()
     "Host: localhost\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
     "Cookie: name1=value1;\r\n"
     "\r\n";
 
@@ -351,6 +383,7 @@ void HttpRequestTest::testCreateRequest_with_cookie()
     "Host: localhost\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
     "Cookie: name1=value1;name2=value2;\r\n"
     "\r\n";
 
@@ -364,6 +397,7 @@ void HttpRequestTest::testCreateRequest_with_cookie()
     "Host: tt.localhost\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
     "Cookie: name1=value1;name2=value2;name3=value3;\r\n"
     "\r\n";
 
@@ -377,6 +411,7 @@ void HttpRequestTest::testCreateRequest_with_cookie()
     "Host: tt.localhost\r\n"
     "Pragma: no-cache\r\n"
     "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
     "Cookie: name1=value1;name2=value2;name3=value3;name4=value4;\r\n"
     "\r\n";
 
@@ -388,7 +423,7 @@ void HttpRequestTest::testCreateProxyRequest()
 {
   RequestHandle request = new Request();
   request->setUrl("http://localhost/archives/aria2-1.0.0.tar.bz2");
-  SegmentHandle segment = new Segment();
+  SegmentHandle segment = new PiecedSegment(1024*1024, new Piece(0, 1024*1024));
 
   HttpRequest httpRequest;
 
@@ -397,8 +432,18 @@ void HttpRequestTest::testCreateProxyRequest()
 
   string expectedText = "CONNECT localhost:80 HTTP/1.1\r\n"
     "User-Agent: aria2\r\n"
-    "Proxy-Connection: close\r\n"
     "Host: localhost:80\r\n"
+    "Proxy-Connection: close\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createProxyRequest());
+
+  request->setKeepAlive(true);
+
+  expectedText = "CONNECT localhost:80 HTTP/1.1\r\n"
+    "User-Agent: aria2\r\n"
+    "Host: localhost:80\r\n"
+    "Proxy-Connection: Keep-Alive\r\n"
     "\r\n";
 
   CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createProxyRequest());
@@ -410,7 +455,7 @@ void HttpRequestTest::testIsRangeSatisfied()
   RequestHandle request = new Request();
   request->setUrl("http://localhost:8080/archives/aria2-1.0.0.tar.bz2");
   request->setKeepAlive(false);
-  SegmentHandle segment = new Segment();
+  SegmentHandle segment = new PiecedSegment(1024*1024, new Piece(0, 1024*1024));
 
   HttpRequest httpRequest;
 
@@ -421,15 +466,13 @@ void HttpRequestTest::testIsRangeSatisfied()
 
   CPPUNIT_ASSERT(httpRequest.isRangeSatisfied(range));
 
-  segment->index = 1;
-  segment->length = 1024*1024;
-  segment->segmentLength = 1024*1024;
-  segment->writtenLength = 0;
-
-  int64_t entityLength = segment->segmentLength*10;
+  segment = new PiecedSegment(1024*1024, new Piece(1, 1024*1024));
+  httpRequest.setSegment(segment);
 
   CPPUNIT_ASSERT(!httpRequest.isRangeSatisfied(range));
 
+  int64_t entityLength = segment->getSegmentLength()*10;
+
   range = new Range(segment->getPosition(), 0, entityLength);
 
   CPPUNIT_ASSERT(httpRequest.isRangeSatisfied(range));
@@ -447,11 +490,50 @@ void HttpRequestTest::testIsRangeSatisfied()
   CPPUNIT_ASSERT(!httpRequest.isRangeSatisfied(range));
 
   range = new Range(segment->getPosition(),
-		    segment->getPosition()+segment->length-1, entityLength);
+		    segment->getPosition()+segment->getLength()-1,
+		    entityLength);
 
   CPPUNIT_ASSERT(httpRequest.isRangeSatisfied(range));
 
-  range = new Range(0, segment->getPosition()+segment->length-1, entityLength);
+  range = new Range(0, segment->getPosition()+segment->getLength()-1,
+		    entityLength);
 
   CPPUNIT_ASSERT(!httpRequest.isRangeSatisfied(range));
 }
+
+void HttpRequestTest::testUserAgent()
+{
+  Option option;
+
+  AuthConfigFactoryHandle authConfigFactory = new AuthConfigFactory(&option);
+  AuthConfigFactorySingleton::instance(authConfigFactory);
+
+  RequestHandle request = new Request();
+  request->setUrl("http://localhost:8080/archives/aria2-1.0.0.tar.bz2");
+
+  SegmentHandle segment = new PiecedSegment(1024, new Piece(0, 1024));
+
+  HttpRequest httpRequest;
+  httpRequest.setRequest(request);
+  httpRequest.setSegment(segment);
+  httpRequest.setUserAgent("aria2 (Linux)");
+
+  string expectedText = "GET /archives/aria2-1.0.0.tar.bz2 HTTP/1.1\r\n"
+    "User-Agent: aria2 (Linux)\r\n"
+    "Accept: */*\r\n"
+    "Host: localhost:8080\r\n"
+    "Pragma: no-cache\r\n"
+    "Cache-Control: no-cache\r\n"
+    "Connection: close\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedText, httpRequest.createRequest());
+
+  string expectedTextForProxy = "CONNECT localhost:8080 HTTP/1.1\r\n"
+    "User-Agent: aria2 (Linux)\r\n"
+    "Host: localhost:8080\r\n"
+    "Proxy-Connection: close\r\n"
+    "\r\n";
+
+  CPPUNIT_ASSERT_EQUAL(expectedTextForProxy, httpRequest.createProxyRequest());
+}
