@@ -3,6 +3,7 @@
 
 #include "BtContext.h"
 #include "Util.h"
+#include "AnnounceTier.h"
 
 class MockBtContext : public BtContext  {
 private:
@@ -16,6 +17,7 @@ private:
   unsigned char peerId[20];
   FileEntries fileEntries;
   AnnounceTiers announceTiers;
+  Integers fastSet;
 public:
   MockBtContext():totalLength(0),
 		  pieceLength(0),
@@ -117,6 +119,30 @@ public:
     memcpy(this->peerId, peerId, sizeof(this->peerId));
   }
 
+  virtual Integers computeFastSet(const string& ipaddr, int32_t fastSetSize)
+  {
+    return fastSet;
+  }
+
+  void setFastSet(const Integers& fastSet)
+  {
+    this->fastSet = fastSet;
+  }
+
+  virtual string getPieceHashAlgo() const
+  {
+    return "sha1";
+  }
+
+  virtual string getActualBasePath() const
+  {
+    return _dir+"/"+name;
+  }
+
+  virtual RequestGroup* getOwnerRequestGroup()
+  {
+    return 0;
+  }
 };
 
 typedef SharedHandle<MockBtContext> MockBtContextHandle;
