@@ -3,6 +3,8 @@
 
 #include "PieceStorage.h"
 #include "BitfieldMan.h"
+#include "Piece.h"
+#include "DiskAdaptor.h"
 
 class MockPieceStorage : public PieceStorage {
 private:
@@ -15,8 +17,19 @@ private:
   bool endGame;
   DiskAdaptorHandle diskAdaptor;
   Integers pieceLengthList;
+  Pieces inFlightPieces;
+  bool _allDownloadFinished;
 public:
-  MockPieceStorage():diskAdaptor(0) {}
+  MockPieceStorage():totalLength(0),
+		     filteredTotalLength(0),
+		     completedLength(0),
+		     filteredCompletedLength(0),
+		     bitfieldMan(0),
+		     selectiveDownloadingMode(false),
+		     endGame(false),
+		     diskAdaptor(0),
+		     _allDownloadFinished(false) {}
+
   virtual ~MockPieceStorage() {}
 
   virtual bool hasMissingPiece(const PeerHandle& peer) {
@@ -31,7 +44,26 @@ public:
     return new Piece();
   }
 
-  virtual PieceHandle getPiece(int index) {
+  virtual PieceHandle getMissingPiece()
+  {
+    return new Piece();
+  }
+
+  virtual PieceHandle getMissingPiece(int32_t index)
+  {
+    return new Piece();
+  }
+
+  virtual bool isPieceUsed(int32_t index)
+  {
+    return false;
+  }
+
+  virtual void markPieceMissing(int32_t index) {}
+
+  virtual void markPiecesDone(int64_t) {}
+
+  virtual PieceHandle getPiece(int32_t index) {
     return new Piece();
   }
 
@@ -39,7 +71,7 @@ public:
 
   virtual void cancelPiece(const PieceHandle& piece) {}
 
-  virtual bool hasPiece(int index) {
+  virtual bool hasPiece(int32_t index) {
     return false;
   }
 
@@ -77,7 +109,7 @@ public:
   
   virtual void setFileFilter(const Strings& filePaths) {}
 
-  virtual void setFileFilter(const Integers& fileIndexes) {}
+  virtual void setFileFilter(IntSequence seq) {}
 
   virtual void clearFileFilter() {}
 
@@ -86,7 +118,12 @@ public:
   }
 
   virtual bool allDownloadFinished() {
-    return false;
+    return _allDownloadFinished;
+  }
+
+  void setAllDownloadFinished(bool f)
+  {
+    _allDownloadFinished = f;
   }
 
   virtual void initStorage() {}
@@ -153,7 +190,21 @@ public:
 
   virtual void markAllPiecesDone() {}
 
-  virtual void checkIntegrity() {}
+  virtual void addInFlightPiece(const Pieces& pieces)
+  {
+    copy(pieces.begin(), pieces.end(), back_inserter(inFlightPieces));
+  }
+
+  virtual int32_t countInFlightPiece()
+  {
+    return inFlightPieces.size();
+  }
+
+  virtual Pieces getInFlightPieces()
+  {
+    return inFlightPieces;
+  }
+
 };
 
 typedef SharedHandle<MockPieceStorage> MockPieceStorageHandle;
