@@ -1,4 +1,6 @@
+#include "common.h"
 #include "MultiDiskWriter.h"
+#include "a2io.h"
 #include <string>
 #include <cppunit/extensions/HelperMacros.h>
 
