@@ -1,5 +1,6 @@
+#include "common.h"
 #include "PeerMessageUtil.h"
-#include <netinet/in.h>
+#include "a2netcompat.h"
 #include <cppunit/extensions/HelperMacros.h>
 
 using namespace std;
