@@ -21,16 +21,16 @@ class RequestTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testSetUrl12);
   CPPUNIT_TEST(testSetUrl13);
   CPPUNIT_TEST(testSetUrl14);
+  CPPUNIT_TEST(testSetUrl15);
+  CPPUNIT_TEST(testSetUrl16);
+  CPPUNIT_TEST(testSetUrl17);
+  CPPUNIT_TEST(testSetUrl_username);
+  CPPUNIT_TEST(testSetUrl_usernamePassword);
+  CPPUNIT_TEST(testSetUrl_zeroUsername);
   CPPUNIT_TEST(testRedirectUrl);
   CPPUNIT_TEST(testRedirectUrl2);
   CPPUNIT_TEST(testResetUrl);
-  CPPUNIT_TEST(testSafeChar);
   CPPUNIT_TEST(testInnerLink);
-  CPPUNIT_TEST(testMetalink);
-  CPPUNIT_TEST(testResolveHttpAuthConfig);
-  CPPUNIT_TEST(testResolveHttpAuthConfig_noCandidate);
-  CPPUNIT_TEST(testResolveHttpProxyAuthConfig);
-  CPPUNIT_TEST(testResolveFtpAuthConfig);
   CPPUNIT_TEST_SUITE_END();
   
 public:
@@ -48,16 +48,16 @@ public:
   void testSetUrl12();
   void testSetUrl13();
   void testSetUrl14();
+  void testSetUrl15();
+  void testSetUrl16();
+  void testSetUrl17();
+  void testSetUrl_username();
+  void testSetUrl_usernamePassword();
+  void testSetUrl_zeroUsername();
   void testRedirectUrl();
   void testRedirectUrl2();
   void testResetUrl();
-  void testSafeChar();
   void testInnerLink();
-  void testMetalink();
-  void testResolveHttpAuthConfig();
-  void testResolveHttpAuthConfig_noCandidate();
-  void testResolveHttpProxyAuthConfig();
-  void testResolveFtpAuthConfig();
 };
 
 
@@ -72,10 +72,12 @@ void RequestTest::testSetUrl1() {
   CPPUNIT_ASSERT_EQUAL(string("http://aria.rednoah.com/"), req.getCurrentUrl());
   CPPUNIT_ASSERT_EQUAL(string(""), req.getPreviousUrl());
   CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
-  CPPUNIT_ASSERT_EQUAL(80, req.getPort());
+  CPPUNIT_ASSERT_EQUAL((int32_t)80, req.getPort());
   CPPUNIT_ASSERT_EQUAL(string("aria.rednoah.com"), req.getHost());
   CPPUNIT_ASSERT_EQUAL(string("/"), req.getDir());
   CPPUNIT_ASSERT_EQUAL(string(""), req.getFile());
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getUsername());
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getPassword());
 }
 
 void RequestTest::testSetUrl2() {
@@ -90,7 +92,7 @@ void RequestTest::testSetUrl2() {
   // previousUrl must equal to referer;
   CPPUNIT_ASSERT_EQUAL(req.getReferer(), req.getPreviousUrl());
   CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
-  CPPUNIT_ASSERT_EQUAL(8080, req.getPort());
+  CPPUNIT_ASSERT_EQUAL((int32_t)8080, req.getPort());
   CPPUNIT_ASSERT_EQUAL(string("aria.rednoah.com"), req.getHost());
   CPPUNIT_ASSERT_EQUAL(string("/"), req.getDir());
   CPPUNIT_ASSERT_EQUAL(string("index.html"), req.getFile());
@@ -102,7 +104,7 @@ void RequestTest::testSetUrl3() {
 
   CPPUNIT_ASSERT(v);
   CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
-  CPPUNIT_ASSERT_EQUAL(80, req.getPort());
+  CPPUNIT_ASSERT_EQUAL((int32_t)80, req.getPort());
   CPPUNIT_ASSERT_EQUAL(string("aria.rednoah.com"), req.getHost());
   CPPUNIT_ASSERT_EQUAL(string("/aria2"), req.getDir());
   CPPUNIT_ASSERT_EQUAL(string("index.html"), req.getFile());
@@ -114,7 +116,7 @@ void RequestTest::testSetUrl4() {
 
   CPPUNIT_ASSERT(v);
   CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
-  CPPUNIT_ASSERT_EQUAL(80, req.getPort());
+  CPPUNIT_ASSERT_EQUAL((int32_t)80, req.getPort());
   CPPUNIT_ASSERT_EQUAL(string("aria.rednoah.com"), req.getHost());
   CPPUNIT_ASSERT_EQUAL(string("/aria2/aria3"), req.getDir());
   CPPUNIT_ASSERT_EQUAL(string("index.html"), req.getFile());
@@ -126,7 +128,7 @@ void RequestTest::testSetUrl5() {
 
   CPPUNIT_ASSERT(v);
   CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
-  CPPUNIT_ASSERT_EQUAL(80, req.getPort());
+  CPPUNIT_ASSERT_EQUAL((int32_t)80, req.getPort());
   CPPUNIT_ASSERT_EQUAL(string("aria.rednoah.com"), req.getHost());
   CPPUNIT_ASSERT_EQUAL(string("/aria2/aria3"), req.getDir());
   CPPUNIT_ASSERT_EQUAL(string(""), req.getFile());
@@ -138,7 +140,7 @@ void RequestTest::testSetUrl6() {
 
   CPPUNIT_ASSERT(v);
   CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
-  CPPUNIT_ASSERT_EQUAL(80, req.getPort());
+  CPPUNIT_ASSERT_EQUAL((int32_t)80, req.getPort());
   CPPUNIT_ASSERT_EQUAL(string("aria.rednoah.com"), req.getHost());
   CPPUNIT_ASSERT_EQUAL(string("/aria2"), req.getDir());
   CPPUNIT_ASSERT_EQUAL(string("aria3"), req.getFile());
@@ -175,7 +177,7 @@ void RequestTest::testSetUrl10() {
 void RequestTest::testSetUrl11() {
   Request req;
   bool v = req.setUrl("http://host?query/");
-  
+
   CPPUNIT_ASSERT(v);
   CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
   CPPUNIT_ASSERT_EQUAL(string("host"), req.getHost());
@@ -212,56 +214,94 @@ void RequestTest::testSetUrl14() {
   CPPUNIT_ASSERT(v);
   CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
   CPPUNIT_ASSERT_EQUAL(string("host"), req.getHost());
-  CPPUNIT_ASSERT_EQUAL(8080, req.getPort());
+  CPPUNIT_ASSERT_EQUAL((int32_t)8080, req.getPort());
   CPPUNIT_ASSERT_EQUAL(string("/"), req.getDir());
   CPPUNIT_ASSERT_EQUAL(string("abc?query"), req.getFile());
 }
 
+void RequestTest::testSetUrl15()
+{
+  Request req;
+  // 2 slashes after host name and dir
+  bool v = req.setUrl("http://host//dir1/dir2//file");
+  CPPUNIT_ASSERT(v);
+  CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
+  CPPUNIT_ASSERT_EQUAL(string("host"), req.getHost());
+  CPPUNIT_ASSERT_EQUAL(string("/dir1/dir2"), req.getDir());
+  CPPUNIT_ASSERT_EQUAL(string("file"), req.getFile());
+}
+
+void RequestTest::testSetUrl16()
+{
+  Request req;
+  // 2 slashes before file
+  bool v = req.setUrl("http://host//file");
+  CPPUNIT_ASSERT(v);
+  CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
+  CPPUNIT_ASSERT_EQUAL(string("host"), req.getHost());
+  CPPUNIT_ASSERT_EQUAL(string("/"), req.getDir());
+  CPPUNIT_ASSERT_EQUAL(string("file"), req.getFile());
+}
+
+void RequestTest::testSetUrl17()
+{
+  Request req;
+  bool v = req.setUrl("http://host:80/file<with%2 %20space/file with space;param?a=/?");
+  CPPUNIT_ASSERT(v);
+  CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
+  CPPUNIT_ASSERT_EQUAL(string("host"), req.getHost());
+  CPPUNIT_ASSERT_EQUAL(string("/file%3cwith%252%20%20space"), req.getDir());
+  CPPUNIT_ASSERT_EQUAL(string("file%20with%20space;param?a=/?"), req.getFile());
+}
+
 void RequestTest::testRedirectUrl() {
   Request req;
-  bool v = req.setUrl("http://aria.rednoah.com:8080/aria2/index.html");
+  req.setKeepAlive(true);
+  req.setUrl("http://aria.rednoah.com:8080/aria2/index.html");
   
   bool v2 = req.redirectUrl("http://aria.rednoah.co.jp/");
   CPPUNIT_ASSERT(v2);
+  // keep-alive set to be false after redirection
+  CPPUNIT_ASSERT(!req.isKeepAlive());
   // url must be the same
   CPPUNIT_ASSERT_EQUAL(string("http://aria.rednoah.com:8080/aria2/index.html"),
 		       req.getUrl());
   // currentUrl must be updated
   CPPUNIT_ASSERT_EQUAL(string("http://aria.rednoah.co.jp/"), req.getCurrentUrl());
-  // previousUrl must be updated
-  CPPUNIT_ASSERT_EQUAL(string("http://aria.rednoah.com:8080/aria2/index.html"), req.getPreviousUrl());
+  // previousUrl must be "" when redirection
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getPreviousUrl());
   CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
   CPPUNIT_ASSERT_EQUAL(string("aria.rednoah.co.jp"), req.getHost());
-  CPPUNIT_ASSERT_EQUAL(80, req.getPort());
+  CPPUNIT_ASSERT_EQUAL((int32_t)80, req.getPort());
   CPPUNIT_ASSERT_EQUAL(string("/"), req.getDir());
   CPPUNIT_ASSERT_EQUAL(string(""), req.getFile());
 }
 
 void RequestTest::testRedirectUrl2() {
   Request req;
-  bool v = req.setUrl("http://aria.rednoah.com/download.html");
+  req.setUrl("http://aria.rednoah.com/download.html");
   CPPUNIT_ASSERT_EQUAL(string(""), req.getPreviousUrl());
   req.setReferer("http://aria.rednoah.com/");
   // previousUrl is updated when referer is specified
   CPPUNIT_ASSERT_EQUAL(string("http://aria.rednoah.com/"), req.getPreviousUrl());
-  bool v2 = req.redirectUrl("http://aria.rednoah.com/403.html");
+  req.redirectUrl("http://aria.rednoah.com/403.html");
 
-  // previousUrl is updated
-  CPPUNIT_ASSERT_EQUAL(string("http://aria.rednoah.com/download.html"), req.getPreviousUrl());
+  // previousUrl must be "" when redirection
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getPreviousUrl());
   // referer is unchagned
   CPPUNIT_ASSERT_EQUAL(string("http://aria.rednoah.com/"), req.getReferer());
 
-  bool v3 = req.redirectUrl("http://aria.rednoah.com/error.html");
+  req.redirectUrl("http://aria.rednoah.com/error.html");
 
-  // previousUrl is update
-  CPPUNIT_ASSERT_EQUAL(string("http://aria.rednoah.com/403.html"), req.getPreviousUrl());
+  // previousUrl must be "" when redirection
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getPreviousUrl());
 }
   
 void RequestTest::testResetUrl() {
   Request req;
-  bool v = req.setUrl("http://aria.rednoah.com:8080/aria2/index.html");
+  req.setUrl("http://aria.rednoah.com:8080/aria2/index.html");
   req.setReferer("http://aria.rednoah.com:8080/");
-  bool v2 = req.redirectUrl("ftp://aria.rednoah.co.jp/");
+  req.redirectUrl("ftp://aria.rednoah.co.jp/");
 
   bool v3 = req.resetUrl();
   CPPUNIT_ASSERT(v3);
@@ -273,18 +313,12 @@ void RequestTest::testResetUrl() {
   // referer is unchanged
   CPPUNIT_ASSERT_EQUAL(string("http://aria.rednoah.com:8080/"), req.getReferer());
   CPPUNIT_ASSERT_EQUAL(string("http"), req.getProtocol());
-  CPPUNIT_ASSERT_EQUAL(8080, req.getPort());
+  CPPUNIT_ASSERT_EQUAL((int32_t)8080, req.getPort());
   CPPUNIT_ASSERT_EQUAL(string("aria.rednoah.com"), req.getHost());
   CPPUNIT_ASSERT_EQUAL(string("/aria2"), req.getDir());
   CPPUNIT_ASSERT_EQUAL(string("index.html"), req.getFile());
 }
 
-void RequestTest::testSafeChar() {
-  Request req;
-  bool v = req.setUrl("http://aria.rednoah.com/|<>");
-  CPPUNIT_ASSERT(!v);
-}
-
 void RequestTest::testInnerLink() {
   Request req;
   bool v = req.setUrl("http://aria.rednoah.com/index.html#download");
@@ -292,89 +326,66 @@ void RequestTest::testInnerLink() {
   CPPUNIT_ASSERT_EQUAL(string("index.html"), req.getFile());
 }
 
-void RequestTest::testMetalink() {
+void RequestTest::testSetUrl_zeroUsername()
+{
   Request req;
-  bool v = req.setUrl("http://aria.rednoah.com/download/aria.tar.bz2#!metalink3!http://aria2.sourceforge.net/download/aria.metalink");
-  CPPUNIT_ASSERT(v);
-  CPPUNIT_ASSERT_EQUAL(string("aria2.sourceforge.net"), req.getHost());
+  CPPUNIT_ASSERT(req.setUrl("ftp://@localhost/download/aria2-1.0.0.tar.bz2"));
+  CPPUNIT_ASSERT_EQUAL(string("ftp"), req.getProtocol());
+  CPPUNIT_ASSERT_EQUAL((int32_t)21, req.getPort());
+  CPPUNIT_ASSERT_EQUAL(string("localhost"), req.getHost());
   CPPUNIT_ASSERT_EQUAL(string("/download"), req.getDir());
-  CPPUNIT_ASSERT_EQUAL(string("aria.metalink"), req.getFile());
+  CPPUNIT_ASSERT_EQUAL(string("aria2-1.0.0.tar.bz2"), req.getFile());
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getUsername());
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getPassword());
+
+  CPPUNIT_ASSERT(req.setUrl("ftp://:@localhost/download/aria2-1.0.0.tar.bz2"));
+  CPPUNIT_ASSERT_EQUAL(string("ftp"), req.getProtocol());
+  CPPUNIT_ASSERT_EQUAL((int32_t)21, req.getPort());
+  CPPUNIT_ASSERT_EQUAL(string("localhost"), req.getHost());
+  CPPUNIT_ASSERT_EQUAL(string("/download"), req.getDir());
+  CPPUNIT_ASSERT_EQUAL(string("aria2-1.0.0.tar.bz2"), req.getFile());
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getUsername());
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getPassword());
+
+  CPPUNIT_ASSERT(req.setUrl("ftp://:pass@localhost/download/aria2-1.0.0.tar.bz2"));
+  CPPUNIT_ASSERT_EQUAL(string("ftp"), req.getProtocol());
+  CPPUNIT_ASSERT_EQUAL((int32_t)21, req.getPort());
+  CPPUNIT_ASSERT_EQUAL(string("localhost"), req.getHost());
+  CPPUNIT_ASSERT_EQUAL(string("/download"), req.getDir());
+  CPPUNIT_ASSERT_EQUAL(string("aria2-1.0.0.tar.bz2"), req.getFile());
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getUsername());
+  CPPUNIT_ASSERT_EQUAL(string("pass"), req.getPassword());
 
-  bool v2 = req.setUrl("http://aria.rednoah.com/download/aria.tar.bz2#!metalink3!");
-  CPPUNIT_ASSERT(!v2);
 }
 
-void RequestTest::testResolveHttpAuthConfig()
+void RequestTest::testSetUrl_username()
 {
   Request req;
-  req.setUrl("http://localhost/download/aria2-1.0.0.tar.bz2");
-  // with DefaultAuthResolver
-  DefaultAuthResolverHandle defaultAuthResolver = new DefaultAuthResolver();
-  req.setHttpAuthResolver(defaultAuthResolver);
-  CPPUNIT_ASSERT(!req.resolveHttpAuthConfig().isNull());
-  CPPUNIT_ASSERT_EQUAL(string(":"),
-		       req.resolveHttpAuthConfig()->getAuthText());
-
-  // with Netrc
-  NetrcHandle netrc = new Netrc();
-  netrc->addAuthenticator(new DefaultAuthenticator("default", "defaultpassword", "defaultaccount"));
-  NetrcAuthResolverHandle netrcAuthResolver = new NetrcAuthResolver();
-  netrcAuthResolver->setNetrc(netrc);
-  req.setHttpAuthResolver(netrcAuthResolver);
-  AuthConfigHandle authConfig1 = req.resolveHttpAuthConfig();
-  CPPUNIT_ASSERT(!authConfig1.isNull());
-  CPPUNIT_ASSERT_EQUAL(string("default:defaultpassword"),
-		       authConfig1->getAuthText());
-
-  // with Netrc + user defined
-  AuthConfigHandle authConfig =
-    new AuthConfig("userDefinedUser", "userDefinedPassword");
-  netrcAuthResolver->setUserDefinedAuthConfig(authConfig);
-  AuthConfigHandle authConfig2 = req.resolveHttpAuthConfig();
-  CPPUNIT_ASSERT(!authConfig2.isNull());
-  CPPUNIT_ASSERT_EQUAL(string("userDefinedUser:userDefinedPassword"),
-		       authConfig2->getAuthText());
+  CPPUNIT_ASSERT(req.setUrl("ftp://aria2user@localhost/download/aria2-1.0.0.tar.bz2"));
+  CPPUNIT_ASSERT_EQUAL(string("ftp"), req.getProtocol());
+  CPPUNIT_ASSERT_EQUAL((int32_t)21, req.getPort());
+  CPPUNIT_ASSERT_EQUAL(string("localhost"), req.getHost());
+  CPPUNIT_ASSERT_EQUAL(string("/download"), req.getDir());
+  CPPUNIT_ASSERT_EQUAL(string("aria2-1.0.0.tar.bz2"), req.getFile());
+  CPPUNIT_ASSERT_EQUAL(string("aria2user"), req.getUsername());
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getPassword());
 }
 
-void RequestTest::testResolveHttpAuthConfig_noCandidate()
+void RequestTest::testSetUrl_usernamePassword()
 {
   Request req;
-  req.setUrl("http://localhost/download/aria2-1.0.0.tar.bz2");
+  CPPUNIT_ASSERT(req.setUrl("ftp://aria2user%40:aria2pass%40@localhost/download/aria2-1.0.0.tar.bz2"));
+  CPPUNIT_ASSERT_EQUAL(string("ftp"), req.getProtocol());
+  CPPUNIT_ASSERT_EQUAL((int32_t)21, req.getPort());
+  CPPUNIT_ASSERT_EQUAL(string("localhost"), req.getHost());
+  CPPUNIT_ASSERT_EQUAL(string("/download"), req.getDir());
+  CPPUNIT_ASSERT_EQUAL(string("aria2-1.0.0.tar.bz2"), req.getFile());
+  CPPUNIT_ASSERT_EQUAL(string("aria2user@"), req.getUsername());
+  CPPUNIT_ASSERT_EQUAL(string("aria2pass@"), req.getPassword());
 
-  DefaultAuthResolverHandle defaultAuthResolver = new DefaultAuthResolver();
-  req.setHttpAuthResolver(defaultAuthResolver);
-  CPPUNIT_ASSERT_EQUAL(string(":"),
-		       req.resolveHttpAuthConfig()->getAuthText());
-}
+  // make sure that after new url is set, username and password are updated.
+  CPPUNIT_ASSERT(req.setUrl("ftp://localhost/download/aria2-1.0.0.tar.bz2"));
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getUsername());
+  CPPUNIT_ASSERT_EQUAL(string(""), req.getPassword());
 
-void RequestTest::testResolveHttpProxyAuthConfig()
-{
-  Request req;
-  req.setUrl("http://localhost/download/aria2-1.0.0.tar.bz2");
-  // with Netrc
-  NetrcHandle netrc = new Netrc();
-  netrc->addAuthenticator(new DefaultAuthenticator("default", "defaultpassword", "defaultaccount"));
-  NetrcAuthResolverHandle netrcAuthResolver = new NetrcAuthResolver();
-  netrcAuthResolver->setNetrc(netrc);
-  req.setHttpProxyAuthResolver(netrcAuthResolver);
-  AuthConfigHandle authConfig1 = req.resolveHttpProxyAuthConfig();
-  CPPUNIT_ASSERT(!authConfig1.isNull());
-  CPPUNIT_ASSERT_EQUAL(string("default:defaultpassword"),
-		       authConfig1->getAuthText());
-}
-
-void RequestTest::testResolveFtpAuthConfig()
-{
-  Request req;
-  req.setUrl("http://localhost/download/aria2-1.0.0.tar.bz2");
-  // with Netrc
-  NetrcHandle netrc = new Netrc();
-  netrc->addAuthenticator(new DefaultAuthenticator("default", "defaultpassword", "defaultaccount"));
-  NetrcAuthResolverHandle netrcAuthResolver = new NetrcAuthResolver();
-  netrcAuthResolver->setNetrc(netrc);
-  req.setFtpAuthResolver(netrcAuthResolver);
-  AuthConfigHandle authConfig1 = req.resolveFtpAuthConfig();
-  CPPUNIT_ASSERT(!authConfig1.isNull());
-  CPPUNIT_ASSERT_EQUAL(string("default:defaultpassword"),
-		       authConfig1->getAuthText());
 }
