@@ -1,6 +1,8 @@
 #include "TimeSeedCriteria.h"
+#include "Util.h"
 
 #include <cppunit/extensions/HelperMacros.h>
+#include <stdlib.h>
 
 class TimeSeedCriteriaTest:public CppUnit::TestFixture {
 
@@ -17,7 +19,7 @@ CPPUNIT_TEST_SUITE_REGISTRATION(TimeSeedCriteriaTest);
 
 void TimeSeedCriteriaTest::testEvaluate() {
   TimeSeedCriteria cri(1);
-  sleep(1);
+  Util::sleep(1);
   CPPUNIT_ASSERT(cri.evaluate());
   cri.reset();
   cri.setDuration(10);
